# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def check_tools_for_ome_calibration(paths_and_names, log_file_list):
    """ check_tools_for_ome_calibration checks if necessary switches in module configuration files are set to perform OME calibration.

        The input list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        The input list "log_file_list" contains all system prints of function: set_configuration_parameter.py.

        The output list "log_file_list" contains all system prints of functions: set_configuration_parameter.py and select_engine_from_existing_project.py.

        :param: paths_and_names: input list of lists
        :param: log_file_list: input list
        :return: log_file_list: output list of lists
    """

    ''' imports for python '''
    from datetime import datetime
    from inspect import currentframe, getframeinfo
    from sub_function.read_xml_file import read_xml_file

    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory_rce = \
        ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]

    ''' initialize local parameter '''
    calibrate_ome_flag = 1
    function_name = getframeinfo(currentframe()).function

    # try to open weight_and_balance_analysis module configuration file
    frame_info = getframeinfo(currentframe())
    path = path_of_working_directory_rce + current_workflow_name + '/fuselage_design_conf.xml'
    root_of_mass_estimation_tree, _ = read_xml_file(path, path_of_working_directory_rce, 'fuselage_design',
                                                           function_name, frame_info.lineno, log_file_list,
                                                           current_workflow_name)

    # read fuselage mass flag from weight_and_balance_analysis configuration file
    fuselage_mass_flag = root_of_mass_estimation_tree.find(
        './program_settings/configuration/fidelity/fuselage_design_tu_berlin/general/mass_technology_factors/'
        'fuselage_structural_mass_technology_factor/value')

    # check if fuselage mass flag from weight_and_balance_analysis configuration file is equal to zero
    #  -> if true: -> OME calibration is not possible -> raise an error and exit calibration mode.
    if fuselage_mass_flag is None:
        print('In fuselage_design configuration file, the fuselage mass technology factor node is missing. \n'
              'OME calibration is not possible. Exit calibration mode. ')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'In fuselage_design configuration file, the fuselage mass technology factor node is missing. \n'
                             '                    OME calibration is not possible. Exit calibration mode.')
        calibrate_ome_flag = 0

    return calibrate_ome_flag, log_file_list
