# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def final_ome_calibration_steps(paths_and_names):
    """ final_ome_calibration_steps stores the final ome calibration results to csv-file and saves th current aircraft exchange file.

        The input list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        :param: paths_and_names: input list of lists
        :return: none
    """

    ''' imports for python '''
    import os
    import shutil
    from inspect import currentframe, getframeinfo
    from sub_function.read_xml_file import read_xml_file
    from calibration.save_calibration_results_to_file import save_calibration_results_to_file

    ''' read data for script execution '''
    aircraft_exchange_file = ([item for item in paths_and_names if 'aircraft_exchange_file' in item])[-1][-1]
    aircraft_project = ([item for item in paths_and_names if 'aircraft_project' in item])[-1][-1]
    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory_rce = ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]

    ''' initialize local parameter '''
    log_file_list = []
    function_name = getframeinfo(currentframe()).function

    ''' save aircraft exchange file of final ome calibration loop to temporary results directory '''
    aircraft_exchange_files_of_ome_calibration = path_of_working_directory_rce + current_workflow_name \
        + '/temporaryResults/aircraft_exchange_files/aircraft_exchange_files_of_sizing_loop/OMEcalibration/'
    if not os.path.isdir(aircraft_exchange_files_of_ome_calibration):
        os.makedirs(aircraft_exchange_files_of_ome_calibration)

    list_of_exchange_elements = os.listdir(aircraft_exchange_files_of_ome_calibration)
    count_of_calibration = len(list_of_exchange_elements) + 1

    # save aircraft exchange file of final ome calibration loop
    project_file_name = path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project \
            + '/' + aircraft_exchange_file
    shutil.copy(project_file_name, aircraft_exchange_files_of_ome_calibration + aircraft_exchange_file[:-4]
                + '_it' + str(count_of_calibration) + '.xml')

    ''' estimate ome residual from pre last to final ome value '''
    # read final ome value from aircraft exchange file
    frame_info = getframeinfo(currentframe())
    path = path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project \
        + '/' + aircraft_exchange_file
    root_of_aircraft_exchange_tree, _ = read_xml_file(path, path_of_working_directory_rce,
                                                             aircraft_exchange_file, function_name, frame_info.lineno,
                                                             log_file_list, current_workflow_name)

    final_ome = float(root_of_aircraft_exchange_tree.find(
        './analysis/masses_cg_inertia/operating_mass_empty/mass_properties/mass/value').text)

    # read pre-final ome value from aircraft exchange file
    frame_info = getframeinfo(currentframe())
    aircraft_exchange_file = aircraft_exchange_file[:-4] + '_it' + str(count_of_calibration-1) + '.xml'
    path = aircraft_exchange_files_of_ome_calibration + aircraft_exchange_file
    root_of_aircraft_exchange_tree, _ = read_xml_file(path, path_of_working_directory_rce,
                                                             aircraft_exchange_file, function_name,
                                                             frame_info.lineno, log_file_list)

    pre_final_ome = float(root_of_aircraft_exchange_tree.find(
        './analysis/masses_cg_inertia/operating_mass_empty/mass_properties/mass/value').text)

    # estimate final ome residual
    residual_ome = abs(1 - pre_final_ome/final_ome)

    ''' save calibration results to csv file '''
    save_calibration_results_to_file(paths_and_names, count_of_calibration, residual_ome, 'OME')

    return
