# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def reset_free_calibration_variables(paths_and_names, calibration_settings, log_file_list):
    """ reset_free_calibration_variables resets free calibration variables in tool configuration files.

        The input list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        The input list of lists "calibration_settings" contains the following values:
            *  [0][0] string: 'free_variable_OME_calibration',                   [0][1] int:   'switch for free variable OME calibration'
            *  [1][0] string: 'free_variable_MTOM_calibration',                  [1][1] int:   'switch for free variable MTOM calibration'
            *  [2][0] string: 'calibration_target_OME',                          [2][1] float: 'target value for OME calibration'
            *  [3][0] string: 'calibration_target_mtom',                         [3][1] float: 'target value for MTOM calibration'
            *  [4][0] string: 'reset_fuselage_mass_at_start',                    [4][1] int:   'switch to reset free OME calibration values at start'
            *  [5][0] string: 'reset_viscous_drag_counts_at_start',              [5][1] int:   'switch to reset aerodynamic MTOM calibration value at start'
            *  [6][0] string: 'reset_engine_fuel_flow_factor_at_start',          [6][1] int:   'switch to reset engine MTOM calibration value at start'
            *  [7][0] string: 'number_of_iterations_before_mtom_calibration',    [7][1] int:   'maximum number of OME calibration steps before performing a MTOM calibration'
            *  [8][0] string: 'max_iterations_before_exit_OME_calibration',      [8][1] int:   'maximum number of OME calibration steps before exit the calibration'
            *  [9][0] string: 'trunc_Error_factor',                              [9][1] int:   'value for termination criterion -> as factor regarding convergence criteria'
            * [10][0] string: 'damp_MTOM_calibration_lever',                    [10][1] int:   'switch for damping MTOM calibration lever arm'
            * [11][0] string: 'x_rel_max_for_MTOM_damp',                        [11][1] float: 'value of relative MTOM calibration lever arm'
            * [12][0] string: 'damp_OME_calibration_lever',                     [12][1] int:   'Switch for damping OME calibration lever arm'
            * [13][0] string: 'x_rel_max_for_OME_damp',                         [13][1] float: 'value of relative OME calibration lever arm'

        The input list "log_file_list" contains all system prints of function: set_configuration_parameter.py.

        The output list "log_file_list" contains all system prints of functions: set_configuration_parameter.py and select_engine_from_existing_project.py.

        :param: paths_and_names: input list of lists
        :param: calibration_settings: input list of lists
        :param: log_file_list: input list
        :return: log_file_list: output list of lists
    """

    ''' imports for python '''
    import sys
    from inspect import currentframe, getframeinfo
    from datetime import datetime
    from sub_function.read_xml_file import read_xml_file
    from sub_function.write_to_log_file import write_to_log_file

    ''' read data for script execution '''
    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory_rce = \
        ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]

    free_variable_ome_calibration = \
        int(([item for item in calibration_settings if 'free_variable_OME_calibration' in item])[-1][-1])
    free_variable_mtom_calibration = \
        int(([item for item in calibration_settings if 'free_variable_MTOM_calibration' in item])[-1][-1])
    reset_fuselage_mass_at_start = \
        int(([item for item in calibration_settings if 'reset_fuselage_mass_at_start' in item])[-1][-1])
    reset_viscous_drag_counts_at_start = \
        int(([item for item in calibration_settings if 'reset_viscous_drag_counts_at_start' in item])[-1][-1])
    reset_engine_fuel_flow_factor_at_start = \
        int(([item for item in calibration_settings if 'reset_engine_fuel_flow_factor_at_start' in item])[-1][-1])

    function_name = getframeinfo(currentframe()).function

    ''' reset free variables for calibration mode '''
    # reset free OME calibration variables
    if not free_variable_ome_calibration == 0 and reset_fuselage_mass_at_start:
        # try to open weight_and_balance_analysis module configuration file
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/fuselage_design_conf.xml'
        root_of_mass_estimation_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce, 
                                                               'fuselage_design',
                                                               function_name, frame_info.lineno, log_file_list,
                                                               current_workflow_name)

        # reset fuselage scaling factor in weight_and_balance_analysis configuration file to 1
        root_of_mass_estimation_tree.find(
            './program_settings/configuration/fidelity/fuselage_design_tu_berlin/'
            'general/mass_technology_factors/fuselage_structural_mass_technology_factor/value').text = str(1.0)

        # write settings to configuration xml file
        xml_tree.write(path, encoding='utf-8')
        print('For OME calibration fuselage scaling factor in fuselage design configuration file reset to 1.')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'For OME calibration fuselage scaling factor in fuselage design configuration file '
                             'reset to 1.')

    # reset free MTOM calibration variables if mtom calibration is active
    if not free_variable_mtom_calibration == 0:
        # case 1 -> Engine efficiency calibration mode
        if reset_engine_fuel_flow_factor_at_start:
            # try to open propulsion_design module configuration file
            frame_info = getframeinfo(currentframe())
            path = path_of_working_directory_rce + current_workflow_name + '/propulsion_design_conf.xml'
            root_of_engine_sizing_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce,
                                                                 'propulsion_design', function_name, frame_info.lineno,
                                                                 log_file_list, current_workflow_name)

            # reset Engine efficiency scaling factor in propulsion_design configuration file to 1
            root_of_engine_sizing_tree.find(
                './program_settings/technology_factors/engine_efficiency/value').text = str(1.0)

            # write settings to configuration xml file
            xml_tree.write(path, encoding='utf-8')
            print('For MTOM calibration, engine efficiency factor in propulsion_design configuration file reset to 1.')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'For MTOM calibration, engine efficiency scaling factor in propulsion_design '
                                 'configuration file reset to 1.')

        # case 2 -> drag count reduction or Engine efficiency calibration mode and drag count reduction
        if reset_viscous_drag_counts_at_start:
            # try to open aerodynamic_assessment module configuration file
            frame_info = getframeinfo(currentframe())
            path = path_of_working_directory_rce + current_workflow_name + '/aerodynamic_analysis_conf.xml'
            root_of_calculate_polar_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce,
                                                                   'aerodynamic_analysis', function_name,
                                                                   frame_info.lineno, log_file_list,
                                                                   current_workflow_name)

            # reset calibration drag count reduction value in aerodynamic_assessment configuration file to 0
            root_of_calculate_polar_tree.find(
                './program_settings/ViscDragRaymer/DoDragCountCorrection/value').text = str(bool(1)).lower()
            root_of_calculate_polar_tree.find(
                './program_settings/ViscDragRaymer/DeltaViscousDragWing/value').text = str(0)

            # write settings to configuration xml file
            xml_tree.write(path, encoding='utf-8')
            print('For MTOM calibration, drag count reduction factor in aerodynamic_analysis configuration file '
                  'reset to 0.')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'For MTOM calibration, drag count reduction factor in aerodynamic_analysis '
                                 'configuration file reset to 0.')

    return log_file_list
