# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def clean_up_temporary_results(paths_and_names, parameter_for_design_case, control_settings, identifier, log_file_list):
    """ post_operations_of_workflow creates all output files and copies them to the result directory.
        Deletes all temporary files.

        The input list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        The input list of lists "parameter_for_design_case" contains the following values:
            *  [0][0] string: 'program_mode',                     [0][1] string: 'Value of program execution mode - either standard_aircraft_design, or parameter_study, or optimization'
            *  [1][0] string: 'design_mode',                      [1][1] int:   'value to check which design mode is selected'
            *  [2][0] string: 'automatic_trim',                   [2][1] int:   'value to switch off and on the automatic trim function of the workflow'
            *  [3][0] string: 'convergence_criteria',             [3][1] float: 'value of convergence criteria of ome and mtom iteration'
            *  [4][0] string: 'number_of_max_iteration',          [4][1] int:   'value to set maximum number of design sizing iteration steps'
            *  [5][0] string: 'bridge_constraint_analyzer',       [5][1] int: 'value to switch off and on the execution of constraint_analysis module in workflow for each iteration step'
            *  [6][0] string: 'damp_mtom_iteration'               [6][1] bool:  'switch to activate damping of mtom iteration loop'
            *  [7][0] string: 'parallel_execution'                [7][1] int:   'switch to activate the parallel execution mode'
            *  [8][0] string: 'count_of_iteration',               [8][1] int:   'current number of iteration of design sizing loop'
            *  [9][0] string: 'count_of_mission_study_loop',      [9][1] int:   'current number of iteration of mission study loop'
            * [10][0] string: 'parameter_name',                  [10][1] string: 'name of the current input parameter if parameter study will be performed'
            * [11][0] string: 'global_loop_counter',             [11][1] int:    'global loop counter for parameter study mode'
            * [12][0] string: 'study_counter',                   [12][1] int:    'study counter of current input parameter if parameter study will be performed'
            * [13][0] string: 'new_value_of_input',              [13][1] float:  'value of current parameter study input parameter'

        The input list of lists "control_settings" contains the following values:
            *  [0][0] string: 'delete_plots_before_run',                              [0][1] int: 'switch to delete old plots before running'
            *  [1][0] string: 'delete_engines_before_run',                            [1][1] int: 'switch to delete old engine data before running'
            *  [2][0] string: 'clear_old_log_files_at_start',                         [2][1] int: 'switch to delete old log-files before running'
            *  [3][0] string: 'plot_output_on',                                       [3][1] int: 'switch to enable or disable plot generation'
            *  [4][0] string: 'switch_off_plots',                                     [4][1] int: 'switch to enable generation of plots of each iteration step'
            *  [5][0] string: 'report_output_on'                                      [5][1] int: 'switch to enable or disable report generation'
            *  [6][0] string: 'switch_off_reports',                                   [6][1] int: 'switch to enable generation of reports of each iteration step'
            *  [7][0] string: 'tex_report_on'                                         [7][1] int: 'switch to enable or disable tex-report generation'
            *  [8][0] string: 'switch_off_tex',                                       [8][1] int: 'switch to enable generation of tex-reports of each iteration step'
            *  [9][0] string: 'status_flag_to_skip_design_sizing'                     [9][1] int: 'status flag for plot generation of design sizing loop - iniitalized with 0'
            * [10][0] string: 'status_flag_to_skip_study_mission',                   [10][1] int: 'status flag for plot generation of study mission loop - iniitalized with 0'
            * [11][0] string: 'check_settings_for_design_logic',                     [11][1] int: 'value to switch off and on the checks of the design logic'
            * [12][0] string: 'use_control_settings_for_sub_programs',               [12][1] int: 'value for switching off and on the override control settings of subroutines'
            * [13][0] string: 'use_range_type_specific_factors',                     [13][1] int: 'value for switch of range scenario to set range dependent parameters'
            * [14][0] string: 'use_configs_from_existing_project',                   [14][1] int: 'value to activate copy of configuration files from existing project before running'
            * [15][0] string: 'use_engine_from_existing_project',                    [15][1] int: 'value to activate copy of engine data from existing project before running'
            * [16][0] string: 'save_configs_to_result_folder',                       [16][1] int: 'value to switch off and on the saving of configuration files in the result folder'
            * [17][0] string: 'save_log_files_to_result_folder',                     [17][1] int: 'value to switch off and on the saving of log files in the result folder'
            * [18][0] string: 'save_acft_xml_to_result_folder',                      [18][1] int: 'value to switch off and on the saving of aircraft exchange file in the result folder'
            * [19][0] string: 'save_acft_xml_after_each_iteration',                  [19][1] int: 'value to switch off and on the saving of aircraft exchange file in the result folder after each step of iteration'
            * [20][0] string: 'write_info_files',                                    [20][1] int: 'switch to activate the writing of info files after each tool execution'
            * [21][0] string: 'write_info_files_only_at_last_step_of_iteration',     [21][1] int: 'switch to activate the writing of info files only after last step of sizing loop'
            * [22][0] string: 'program_mode',                                        [22][1] string: 'Value of program execution mode - either standard_aircraft_design, or parameter_study, or optimization'

        The input integer "identifier" contains the identifier number of the current workflow execution.

        The input list "log_file_list" contains all system prints of function: post_operations_of_workflow.py.

        :param: paths_and_names: input list of lists
        :param: parameter_for_design_case: input list of lists
        :param: control_settings: input list of lists
        :param: identifier: input integer
        :param: log_file_list: input list
        :return: none
    """

    ''' imports for python '''
    import os
    import shutil
    from datetime import datetime
    from parameter_study.read_parameter_study_settings import read_parameter_study_settings
    from parameter_study.write_parameter_study_csv_output import write_parameter_study_csv_output
    from sub_function.write_to_log_file import write_to_log_file

    ''' read data for script execution '''
    aircraft_project = ([item for item in paths_and_names if 'aircraft_project' in item])[-1][-1]
    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory = \
        ([item for item in paths_and_names if 'path_of_working_directory' in item])[-1][-1]
    path_of_working_directory_rce = \
        ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]

    global_loop_counter = int(([item for item in parameter_for_design_case if 'global_loop_counter' in item])[-1][-1])
    parameter_name = str(([item for item in parameter_for_design_case if 'parameter_name' in item])[-1][-1])
    parameter_study_mode = (([item for item in parameter_for_design_case if 'program_mode' in item])[-1][-1]
                                == "parameter_study")
    optimization_mode = (([item for item in parameter_for_design_case if 'program_mode' in item])[-1][-1]
                                == "optimization")
    study_counter = int(([item for item in parameter_for_design_case if 'study_counter' in item])[-1][-1])

    ''' initialize local parameters '''
    save_results = int(0)
    copy_target_path = str()
    skip_saving_flag = False
    save_only_aircraft_exchange_file = int(0)

    # Check if any design tool has sent an execution error
    #  -> if true: -> add "_Failed_execution" string to result directory of current workflow loop
    if os.path.isfile(path_of_working_directory_rce + current_workflow_name + '/temp/workflowExecutionError.dat'):
        current_workflow_name_new = current_workflow_name + '_Failed_execution'
    else:
        current_workflow_name_new = current_workflow_name

    ''' perform copy and clean up '''
    # check if the parameter study should not be performed -> if true: -> set copy target path to workflow results
    if parameter_study_mode == 0 and optimization_mode == 0:
        copy_target_path = path_of_working_directory + 'workflowResults/' + current_workflow_name_new + '_' \
            + aircraft_project

    # else if condition: parameter study should be performed -> set copy target path to parameter study working dir
    elif parameter_study_mode == 1:
        # call function to read parameter study settings
        parameter_study_list, log_file_list = read_parameter_study_settings(path_of_working_directory, log_file_list,
                                                                            current_workflow_name)

        path_to_current_study = \
            str(([item for item in parameter_study_list if 'path_to_current_study' in item])[-1][-1])
        reproduce_output_file_from_old_study = \
            int(([item for item in parameter_study_list if 'reproduce_output_file_from_old_study' in item])[-1][-1])
        save_results = \
            int(([item for item in parameter_study_list if 'save_results' in item])[-1][-1])
        save_only_aircraft_exchange_file =\
            int(([item for item in parameter_study_list if 'save_only_aircraft_exchange_file' in item])[-1][-1])

        # reference case
        if global_loop_counter == 1:
            # set target path to copy results of reference case
            copy_target_path = path_to_current_study + '/reference_aircraft/results'
            if reproduce_output_file_from_old_study == 1:
                skip_saving_flag = True

        # parameter study iteration
        else:
            copy_target_path = path_to_current_study + '/' + parameter_name + '/' + 'run_' + str(study_counter)
            if study_counter == 0:
                if os.path.isdir(path_to_current_study + '/' + parameter_name):
                    last_number = len(os.listdir(path_to_current_study + '/' + parameter_name))
                else:
                    last_number = int(0)
                copy_target_path = path_to_current_study + '/' + parameter_name + '/' + 'run_' + str(last_number+1)

    # else if condition: optimization should be performed -> set copy target path for results of optimization
    elif optimization_mode == 1:
        # generate directory for optimization results and save outputs of first workflow loop as reference case
        if global_loop_counter == 1:
            copy_target_path = path_of_working_directory + 'workflowResults/' + current_workflow_name_new + '_' \
                               + aircraft_project + '_optimization/referenceCase'
            if not os.path.isdir(path_of_working_directory + 'workflowResults/' + current_workflow_name_new + '_'
                                 + aircraft_project + '_optimization/results'):
                os.makedirs(path_of_working_directory + 'workflowResults/' + current_workflow_name_new + '_'
                            + aircraft_project + '_optimization/results')
                with open(path_of_working_directory + 'workflowResults/' + current_workflow_name_new + '_'
                          + aircraft_project + '_optimization/results/X-values.csv', 'w') as file:
                    file.close()
                with open(path_of_working_directory + 'workflowResults/' + current_workflow_name_new + '_'
                          + aircraft_project + '_optimization/results/Y-values.csv', 'w') as file:
                    file.close()
                with open(path_of_working_directory + 'workflowResults/' + current_workflow_name_new + '_'
                          + aircraft_project + '_optimization/results/G-values.csv', 'w') as file:
                    file.close()

        else:
            copy_target_path = path_of_working_directory + 'workflowResults/' + current_workflow_name_new + '_' \
                               + aircraft_project + '_optimization/run_' + str(global_loop_counter - 1)

    ''' if existing, copy and then delete temporary results directory '''
    if parameter_study_mode == 0 or (save_results == 1 and save_only_aircraft_exchange_file == 0) \
            and not skip_saving_flag:
        # copy temporary results to workflowResults and delete working copy of current workflow
        if os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/temporaryResults'):
            # copy temporary workflow results to workflow results directory
            shutil.copytree(path_of_working_directory_rce + current_workflow_name + '/temporaryResults',
                            copy_target_path)
            print('Workflow results successfully saved!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Workflow results successfully stored!')
            # delete temporary workflow files
            shutil.rmtree(path_of_working_directory_rce + current_workflow_name + '/temporaryResults')
            print('Directory for temporary results cleared!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Directory for temporary results cleared!')

            ''' copy workflow configuration file to log-file results '''
            shutil.copy(path_of_working_directory_rce + current_workflow_name + '/unicado_workflow_conf.xml',
                        copy_target_path + '/config_files/unicado_workflow_conf.xml')
            print('UNICADOworkflow configuration file has been successfully copied to results folder!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'UNICADOworkflow configuration file has been successfully copied to results folder!')

    elif save_results == 1 and save_only_aircraft_exchange_file == 1 and not skip_saving_flag:
        if os.path.isdir(path_of_working_directory_rce + current_workflow_name
                         + '/temporaryResults/aircraft_exchange_files'):
            shutil.copytree(path_of_working_directory_rce + current_workflow_name
                            + '/temporaryResults/aircraft_exchange_files', copy_target_path)
            print('Only aircraft exchange files of current parameter study input successfully stored!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Only aircraft exchange files of current parameter study input successfully stored!')

    if optimization_mode == 1:
        ''' copy optimization configuration file to config files '''
        shutil.copy(path_of_working_directory_rce + current_workflow_name + '/optimization_conf.xml',
                    copy_target_path + '/config_files/optimization_conf.xml')
        print('Optimization configuration file has been successfully copied to results folder!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                             + ': Pptimization configuration file has been successfully copied to results folder!')

    ''' write csv-output file if parameter study is selected '''
    if parameter_study_mode == 1:
        ''' copy parameter study configuration file to config files '''
        shutil.copy(path_of_working_directory_rce + current_workflow_name + '/parameter_study_conf.xml',
                    copy_target_path + '/config_files/parameter_study_conf.xml')
        print('Parameter study configuration file has been successfully copied to results folder!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                             + ': Parameter study configuration file has been successfully copied to results folder!')

        # delete temporary workflow files
        if os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/temporaryResults'):
            shutil.rmtree(path_of_working_directory_rce + current_workflow_name + '/temporaryResults')
            print('Directory for temporary results cleared!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                                 + ': Directory for temporary results cleared!')

        # call function to write parameter study output data to output csv-file
        if not skip_saving_flag:
            status_csv_output, log_file_list = write_parameter_study_csv_output(path_of_working_directory_rce,
                                                                                path_of_working_directory,
                                                                                copy_target_path,
                                                                                parameter_for_design_case,
                                                                                log_file_list, current_workflow_name)

    ''' copy tool error list log-file to log-file results '''
    if not skip_saving_flag:
        if os.path.isfile(path_of_working_directory_rce + current_workflow_name + '/temp/toolErrorList.log'):
            if not os.path.isdir(copy_target_path + '/reporting/'):
                os.mkdir(copy_target_path + '/reporting/')
            shutil.copy(path_of_working_directory_rce + current_workflow_name + '/temp/toolErrorList.log',
                        copy_target_path + '/reporting/toolErrorList.log')
            print('UNICADOworkflow tool error list has been successfully copied to reporting folder of '
                  'workflow results!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'UNICADOworkflow tool error list has been successfully copied to reporting folder of '
                                 'workflow results!')

    ''' generate warning if convergence of design sizing loop has not reached '''
    if os.path.isfile(path_of_working_directory_rce + current_workflow_name + '/temp/convergenceError.dat'):
        print('Warning!')
        print('Maximum number of iterations of design sizing loop reached, but no convergence achieved!')
        print('Please check the maximum number of iterations and the convergence criterion in the '
              'UNICADOworkflow configuration file!')
        log_file_list.append('Warning!')
        log_file_list.append('Maximum number of iterations of design sizing loop reached, but no convergence achieved!')
        log_file_list.append('Please check the maximum number of iterations and the convergence criterion in the '
                             'UNICADOworkflow configuration file!')

    ''' write log-file to system '''
    log_file_list.append('****************************************** end post operations of workflow '
                         '******************************************')
    log_file_list.append('')
    log_file_list.append('********************************************** END OF UNICADO WORKFLOW '
                         '**********************************************')
    write_to_log_file(path_of_working_directory_rce, current_workflow_name, log_file_list)

    ''' copy workflow log-file to log-file results '''
    if not os.path.isdir(copy_target_path + '/reporting/log_files'):
        os.makedirs(copy_target_path + '/reporting/log_files')
    shutil.copy(path_of_working_directory_rce + current_workflow_name + '/unicado_workflow.log',
                copy_target_path + '/reporting/log_files/unicado_workflow.log')

    ''' delete identifier file of current workflow execution '''
    # check if the parameter study mode and the optimization mode is not selected
    #  -> if true: -> delete identifier file of current running workflow execution
    if parameter_study_mode == 0 and optimization_mode == 0:
        # remove identifier file of current workflow execution to prepare shut down after workflow termination
        os.remove(path_of_working_directory_rce + 'identifier/' + str(identifier) + '.dat')
        if len(os.listdir(path_of_working_directory_rce + 'identifier/')) == 0:
            os.rmdir(path_of_working_directory_rce + 'identifier/')

        # delete temporary workflow directory of current workflow execution
        shutil.rmtree(path_of_working_directory_rce + current_workflow_name)

    return
