# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def estimate_design_sizing_residuals(paths_and_names, design_variables, mtom_loop_vector, number_of_max_iteration,
                                     count_of_iteration, convergence_criteria):
    """ estimate_mission_study_residuals calculates the residuals of design variables of design sizing loop

        The input list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        The input list of lists "design_variables" contains the following values:
            *  [0][0] string: 'm_take_off_max_last',                     [0][1] float: 'value of maximum take off mass of last iteration step'
            *  [1][0] string: 'm_operating_empty_last',                  [1][1] float: 'value of operating empty mass of last iteration step'
            *  [2][0] string: 'mission_energy_last',                     [2][1] float: 'value of mission energy of last iteration step'
            *  [3][0] string: 'x_center_of_gravity_last',                [3][1] float: 'value of x position of center of gravity of last iteration step'
            *  [4][0] string: 'cm_cruise',                               [4][1] float: 'value of the current moment coefficient'
            *  [5][0] string: 'i_stab',                                  [5][1] float: 'value of the current trimming angle of horizontal stabiliser'

        The input list "mtom_loop_vector" contains all estimated mtom values.

        The input int "number_of_max_iteration" contains the current maximum number of iteration steps subtracted by the value of performed iterations.

        The input int "count_of_iteration" contains the current number of iteration loop.

        The input float "convergence_criteria" contains the convergence criteria of current iteration process.

        The output list of lists "design_variables" contains the following values:
            *  [0][0] string: 'm_take_off_max_last',                     [0][1] float: 'value of maximum take off mass of last iteration step'
            *  [1][0] string: 'm_operating_empty_last',                  [1][1] float: 'value of operating empty mass of last iteration step'
            *  [2][0] string: 'mission_energy_last',                     [2][1] float: 'value of mission energy of last iteration step'
            *  [3][0] string: 'x_center_of_gravity_last',                [3][1] float: 'value of x position of center of gravity of last iteration step'
            *  [4][0] string: 'cm_cruise',                               [4][1] float: 'value of the current moment coefficient'
            *  [5][0] string: 'i_stab',                                  [5][1] float: 'value of the current trimming angle of horizontal stabiliser'
            *  [6][0] string: 'm_take_off_max',                          [6][1] float: 'value of maximum take off mass of current iteration step'
            *  [7][0] string: 'm_operating_empty',                       [7][1] float: 'value of operating empty mass of current iteration step'
            *  [8][0] string: 'mission_energy',                          [8][1] float: 'value of mission energy of current iteration step'
            *  [9][0] string: 'x_center_of_gravity',                     [9][1] float: 'value of x position of center of gravity of current iteration step'
            * [10][0] string: 'residual_m_take_off_max',                [10][1] float: 'value of residual of maximum take off mass of current iteration step'
            * [11][0] string: 'residual_m_operating_empty',             [11][1] float: 'value of residual of operating empty mass of current iteration step'
            * [12][0] string: 'residual_mission_energy',                [12][1] float: 'value of residual of mission energy of current iteration step'
            * [13][0] string: 'residual_x_center_of_gravity',           [13][1] float: 'value of residual of x position of center of gravity of current iteration step'
            * [14][0] string: 'status_residual_m_take_off_max',         [14][1] bool:  'status of convergence for maximum take of mass'
            * [15][0] string: 'status_residual_m_operating_empty',      [15][1] bool:  'status of convergence for mass operating empty'
            * [16][0] string: 'status_residual_mission_energy',         [16][1] bool:  'status of convergence for mission energy'
            * [17][0] string: 'status_residual_x_center_of_gravity',    [17][1] bool:  'status of convergence for the aft x-position of the center of gravity'

        The output list "mtom_loop_vector" contains all estimated mtom values.

        :param: paths_and_names: input list of lists
        :param: design_variables: input list of lists
        :param: mtom_loop_vector: input list
        :param: number_of_max_iteration: input int
        :param: count_of_iteration: input int
        :param: convergence_criteria: input float
        :return: design_variables: output list of lists, mtom_loop_vector: output list
    """

    ''' import for python '''
    from datetime import datetime
    from inspect import currentframe, getframeinfo
    from sub_function.read_xml_file import read_xml_file
    from sub_function.get_mission_energy_specific_segment import get_mission_energy_specific_segment

    ''' read data for script execution '''
    aircraft_exchange_file = ([item for item in paths_and_names if 'aircraft_exchange_file' in item])[-1][-1]
    aircraft_project = ([item for item in paths_and_names if 'aircraft_project' in item])[-1][-1]
    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory_rce = \
        ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]

    m_take_off_max_last = ([item for item in design_variables if 'm_take_off_max_last' in item])[-1][-1]
    m_operating_empty_last = ([item for item in design_variables if 'm_operating_empty_last' in item])[-1][-1]
    mission_energy_last = ([item for item in design_variables if 'mission_energy_last' in item])[-1][-1]
    x_center_of_gravity_last = ([item for item in design_variables if 'x_center_of_gravity_last' in item])[-1][-1]

    ''' initialize local parameter '''
    # date_time_counter is the number of characters of date and time print of workflow logfile
    #  -> should not be printed in the workflow console
    character_length_of_date_and_time = 21
    valid_data = True
    log_file_list = ['', '------------------------------ Current loop of iteration: ' + str(count_of_iteration)
                     + ' ---------',
                     '{0}: * results:'.format(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')))]
    status_residual_mission_energy = False
    status_residual_m_take_off_max = False
    status_residual_m_operating_empty = False
    status_residual_x_center_of_gravity = False
    function_name = getframeinfo(currentframe()).function

    ''' open and read aircraft exchange file '''
    frame_info = getframeinfo(currentframe())
    path = path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/' + aircraft_exchange_file
    root_of_aircraft_exchange_tree, _ = read_xml_file(path, path_of_working_directory_rce,
                                                             aircraft_exchange_file, function_name, frame_info.lineno,
                                                             log_file_list, current_workflow_name)

    # read iteration variables
    m_take_off_max = root_of_aircraft_exchange_tree.find(
        "./analysis/masses_cg_inertia/maximum_takeoff_mass/mass_properties/mass/value")
    if m_take_off_max is not None:
        m_take_off_max = float(m_take_off_max.text)
    else:
        m_take_off_max = 0

    m_operating_empty = root_of_aircraft_exchange_tree.find(
        "./analysis/masses_cg_inertia/operating_mass_empty/mass_properties/mass/value")
    if m_operating_empty is not None:
        m_operating_empty = float(m_operating_empty.text)
    else:
        m_operating_empty = 0

    mission_energy = get_mission_energy_specific_segment(
        root_of_aircraft_exchange_tree, 'design_mission', 'loaded_mission_energy/mission_energy')

    x_center_of_gravity = root_of_aircraft_exchange_tree.find(
        "./analysis/masses_cg_inertia/maximum_takeoff_mass/mass_properties/center_of_gravity/x/value")
    if x_center_of_gravity is not None:
        x_center_of_gravity = float(x_center_of_gravity.text)
    else:
        x_center_of_gravity = 0
    # save current mtom value to mtom loop vector
    mtom_loop_vector.append(m_take_off_max)

    # rel. residual m_take_off_max
    if m_take_off_max > 0:
        residual_m_take_off_max = round(abs(1 - m_take_off_max_last / m_take_off_max), 6)
        if residual_m_take_off_max < convergence_criteria:
            status_residual_m_take_off_max = True
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   Current m_take_off_max: '
                             + str(m_take_off_max) + '; Res. m_take_off_max: ' + str(residual_m_take_off_max)
                             + '; Convergence: ' + str(status_residual_m_take_off_max))
    else:
        residual_m_take_off_max = 1
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   Current m_take_off_max: '
                             + str(m_take_off_max) + '; Res. m_take_off_max: ' + str(residual_m_take_off_max)
                             + '; Convergence: ' + str(status_residual_m_take_off_max))

    # rel. residual m_operating_empty
    if m_operating_empty > 0:
        residual_m_operating_empty = round(abs(1 - m_operating_empty_last / m_operating_empty), 6)
        if residual_m_operating_empty < convergence_criteria:
            status_residual_m_operating_empty = True
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current m_operating_empty: ' + str(m_operating_empty) + '; Res. m_operating_empty: '
                             + str(residual_m_operating_empty) + '; Convergence: '
                             + str(status_residual_m_operating_empty))

    else:
        residual_m_operating_empty = 1
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current m_operating_empty: ' + str(m_operating_empty) + '; Res. m_operating_empty: '
                             + str(residual_m_operating_empty) + '; Convergence: '
                             + str(status_residual_m_operating_empty))

    # rel. residual x_center_of_gravity
    if x_center_of_gravity > 0:
        residual_x_center_of_gravity = round(abs(1 - x_center_of_gravity_last / x_center_of_gravity), 6)
        if residual_x_center_of_gravity < convergence_criteria:
            status_residual_x_center_of_gravity = True
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   Current x_center_of_gravity: '
                             + str(x_center_of_gravity) + '; Res. x_center_of_gravity: '
                             + str(residual_x_center_of_gravity) + '; Convergence: '
                             + str(status_residual_x_center_of_gravity))
    else:
        residual_x_center_of_gravity = 1
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current xCoG: ' + str(x_center_of_gravity) + '; Res. xCoG: '
                             + str(residual_x_center_of_gravity) + '; Convergence: '
                             + str(status_residual_x_center_of_gravity))

    # rel. residual mission_energy
    if mission_energy > 0:
        residual_mission_energy = round(abs(1 - mission_energy_last / mission_energy), 6)
        if residual_mission_energy < convergence_criteria:
            status_residual_mission_energy = True
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current mission_energy: ' + str(mission_energy) + '; Res. mission_energy: '
                             + str(residual_mission_energy) + '; Convergence: '
                             + str(status_residual_mission_energy))
    else:
        residual_mission_energy = 1
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current mission_energy: ' + str(mission_energy) + '; Res. mission_energy: '
                             + str(residual_mission_energy) + '; Convergence: ' + str(status_residual_mission_energy))

    ''' check for maximum iteration reached '''
    if number_of_max_iteration <= 0:
        mission_energy = mission_energy_last
        m_take_off_max = m_take_off_max_last
        m_operating_empty = m_operating_empty_last
        x_center_of_gravity = x_center_of_gravity_last
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'WARNING: Number of maximum iteration steps reached!')
        valid_data = False

    ''' write log-file to system '''
    log_file = open(path_of_working_directory_rce + current_workflow_name + '/unicado_workflow.log', 'a')
    # loop across all elements of log-file list to print to workflow console and write to workflow log-file
    for row in log_file_list:
        # print current log-file list entry to workflow console
        print(row[character_length_of_date_and_time:])

        # write current entry of log-file list to workflow log-file
        log_file.write(row + '\n')
    log_file.close()

    ''' append calculated design values to list of list "design_variables" '''
    design_variables.append(['m_take_off_max', m_take_off_max])
    design_variables.append(['m_operating_empty', m_operating_empty])
    design_variables.append(['mission_energy', mission_energy])
    design_variables.append(['x_center_of_gravity', x_center_of_gravity])
    design_variables.append(['residual_m_take_off_max', residual_m_take_off_max])
    design_variables.append(['residual_m_operating_empty', residual_m_operating_empty])
    design_variables.append(['residual_mission_energy', residual_mission_energy])
    design_variables.append(['residual_x_center_of_gravity', residual_x_center_of_gravity])
    design_variables.append(['status_residual_m_take_off_max', status_residual_m_take_off_max])
    design_variables.append(['status_residual_m_operating_empty', status_residual_m_operating_empty])
    design_variables.append(['status_residual_mission_energy', status_residual_mission_energy])
    design_variables.append(['status_residual_x_center_of_gravity', status_residual_x_center_of_gravity])

    return design_variables, mtom_loop_vector, valid_data
