# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def estimate_mission_study_residuals(paths_and_names, mission_study_variables, count_of_mission_study_loop,
                                     convergence_criteria):
    """ estimate_mission_study_residuals calculates the residuals of design variables of mission study loop

        The input list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        The input list of lists "mission_study_variables" contains the following values:
            * [0][0] string: 'm_take_off_max_study_last',       [0][1] float: 'value of maximum take of mass of last mission study loop'
            * [1][0] string: 'mission_energy_study_last',       [1][1] float: 'value of mission energy of last mission study loop'
            * [2][0] string: 'range_study_last',                [2][1] float: 'value of range of of last mission study loop'
            * [3][0] string: 'fl_initial_cruise_study_last',    [3][1] float: 'value of cruise flight level of last mission study loop'

        The input int "count_of_iteration" contains the current number of iteration loop.

        The input float "convergence_criteria" contains the convergence criteria of current iteration process.

        The output list of lists "mission_study_variables" contains the following values:
            *  [0][0] string: 'm_take_off_max_study',                    [0][1] float: 'value of maximum take off mass of current mission study loop'
            *  [1][0] string: 'mission_energy_study',                    [1][1] float: 'value of mission energy of current mission study loop'
            *  [2][0] string: 'range_study',                             [2][1] float: 'value of range of current mission study loop'
            *  [3][0] string: 'fl_initial_cruise_study',                 [3][1] float: 'value of cruise flight level of current mission study loop'
            *  [4][0] string: 'residual_m_take_off_max_study',           [4][1] float: 'value of residual of maximum take off mass of current mission study loop'
            *  [5][0] string: 'residual_mission_energy_study',           [5][1] float: 'value of residual of mission energy of current mission study loop'
            *  [6][0] string: 'residual_range_study',                    [6][1] float: 'value of residual of range of current mission study loop'
            *  [7][0] string: 'residual_fl_initial_cruise_study',        [7][1] float: 'value of residual of cruise flight level of current mission study loop'
            *  [8][0] string: 'status_m_take_off_max_study',             [8][1] bool:  'status of convergence for study take of mass'
            *  [9][0] string: 'status_mission_energy_study',             [9][1] bool:  'status of convergence for study mission energy'
            * [10][0] string: 'status_range_study',                     [10][1] bool:  'status of convergence for mission study range'
            * [11][0] string: 'status_fl_initial_cruise_study',         [11][1] bool:  'status of convergence of initial mission study flight level'

        The output list "mtom_loop_vector" contains all estimated mtom values.

        :param: paths_and_names: input list of lists
        :param: mission_study_variables: input list of lists
        :param: count_of_mission_study_loop: input int
        :param: convergence_criteria: input float
        :return: design_variables: output list of lists
    """

    ''' import for python '''
    import os
    import shutil
    from datetime import datetime
    from inspect import currentframe, getframeinfo
    from sub_function.read_xml_file import read_xml_file
    from sub_function.get_mission_energy_specific_segment import get_mission_energy_specific_segment

    ''' read data for script execution '''
    aircraft_exchange_file = ([item for item in paths_and_names if 'aircraft_exchange_file' in item])[-1][-1]
    aircraft_project = ([item for item in paths_and_names if 'aircraft_project' in item])[-1][-1]
    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory_rce = \
        ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]

    m_take_off_max_study_last = \
        ([item for item in mission_study_variables if 'm_take_off_max_study_last' in item])[-1][-1]
    mission_energy_study_last = \
        ([item for item in mission_study_variables if 'mission_energy_study_last' in item])[-1][-1]
    range_study_last = \
        ([item for item in mission_study_variables if 'range_study_last' in item])[-1][-1]
    fl_initial_cruise_study_last = \
        ([item for item in mission_study_variables if 'fl_initial_cruise_study_last' in item])[-1][-1]

    ''' initialize local parameter '''
    log_file_list = ['', '------------------------------ Current loop of mission study: '
                     + str(count_of_mission_study_loop + 1) + ' ---------', '* results:']
    status_residual_range_study = False
    status_fl_initial_cruise_study = False
    status_residual_mission_energy_study = False
    status_residual_m_take_off_max_study = False

    function_name = getframeinfo(currentframe()).function

    ''' open and read aircraft exchange file '''
    frame_info = getframeinfo(currentframe())
    path = path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/' + aircraft_exchange_file
    root_of_aircraft_exchange_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce,
                                                             aircraft_exchange_file, function_name, frame_info.lineno,
                                                             log_file_list, current_workflow_name)

    # read iteration variables
    m_take_off_max_study = root_of_aircraft_exchange_tree.find("./analysis/mission/study_mission/takeoff_mass/value")
    if m_take_off_max_study is not None:
        m_take_off_max_study = float(m_take_off_max_study.text)
    else:
        m_take_off_max_study = 0

    fl_initial_cruise_study = root_of_aircraft_exchange_tree.find(
        "./analysis/mission/study_mission/cruise/cruise_steps/cruise_step[" + "@ID='0'" + "]/altitude/value")
    if fl_initial_cruise_study is not None:
        fl_initial_cruise_study = float(fl_initial_cruise_study.text)
    else:
        fl_initial_cruise_study = 0

    mission_energy_study = get_mission_energy_specific_segment(
        root_of_aircraft_exchange_tree, 'study_mission', 'loaded_mission_energy/mission_energy')

    range_study = root_of_aircraft_exchange_tree.find("./analysis/mission/study_mission/range/value")
    if range_study is not None:
        range_study = float(range_study.text)
    else:
        range_study = 0

    # rel. residual m_take_off_max_study
    if m_take_off_max_study > 0:
        residual_m_take_off_max_study = round(abs(1 - m_take_off_max_study_last / m_take_off_max_study), 6)
        if residual_m_take_off_max_study < convergence_criteria:
            status_residual_m_take_off_max_study = True
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current mTOM_study: ' + str(m_take_off_max_study)
                             + '; Res. mTOM_study: ' + str(residual_m_take_off_max_study) + '; Convergence: '
                             + str(status_residual_m_take_off_max_study))
    else:
        residual_m_take_off_max_study = 1
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current mTOM_study: ' + str(m_take_off_max_study) + '; Res. mTOM_study: '
                             + str(residual_m_take_off_max_study) + '; Convergence: '
                             + str(status_residual_m_take_off_max_study))

    # rel. residual mission range
    if range_study > 0:
        residual_range_study = round(abs(1 - range_study_last / range_study), 6)
        if residual_range_study < convergence_criteria:
            status_residual_range_study = True
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current missionRange_study: ' + str(range_study) + '; Res. missionRange_study: '
                             + str(residual_range_study) + '; Convergence: ' + str(status_residual_range_study))

    else:
        residual_range_study = 1
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current missionRange_study: ' + str(range_study) + '; Res. missionRange_study: '
                             + str(residual_range_study) + '; Convergence: ' + str(status_residual_range_study))

    # rel. residual fl_initial_cruise_study
    if fl_initial_cruise_study > 0:
        residual_fl_initial_cruise_study = round(abs(1 - fl_initial_cruise_study_last / fl_initial_cruise_study), 6)
        if residual_fl_initial_cruise_study < convergence_criteria:
            status_fl_initial_cruise_study = True
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current initialCruiseFlightLevel_study: ' + str(fl_initial_cruise_study)
                             + '; Res. initialCruiseFlightLevel_study: ' + str(residual_fl_initial_cruise_study)
                             + '; Convergence: ' + str(status_fl_initial_cruise_study))
    else:
        residual_fl_initial_cruise_study = 1
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current initialCruiseFlightLevel_study: ' + str(fl_initial_cruise_study)
                             + '; Res. initialCruiseFlightLevel_study: ' + str(residual_fl_initial_cruise_study)
                             + '; Convergence: ' + str(fl_initial_cruise_study))

    # rel. residual mission_energy
    if mission_energy_study > 0:
        residual_mission_energy_study = round(abs(1 - mission_energy_study_last / mission_energy_study), 6)
        if residual_mission_energy_study < convergence_criteria:
            status_residual_mission_energy_study = True
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current mission_energy_study: ' + str(mission_energy_study)
                             + '; Res. mission_energy_study: ' + str(residual_mission_energy_study) + '; Convergence: '
                             + str(status_residual_mission_energy_study))
    else:
        residual_mission_energy_study = 1
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ':   '
                             'Current mission_energy_study: ' + str(mission_energy_study) + '; Res. mission_energy_study: '
                             + str(residual_mission_energy_study) + '; Convergence: '
                             + str(status_residual_mission_energy_study))

    ''' save aircraft-exchange-file of each loop '''
    file_name = aircraft_exchange_file[:len(aircraft_exchange_file) - 4]
    file_destination = current_workflow_name \
        + '/temporaryResults/aircraft_exchange_files/aircraft_exchange_files_of_study_loop/' + file_name + '_it' \
        + str(count_of_mission_study_loop + 1) + '.xml'
    shutil.copy(path, path_of_working_directory_rce + file_destination)

    log_file_list.append('')

    ''' write log-file to system '''
    if os.path.isfile(path_of_working_directory_rce + current_workflow_name + '/temp/workflowExecutionError.dat'):
        log_file_list = ['', '------------------------------ Current loop of mission study: '
                         + str(count_of_mission_study_loop + 1) + ' ---------', '',
                         str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                         'Error in design tool execution!',
                         str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                         'At least one of the enabled design tools has send an error. '
                         'The actual design loop is failed and will be canceled!',
                         str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                         'If parameter study or optimization is selected, '
                         'current loop of iteration will skipped and the next iteration will be performed!']

    log_file = open(path_of_working_directory_rce + current_workflow_name + '/unicado_workflow.log', 'a')
    for row in log_file_list:
        print(row[21:])
        log_file.write(row + '\n')
    log_file.close()

    ''' append calculated design values to list of list "design_variables" '''
    mission_study_variables.append(['m_take_off_max_study', m_take_off_max_study])
    mission_study_variables.append(['mission_energy_study', mission_energy_study])
    mission_study_variables.append(['range_study', range_study])
    mission_study_variables.append(['fl_initial_cruise_study', fl_initial_cruise_study])
    mission_study_variables.append(['residual_m_take_off_max_study', residual_m_take_off_max_study])
    mission_study_variables.append(['residual_mission_energy_study', residual_mission_energy_study])
    mission_study_variables.append(['residual_range_study', residual_range_study])
    mission_study_variables.append(['residual_fl_initial_cruise_study', residual_fl_initial_cruise_study])
    mission_study_variables.append(['status_m_take_off_max_study', status_residual_m_take_off_max_study])
    mission_study_variables.append(['status_mission_energy_study', status_residual_mission_energy_study])
    mission_study_variables.append(['status_range_study', status_residual_range_study])
    mission_study_variables.append(['status_fl_initial_cruise_study', status_fl_initial_cruise_study])

    return mission_study_variables
