# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def initialize_mission_study_analysis(paths_and_names, control_settings, parameter_for_design_case,
                                      module_list_of_sizing_loop):
    """ initialize_mission_study_analysis initialize the mission study sizing loop to perform off design analysis.
        Set initial design variables of mission study analysis to output list of lists.

        The input list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        The input list of lists "control_settings" contains the following values:
            *  [0][0] string: 'delete_plots_before_run',                              [0][1] int: 'switch to delete old plots before running'
            *  [1][0] string: 'delete_engines_before_run',                            [1][1] int: 'switch to delete old engine data before running'
            *  [2][0] string: 'clear_old_log_files_at_start',                         [2][1] int: 'switch to delete old log-files before running'
            *  [3][0] string: 'plot_output_on',                                       [3][1] int: 'switch to enable or disable plot generation'
            *  [4][0] string: 'switch_off_plots',                                     [4][1] int: 'switch to enable generation of plots of each iteration step'
            *  [5][0] string: 'report_output_on'                                      [5][1] int: 'switch to enable or disable report generation'
            *  [6][0] string: 'switch_off_reports',                                   [6][1] int: 'switch to enable generation of reports of each iteration step'
            *  [7][0] string: 'tex_report_on'                                         [7][1] int: 'switch to enable or disable tex-report generation'
            *  [8][0] string: 'switch_off_tex',                                       [8][1] int: 'switch to enable generation of tex-reports of each iteration step'
            *  [9][0] string: 'status_flag_to_skip_design_sizing'                     [9][1] int: 'status flag for plot generation of design sizing loop - iniitalized with 0'
            * [10][0] string: 'status_flag_to_skip_study_mission',                   [10][1] int: 'status flag for plot generation of study mission loop - iniitalized with 0'
            * [11][0] string: 'check_settings_for_design_logic',                     [11][1] int: 'value to switch off and on the checks of the design logic'
            * [12][0] string: 'use_control_settings_for_sub_programs',               [12][1] int: 'value for switching off and on the override control settings of subroutines'
            * [13][0] string: 'range_type_specific_factors',                         [13][1] int: 'value for switch of range scenario to set range dependent parameters'
            * [14][0] string: 'use_configs_from_existing_project',                   [14][1] int: 'value to activate copy of configuration files from existing project before running'
            * [15][0] string: 'use_engine_from_existing_project',                    [15][1] int: 'value to activate copy of engine data from existing project before running'
            * [16][0] string: 'save_configs_to_result_folder',                       [16][1] int: 'value to switch off and on the saving of configuration files in the result folder'
            * [17][0] string: 'save_log_files_to_result_folder',                     [17][1] int: 'value to switch off and on the saving of log files in the result folder'
            * [18][0] string: 'save_acft_xml_to_result_folder',                      [18][1] int: 'value to switch off and on the saving of aircraft exchange file in the result folder'
            * [19][0] string: 'save_acft_xml_after_each_iteration',                  [19][1] int: 'value to switch off and on the saving of aircraft exchange file in the result folder after each step of iteration'
            * [20][0] string: 'write_info_files',                                    [20][1] int: 'switch to activate the writing of info files after each tool execution'
            * [21][0] string: 'write_info_files_only_at_last_step_of_iteration',     [21][1] int: 'switch to activate the writing of info files only after last step of sizing loop'
            * [22][0] string: 'program_mode',                                        [22][1] string: 'Value of program execution mode - either standard_aircraft_design, or parameter_study, or optimization'

        The input list of lists "parameter_for_design_case" contains the following values:
            *  [0][0] string: 'program_mode',                     [0][1] string: 'Value of program execution mode - either standard_aircraft_design, or parameter_study, or optimization'
            *  [1][0] string: 'design_mode',                      [1][1] int:   'value to check which design mode is selected'
            *  [2][0] string: 'automatic_trim',                   [2][1] int:   'value to switch off and on the automatic trim function of the workflow'
            *  [3][0] string: 'convergence_criteria',             [3][1] float: 'value of convergence criteria of ome and mtom iteration'
            *  [4][0] string: 'number_of_max_iteration',          [4][1] int:   'value to set maximum number of design sizing iteration steps'
            *  [5][0] string: 'bridge_constraint_analyzer',       [5][1] int: 'value to switch off and on the execution of constraint_analysis module in workflow for each iteration step'
            *  [6][0] string: 'damp_mtom_iteration'               [6][1] bool:  'switch to activate damping of mtom iteration loop'
            *  [7][0] string: 'parallel_execution'                [7][1] int:   'switch to activate the parallel execution mode'
            *  [8][0] string: 'count_of_iteration',               [8][1] int:   'current number of iteration of design sizing loop'
            *  [9][0] string: 'count_of_mission_study_loop',      [9][1] int:   'current number of iteration of mission study loop'
            * [10][0] string: 'parameter_name',                  [10][1] string: 'name of the current input parameter if parameter study will be performed'
            * [11][0] string: 'global_loop_counter',             [11][1] int:    'global loop counter for parameter study mode'
            * [12][0] string: 'study_counter',                   [12][1] int:    'study counter of current input parameter if parameter study will be performed'
            * [13][0] string: 'new_value_of_input',              [13][1] float:  'value of current parameter study input parameter'

        The input list of lists "module_list_of_sizing_loop" is a dynamic list of used sizing loop modules of current workflow execution:
            *  [:][0] string: 'sizing_loop_module',          [0][:] string: 'name of the used sizing loop module of current workflow execution'

        The output list of lists "mission_study_variables" contains the following values:
            * [0][0] string: 'm_take_off_max_study',       [0][1] float: 'value of initial maximum take off mass of mission study'
            * [1][0] string: 'mission_energy_study',       [1][1] float: 'value of initial mission energy of mission study'
            * [2][0] string: 'range_study',                [2][1] float: 'value of initial range of mission study'
            * [3][0] string: 'fl_initial_cruise_study',    [3][1] float: 'value of initial cruise flight level of mission study'

        :param: paths_and_names: input list of lists
        :param: control_settings: input list of lists
        :param: parameter_for_design_case: input list of lists
        :param: module_list_of_sizing_loop: input list of lists
        :return: mission_study_variables: output list of lists
    """

    ''' imports for python '''
    import os
    import shutil
    from datetime import datetime
    from inspect import currentframe, getframeinfo
    from sub_function.read_xml_file import read_xml_file
    from sub_function.get_mission_energy_specific_segment import get_mission_energy_specific_segment
    from sub_function.write_to_log_file import write_to_log_file

    ''' create list for log-file string '''
    log_file_list = [str(datetime.now().strftime(
        '%Y-%m-%d %H:%M:%S')) + ': Generation of plots and reports of design sizing loop finished!',
        '****************************************** end activate plots and reports '
        '******************************************',
        '',
        '**************************************** initialize mission study analysis '
        '****************************************']

    ''' read data for script execution '''
    aircraft_exchange_file = ([item for item in paths_and_names if 'aircraft_exchange_file' in item])[-1][-1]
    aircraft_project = ([item for item in paths_and_names if 'aircraft_project' in item])[-1][-1]
    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory = ([item for item in paths_and_names if 'path_of_working_directory' in item])[-1][-1]
    path_of_working_directory_rce = \
        ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]

    write_info_files = ([item for item in control_settings if 'write_info_files' in item])[-1][-1]
    write_info_files_only_at_last_step_of_iteration = \
        ([item for item in control_settings if 'write_info_files_only_at_last_step_of_iteration' in item])[-1][-1]
    switch_off_plots = int(([item for item in control_settings if 'switch_off_plots' in item])[-1][-1])
    switch_off_reports = int(([item for item in control_settings if 'switch_off_reports' in item])[-1][-1])
    switch_off_tex = int(([item for item in control_settings if 'switch_off_tex' in item])[-1][-1])

    count_of_iteration = ([item for item in parameter_for_design_case if 'count_of_iteration' in item])[-1][-1]
    design_mode = int(([item for item in parameter_for_design_case if 'design_mode' in item])[-1][-1])

    function_name = getframeinfo(currentframe()).function

    # check if mission study analysis should not be performed (design_mode == 4)
    #  -> if true: -> skip mission analysis loop and perform post operations of workflow
    if not design_mode == 4:
        ''' restore aircraft-exchange-file of last loop '''
        # The -4 removes the file extension ".xml" from the passed string
        file_name = aircraft_exchange_file[:len(aircraft_exchange_file) - 4]

        # check if convergence or maximum iteration steps reached
        if not count_of_iteration == 0:
            file_source = current_workflow_name \
                + '/temporaryResults/aircraft_exchange_files/aircraft_exchange_files_of_sizing_loop/' \
                + file_name + '_finalLoop.xml'
            destination = path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/' \
                + aircraft_exchange_file
            shutil.copy(path_of_working_directory_rce + file_source, destination)
            print('Aircraft exchange file for mission study loop restored!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Aircraft exchange file for mission study loop restored!')

        # copy and remove old file
        ''' save aircraft exchange files to results folder '''
        if not os.path.isdir(path_of_working_directory_rce + current_workflow_name
                             + '/temporaryResults/aircraft_exchange_files/final_aircraft_exchange_files'):
            os.makedirs(path_of_working_directory_rce + current_workflow_name
                        + '/temporaryResults/aircraft_exchange_files/final_aircraft_exchange_files')

        if not count_of_iteration == 0:
            file = current_workflow_name \
                + '/temporaryResults/aircraft_exchange_files/aircraft_exchange_files_of_sizing_loop/' \
                + file_name + '_finalLoop.xml'
            shutil.copy(path_of_working_directory_rce + file,
                        path_of_working_directory_rce + current_workflow_name
                        + '/temporaryResults/aircraft_exchange_files/final_aircraft_exchange_files/' + file_name
                        + '_final_sizing_loop_file.xml')
            os.remove(path_of_working_directory_rce + file)

        ''' open and read aircraft exchange file '''
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/' \
            + aircraft_exchange_file
        root_of_aircraft_exchange_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce,
                                                                 aircraft_exchange_file, function_name,
                                                                 frame_info.lineno, log_file_list,
                                                                 current_workflow_name)

        # read variables for missionStudy iteration
        m_take_off_max_study = root_of_aircraft_exchange_tree.find(
            "./analysis/mission/study_mission/takeoff_mass/value")
        if m_take_off_max_study is not None:
            m_take_off_max_study = float(m_take_off_max_study.text)
        else:
            m_take_off_max_study = 0

        range_study = root_of_aircraft_exchange_tree.find(
            "./analysis/mission/study_mission/range/value")
        if range_study is not None:
            range_study = float(range_study.text)
        else:
            range_study = 0

        fl_initial_cruise_study = root_of_aircraft_exchange_tree.find(
            "./analysis/mission/study_mission/cruise/cruise_steps/cruise_step/altitude/value")
        if fl_initial_cruise_study is not None:
            fl_initial_cruise_study = float(fl_initial_cruise_study.text)
        else:
            fl_initial_cruise_study = 0
        
        mission_study_variables = [['m_take_off_max_study', m_take_off_max_study],
                                   ['mission_energy_study', get_mission_energy_specific_segment(
                                       root_of_aircraft_exchange_tree, 'study_mission',
                                       'loaded_mission_energy/mission_energy')],
                                   ['range_study', range_study],
                                   ['fl_initial_cruise_study', fl_initial_cruise_study]]

        ''' paths to temporary results of current workflow '''
        path_of_mission_study_files = path_of_working_directory_rce + current_workflow_name \
            + '/temporaryResults/config_files/preExecution/studyLoop/'
        os.makedirs(path_of_mission_study_files)

        # copy aircraft exchange file to temporaryResults directory
        path_of_mission_study_exchange_files = path_of_working_directory_rce + current_workflow_name \
            + '/temporaryResults/aircraft_exchange_files/aircraft_exchange_files_of_study_loop/'
        os.makedirs(path_of_mission_study_exchange_files)
        shutil.copy(path, path_of_mission_study_exchange_files + aircraft_exchange_file[:-4] + '_it0.xml')

        ''' Convert switches in config file to analysis mode '''
        # set switches in create_mission_xml_conf.xml
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/create_mission_xml_conf.xml'
        create_mission_conf_xml_root, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'create_mission_xml',
                                                               function_name, frame_info.lineno, log_file_list,
                                                               current_workflow_name)

        # Change switch from design mission to study mission, in order to create a study mission
        create_mission_conf_xml_root.find('./program_settings/mission_selector/value').text = "study_mission"

        # write settings to configuration xml file
        xml_tree.write(path, encoding='utf-8')

        # copy create_mission_xml_conf.xml file to temporaryResults directory
        shutil.copy(path, path_of_mission_study_files + 'create_mission_xml_conf.xml')

        # set switches in systems_design_conf.xml
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/systems_design_conf.xml'
        systems_design_conf_root, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'systems_design',
                                                           function_name, frame_info.lineno, log_file_list,
                                                           current_workflow_name)

        # Change switch from design mission to study mission,
        #  in order to calculate systems characteristics for a study mission
        systems_design_conf_root.find('./program_settings/mission_mode/value').text = "study"

        # write settings to configuration xml file
        xml_tree.write(path, encoding='utf-8')

        # copy systems_design_conf.xml file to temporaryResults directory
        shutil.copy(path, path_of_mission_study_files + 'systems_design_conf.xml')

        # set switches in mission_analysis_conf.xml
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/mission_analysis_conf.xml'
        mission_analysis_conf_xml_root, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'mission_analysis',
                                                                 function_name, frame_info.lineno, log_file_list,
                                                                 current_workflow_name)

        # for a study mission - switch off the MTOM estimation
        mission_analysis_conf_xml_root.find('./program_settings/mode/mission_methods/mission_type/value').text = \
            "study_mission"

        # write settings to configuration xml file
        xml_tree.write(path, encoding='utf-8')

        # copy mission_analysis_conf.xml file to temporaryResults directory
        shutil.copy(path, path_of_mission_study_files + 'mission_analysis_conf.xml')

        ''' create list of files in working directory '''
        files_in_working_directory = os.listdir(path_of_working_directory)
        # Check if switch switchOfPlots set to active
        if switch_off_plots == 1 or switch_off_reports == 1 or switch_off_tex == 1:
            # loop for all elements in list of files to find the necessary tools to perform mission study loop
            for file_name in files_in_working_directory:
                if (file_name == 'mission_analysis') or (file_name == 'systems_design') \
                        or (file_name == 'create_mission_xml'):
                    # disable plot and report generation for mission study
                    frame_info = getframeinfo(currentframe())
                    path = path_of_working_directory_rce + current_workflow_name + '/' + file_name + '_conf.xml'
                    file_name_tree_root, xml_tree = read_xml_file(path, path_of_working_directory_rce, file_name,
                                                                  function_name, frame_info.lineno, log_file_list,
                                                                  current_workflow_name)

                    if switch_off_plots == 1:
                        file_name_tree_root.find('./control_settings/plot_output/enable/value').text = \
                            str(bool(0)).lower()
                    if switch_off_reports == 1:
                        file_name_tree_root.find('./control_settings/report_output/value').text = str(bool(0)).lower()
                    if switch_off_tex == 1:
                        file_name_tree_root.find('./control_settings/tex_report/value').text = str(bool(0)).lower()

                    # write settings to configuration xml file
                    xml_tree.write(path, encoding='utf-8')

        ''' Write info file after last step of sizing loop iteration '''
        # Check if attribute 'onlyAtLastStepOfIteration' of 'WriteInfoFiles' and is the switch them self equal to 1
        #  -> if true: -> activate writeInfoFiles in sizing loop configuration files only for the last loop of iteration
        if write_info_files_only_at_last_step_of_iteration == 1 and write_info_files == 1:
            i = 0
            for _ in module_list_of_sizing_loop:
                module_name = module_list_of_sizing_loop[i][-1]
                # try to open module configuration file
                frame_info = getframeinfo(currentframe())
                path = path_of_working_directory_rce + current_workflow_name + '/' + module_name + '_conf.xml'
                file_name_tree_root, xml_tree = read_xml_file(path, path_of_working_directory_rce, file_name,
                                                              function_name, frame_info.lineno, log_file_list,
                                                              current_workflow_name)

                file_name_tree_root.find('./control_settings/write_info_files/value').text = str(bool(0)).lower()

                # write settings to configuration xml file
                xml_tree.write(path, encoding='utf-8')
                print('Switch of WriteInfoFile only for last step of iteration in ' + module_name
                      + '_conf.xml successfully set to inactive!')
                log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                     'Switch of WriteInfoFile only for last step of iteration in ' + module_name
                                     + '_conf.xml successfully set to inactive!')
                i += 1

        ''' write log-file to system '''
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'All switches set to perform mission study sizing loop!')
        log_file_list.append('*************************************** end initialize mission study analysis '
                             '***************************************')
        log_file_list.append('')
        log_file_list.append('****************************************** Start Mission Study Iteration! '
                             '******************************************')
        log_file_list.append('')
        write_to_log_file(path_of_working_directory_rce, current_workflow_name, log_file_list)

    else:
        mission_study_variables = [['m_take_off_max_study', 0.0],
                                   ['mission_energy_study', 0.0],
                                   ['range_study', 0.0],
                                   ['fl_initial_cruise_study', 0.0]]

    return mission_study_variables
