# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

"""
A kind of sub-module of a stand-alone module gaussianProcess.py

Author: Daigo Maruyama
Technical contact: d.maruyama@tu-braunschweig.de
"""

### standard libraries (includig numpy, scipy) ###
import numpy as np


class _Kernel:
    def __init__(self, X):
        self._X = X
        self._N, self._D = X.shape

    def covariance(self, K=None):                                                   # so-called Gram matrix
        """
        Create the covariance matrix (so called Gram matrix).

        :param K: a covariance matrix (square and symmetric matrix)
        :type K: np.ndarray (2D)
        """
        if K is None:
            K = np.zeros( (self._N, self._N), dtype=np.float64 )
        self._computeKernel(K)
        return K

    def vectorPrediction(self, x, k=None):                                          # correlation vector
        """
        The number of thepredicted point is fixed at 1 (Nnew=1), not multiple points.

        :param x: arbitrary single point of the input
        :type x: np.ndarray (1D)
        :param k: a part of the whole covariance matrix when a prediction input x is included
        :type k: np.ndarray (1D)
        """
        if k is None:
            k = np.zeros( (self._N, 1), dtype=np.float64 )
        self._computeKernel(k, x)
        return k

    def scalarPrediction(self, x, k=None):                                          # k
        """
        The number of thepredicted point is fixed at 1 (Nnew=1), not multiple points.

        :param x: arbitrary single point of the input
        :type x: np.ndarray (1D)
        :param k: a diagonal part of the whole covariance matrix when a prediction input x is included
        :type k: float
        """
        if k is None:
            k = 0.
        k += self._kernelDiag(x)
        return k

    def _kernel(self, x1, x2):
        raise NotImplementedError("Do not directly access here.")

    def _kernelDiag(self, x):
        raise NotImplementedError("Do not directly access here.")

    def _computeKernel(self, K, x=None):
        if x is None:
            for i in range(self._N):
                K[i,i] += self._kernelDiag(self._X[i])
                for j in range(i+1, self._N):
                    K[i,j] += self._kernel(self._X[i], self._X[j])
                    K[j,i] = K[i,j]
        else:
            for i in range(self._N):
                    K[i,0] += self._kernel(self._X[i], x)


class GaussianKernel(_Kernel):
    def __init__(self, X, theta, p=2):
        _Kernel.__init__(self, X)
        self._theta = theta
        self._p = p

    def _kernel(self, x1, x2):
        c = sum( [ float(np.power(10,self._theta[i], dtype=np.float64)) * np.power( abs(float(x1[i])-float(x2[i])), self._p, dtype=np.float64 ) for i in range(self._D) ] )
        return np.exp(-float(c), dtype=np.float64)

    def _kernelDiag(self, x):
        return 1.

class ConstantKernel(_Kernel):
    def __init__(self, X, theta):                                                       # X is actully not used at all.
        _Kernel.__init__(self, X)
        self._theta = theta

    def _computeKernel(self, K, x=None):                                                # due to the property of this kernel, x is actually not used.
        if x is None:
            for i in range(self._N):
                K[i,i] += self._kernelDiag()
                for j in range(i+1, self._N):
                    K[i,j] += self._kernel()
                    K[j,i] = K[i,j]
        else:
            for i in range(self._N):
                    K[i,0] += self._kernel()

    def _kernel(self, x1=None, x2=None):
        return self._theta

    def _kernelDiag(self, x=None):
        return self._theta

class LinearKernel(_Kernel):
    def __init__(self, X, theta):
        _Kernel.__init__(self, X)
        self._theta = theta

    def _kernel(self, x1, x2):
        return self._theta * np.dot(float(x1), float(x2))

    def _kernelDiag(self, x):
        return self._kernel(x, x)

class RegularizationKernel(_Kernel):
    def __init__(self, X, theta):
        _Kernel.__init__(self, X)
        self._theta = theta

    def _kernel(self, x1=None, x2=None):
        return 0.

    def _kernelDiag(self, x):
        return float(np.power(10,self._theta, dtype=np.float64))                            # x is not used.

