# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

import numpy as np
import sys
import os
import time
import userinput                # userinput file, which defines "formulation"
import optimization             # The API file of the package

"""
The main file to execute EGO (Efficient Global Optimization) described by optimization.py
run: python main.py
in this package

Author: Daigo Maruyama
Technical contact: d.maruyama@tu-braunschweig.de
"""

''' initialize local parameter '''
# absolute path to working directory of current workflow execution
# Attention this path will set dynamically during workflow execution!!! Do not enter anything here!!!
path_to_current_working_dir = 'path'

# --- 1. Formulation -----------------------------------------------------------------------------
# The ingredients for the formulation is extracted from the module "userinput".
functions = userinput.functionsUNICADO
x_bounds, g_bounds = userinput.boundary_preparation_unicado(path_to_current_working_dir)
formulation = optimization.Formulation(functions, x_bounds, g_bounds)

# ------------------------------------------------------------------------------------------------

# --- 1.5. (optional: ordinary optimization) ---
#    xopt, yopt = formulation.ordinary()
#    print("xopt =", xopt)
#    print("yopt =", yopt)
# ----------------------------------------------

# --- 2. Optimization (EGO) ----------------------------------------------------------------------
"""
User-Input:

Define additional parameters for Bayesian optimization
N + Nimposed = "Total computational cost"

The computation in gaussianProcess.py sometimes stops during the optimization of the likelihood function due to violated values of the hyperparameer theta. In that case, please run it again. Random effects are integrated in the algorithm. The solutions for this are related to the issue 1 in below. This process will be improved to be robust.

Notes on accuracy vs efficiency:
1. The greater "multiStart" is, the higher the accuracy and the robustness of the result is. However, the complexity of the computational cost becomes squre proportional to the number of "multiStart".
2. The computational costs (of the surrogate model part) is CUBIC proportional to the number of sample size (N+Nimposed)
    (Strictly to say, the rank of the covariance matrix in K in gaussianProcess.py)
The issues 1 and 2 arise independently of each other. Further computational time can be accelerated by using cython, etc.

Solutions for issue 1: There are various approaches. The efficiency is dependent on cases for users.
 - multistart with random with evolutional algorithm
   - multistart with random initial point with gradient-based opt
Solutions for issue 2: Approximated methods - summarized in the deliverable (in 2020)
"""

# --- example of User-Input ---
"""
Indication of sample size:

1D:
N = 5
Nimposed = 3

2D:
N = 30 to 50
Nimposed = 10

2Dconstraints:
N = 30 to 50        (or N=10, Nimposed=30)
Nimposed = 10
"""
'''
N = 10#5#50                              # initial sample size
Nimposed = 30#10                         # additional sample size searched in the process of Bayesian optimization
proc = 1                                 # the number of processors allocated for one simulation when run in parallel

# some parameters for surrogate models (Details of the parameter explanation ca be seen in gaussianProcess.py)
regularization = True                    # False: The surrogate model passes through all the sample points. True: No guarantee that the surrogate model passes through all the sample points. However, higher probability that the less errors in the processes related to surrogate models. The sample points are assumed to have a monovariate Gaussian noise.
#    kernelmodel = {"Gaussian": True, "Constant": False, "Linear": False}
#    trend = "ordinary"                      # "simple" or "ordinary"
doeMethod = "Sobol"                      # "LHS" or "Halton" or "Sobol"

method = "EIPI"                          # "EI" or "EIPI" or some others
multiStart = 1#10                       # greter: higher accuracy but lower efficiency. multi trials of the optimization of the likelihood function (and the acquisition function as well) to make the surrogate models more robust (the computational time for the construction becomes larger.). If gradientOptSurrogate is True, higher number can be an option for the robustness (avoiding local optimal).
### ----------------------------- ###
gradientOptSurrogate = False              # method to optimize the likelihood function in constructing surrogate models, True: BFGS
'''

''' initialize optimization parameter '''
# read optimization settings from optimization configuration file
N, Nimposed, proc, regularization, doeMethod, method, multiStart, gradientOptSurrogate = \
    userinput.read_optimization_settings(path_to_current_working_dir)

multiFidelity = False#True

'''
Parameters defined inside EGO :
        self._regularization = regularization
        self._basisFunction = basisFunction
        self._doeMethod = doeMethod
        self._kernelmodel = kernelmodel
        self._multiStart = multiStart
        self._trend = 0.

        # temporary for my graph
        self._N = N
        self._Nimposed = Nimposed
        self._activeLearningMethod = activeLearningMethod
'''

if multiFidelity == False:
        xopt, yopt = formulation.EGO(N, Nimposed, proc, regularization = regularization, doeMethod= doeMethod, multiStart= multiStart)
# elif multiFidelity == True:
#         xopt, yopt = formulation.EGO(N, Nimposed, proc).MFEGO(Nlow, N, Nimposed, proc)

print("xopt =", xopt)                   # optimal solution of the design variable
print("yopt =", yopt)                   # optimal solution of the cost function
print("Running parallel function is also fine.\n")
print("Terminated normally")

# visualization (only after Bayesian opt)
input_dim = len(x_bounds)
if input_dim == 1 or input_dim == 2:
    formulation.visualization(Npre=100)                 # Npre: the number of grid for each dimension

for i in range(1+len(g_bounds)):
    print("sensitivity for each function =", formulation.surrogate[i].automatic_relevance_determination())

''' write finish flag to temporary working directory to terminate workflow execution '''
if not os.path.isfile(path_to_current_working_dir + '/temp/optimizationFinishFlag.dat'):
    with open(path_to_current_working_dir + '/temp/optimizationFinishFlag.dat', 'w') as file:
        file.close()

time.sleep(3)

''' delete workflow semaphore to finish workflow '''
# check if the semaphore wait flag for the optimization framework is existing
#  -> if true: -> remove flag to execute optimization framework
if os.path.isfile(path_to_current_working_dir + '/temp/semaphoreFlagOptimization.dat'):
    os.remove(path_to_current_working_dir + '/temp/semaphoreFlagWorkflowExecution.dat')
