# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

# imports for python
import csv
import numpy as np
import os
import sys
import time
import xml.etree.ElementTree
import subprocess, sys, shlex, shutil, os, time
import xml.etree.ElementTree as ET
from datetime import datetime

"""
Userinput file to define a blackbox function which returns all the output values to be evaluated (one cost function
+ arbitrary number of constraints) for an input vector x, which is design variables (Userinput1).
The search bounds of the design variables are also required to be defined (Userinput2).
Then, based this rule please adjust the blackbox function for your optimization formulation.

Author: Daigo Maruyama
Technical contact: d.maruyama@tu-braunschweig.de
"""


def read_optimization_settings(path_to_current_working_dir):
    """
         Preparation of optimization settings to run the optimization framework.

        * The input string 'path_to_current_working_dir' contains the system path of working directory of current workflow execution.

        * The output integer 'n' contains the number of initial samples.

        * The output integer 'n_imposed' contains the number of imposed samples.

        * The output integer 'proc' contains the number of processors to use.

        * The output bool 'regularization' contains the True or False flag for pass surrogate model through all sample points.

        * The output string 'doe_method' contains the name of the doe methode which should be used.

        * The output string 'method' contains the name of the methode which should be used.

        * The output integer 'multi_start' contains the value which accuracy should be reached.

        * The output bool 'gradient_opt_surrogate' contains the status flag which surrogate mode should be used.

        :param: path_to_current_working_dir, input string
        :return: x_bounds_unicado, output numpy array; g_bounds_unicado, output numpy array
    """

    # try to read optimization configuration file
    try:
        xml_tree = xml.etree.ElementTree.ElementTree(file=path_to_current_working_dir + '/optimization_conf.xml')
        root_of_optimization_tree = xml_tree.getroot()

        n = int(root_of_optimization_tree.find(
            './ProgramSettings/OptimizationSettings/numberOfInitialSamples').text)
        n_imposed = int(root_of_optimization_tree.find(
            './ProgramSettings/OptimizationSettings/numberOfImposedSamples').text)
        proc = int(root_of_optimization_tree.find(
            './ProgramSettings/OptimizationSettings/numberOfProcessors').text)
        regularization = bool(int(root_of_optimization_tree.find(
            './ProgramSettings/OptimizationSettings/regularization').text))
        doe_method = str(root_of_optimization_tree.find(
            './ProgramSettings/OptimizationSettings/doeMethod').text)
        method = str(root_of_optimization_tree.find(
            './ProgramSettings/OptimizationSettings/method').text)
        multi_start = int(root_of_optimization_tree.find(
            './ProgramSettings/OptimizationSettings/multiStart').text)
        gradient_opt_surrogate = bool(int(root_of_optimization_tree.find(
            './ProgramSettings/OptimizationSettings/gradientOptSurrogate').text))

    except OSError:
        print('ERROR: The optimization configuration file of current workflow execution could not be opened or is '
              'corrupt!\n'
              '       Optimization could not be performed! The optimization framework is aborted!')
        sys.exit(1)

    return n, n_imposed, proc, regularization, doe_method, method, multi_start, gradient_opt_surrogate


def boundary_preparation_unicado(path_to_current_working_dir):
    """
        Preparation of upper and lower boundaries of design variables and requirements for UNICADO function.

        * The input string 'path_to_current_working_dir' contains the system path of working directory of current workflow execution.

        :param: path_to_current_working_dir, input string
        :return: x_bounds_unicado, output numpy array; g_bounds_unicado, output numpy array
    """

    # constants
    feet = 0.304800609601219

    ''' read design variable-, cost function- and design requirement keys from optimization configuration file '''
    try:
        xml_tree = xml.etree.ElementTree.ElementTree(file=path_to_current_working_dir + '/optimization_conf.xml')
        root_of_optimization_tree = xml_tree.getroot()

        # read upper and lower boundaries of design variables
        x_bounds_unicado = []
        number_of_design_variables = int(root_of_optimization_tree.find(
            './ProgramSettings/designParameter/designVariables').text)

        for i in range(0, number_of_design_variables):
            # read lower boundary of design variables and check for infinity
            lower_boundary = root_of_optimization_tree.find(
                "./ProgramSettings/designParameter/designVariable[" + "@ID='" + str(i) + "'"
                + "]lowerBoundary").text
            if lower_boundary == 'inf':
                lower_boundary = np.inf
            elif lower_boundary == '-inf':
                lower_boundary = -np.inf
            else:
                lower_boundary = float(lower_boundary)

            # read unit of lower boundary of design variables and check for "feet" or "ft"
            lower_boundary_unit = root_of_optimization_tree.find(
                "./ProgramSettings/designParameter/designVariable[" + "@ID='" + str(i) + "'"
                + "]lowerBoundary").get('Unit')
            if lower_boundary_unit == 'feet' or lower_boundary == 'ft':
                lower_boundary *= feet

            # read upper boundary of design variables and check for infinity
            upper_boundary = root_of_optimization_tree.find(
                "./ProgramSettings/designParameter/designVariable[" + "@ID='" + str(i) + "'"
                + "]upperBoundary").text
            if upper_boundary == 'inf':
                upper_boundary = np.inf
            elif upper_boundary == '-inf':
                upper_boundary = -np.inf
            else:
                upper_boundary = float(upper_boundary)

            # read unit of upper boundary of design variables and check for "feet" or "ft"
            upper_boundary_unit = root_of_optimization_tree.find(
                "./ProgramSettings/designParameter/designVariable[" + "@ID='" + str(i) + "'"
                + "]upperBoundary").get('Unit')
            if upper_boundary_unit == 'feet' or upper_boundary_unit == 'ft':
                upper_boundary *= feet

            # add design variables to x_bound array
            design_variable_boundaries = [lower_boundary, upper_boundary]
            x_bounds_unicado.append(design_variable_boundaries)

        # read upper and lower boundaries of design variables
        g_bounds_unicado = []
        number_of_design_requirements = int(root_of_optimization_tree.find(
            './ProgramSettings/designRequirementParameter/designRequirements').text)

        for i in range (0, number_of_design_requirements):
            # read lower boundary of design variables and check for infinity
            lower_boundary = root_of_optimization_tree.find(
                "./ProgramSettings/designRequirementParameter/designRequirement[" + "@ID='" + str(i) + "'"
                + "]lowerBoundary").text
            if lower_boundary == 'inf':
                lower_boundary = np.inf
            elif lower_boundary == '-inf':
                lower_boundary = -np.inf
            else:
                lower_boundary = float(lower_boundary)

            # read unit of lower boundary of design variables and check for "feet" or "ft"
            lower_boundary_unit = root_of_optimization_tree.find(
                "./ProgramSettings/designRequirementParameter/designRequirement[" + "@ID='" + str(i) + "'"
                + "]lowerBoundary").get('Unit')
            if lower_boundary_unit == 'feet' or lower_boundary == 'ft':
                lower_boundary *= feet

            # read upper boundary of design variables and check for infinity
            upper_boundary = root_of_optimization_tree.find(
                "./ProgramSettings/designRequirementParameter/designRequirement[" + "@ID='" + str(i) + "'"
                + "]upperBoundary").text
            if upper_boundary == 'inf':
                upper_boundary = np.inf
            elif upper_boundary == '-inf':
                upper_boundary = -np.inf
            else:
                upper_boundary = float(upper_boundary)

            # read unit of upper boundary of design variables and check for "feet" or "ft"
            upper_boundary_unit = root_of_optimization_tree.find(
                "./ProgramSettings/designRequirementParameter/designRequirement[" + "@ID='" + str(i) + "'"
                + "]upperBoundary").get('Unit')
            if upper_boundary_unit == 'feet' or upper_boundary_unit == 'ft':
                upper_boundary *= feet

            # add design variables to x_bound array
            design_requirement_boundaries = [lower_boundary, upper_boundary]
            g_bounds_unicado.append(design_requirement_boundaries)

    except OSError:
        print('ERROR: The optimization configuration file of current workflow execution could not be opened or is '
              'corrupt!\n'
              '       Optimization could not be performed! The optimization framework is aborted!')
        sys.exit(1)

    return x_bounds_unicado, g_bounds_unicado

"""
Example: 1D function
"""
def functions1D(x):
    """
    Define blackbox functions for your problem setting.

    :param x: design variables
    :type x: np.ndarray (1D)


    f: cost function (a scalar)
    g: constraint functions (a vector)
    return: (f, g), status
    (f, g): np.ndarray (1D)
    status: boolean (if the output is successfully obtained)
    """
#    f  = np.power( (6.*(1.-x)-2.), 2 ) * np.sin(12.*(1.-x)-4.)
    f  = np.power( (6.*x-2.), 2 ) * np.sin(12.*x-4.)
#    g1 = 10.*x - 10.
    """
    generalized:

    g2 = ...
    g3 = ...
    ...
    gM = ...

    if not all( [f == f, g1 == g1, ..., gM == gM] ):
        status = "Failure"
    else:
        status = "Success"

    return np.hstack((f, g1, ..., gM)), status
    """

#    if not all( [f == f, g1 == g1] ):
#        status = "Failure"
#    else:
#        status = "Success"

#    return np.hstack((f, g1)), status
    return f, "Success"


"""
Example: 2D function
"""
def functions2D(x):
    """
    The same rule as the 1D case
    """
    D = 2
    f = sum( 100. * np.power( x[i+1] - np.power( x[i], 2 ), 2 ) + np.power( ( 1. - x[i] ), 2 ) for i in range(D-1) )            # Rosenbrock function

    if f is None:
        status = "Failure"
    else:
        status = 'Success'

    return np.array([f]), status


def functionsUNICADO(x):
    """
    The same as the examples in the above (1D, 2D functions)
    Define blackbox functions for your problem setting.

    :param: x: design variables
    :type: x: numpy array
    """
    ''' initialize local parameter '''
    return_code = 0

    # absolute path to working directory of current workflow execution
    # Attention this path will set dynamically during workflow execution!!! Do not enter anything here!!!
    path_to_current_working_dir = 'path'
    path_to_output_dir = 'output_path'

    ''' write current x values to csv output file '''
    array_str = np.array2string(x, separator=' ', precision=16)
    array_str = array_str[1:-1] + "\n"
    # Write to a text file
    if os.path.isfile(path_to_output_dir + 'X-values.csv'):
        with open(path_to_output_dir + 'X-values.csv', 'a') as file:
            file.write(array_str)
            file.close()

    ''' loop to wait for the semaphore trigger flag to prepare current optimization loop '''
    while True:
        if x is None:
            break
                
        # ToDo: hier abbruchbedinung einbauen???
        
        # check if the semaphore flag is deleted by the UNICADOworkflow -> if true: -> perform preparation steps
        if not os.path.isfile(path_to_current_working_dir + '/temp/semaphoreFlagOptimizationPreparation.dat'):
            with open(path_to_current_working_dir + '/temp/semaphoreFlagOptimizationPreparation.dat', 'w') as file:
                file.close()
            break
        # else condition: -> semaphore flag is existing -> wait 30 seconds and retry the check
        else:
            # check if the working directory of current workflow execution is not existing
            #  -> if true: -> raise an error and abort optimization framework
            if not os.path.isdir(path_to_current_working_dir):
                print('ERROR: The specified path to the working directory of current workflow execution does not exist!'
                      '\n'
                      '       Optimization could not be performed! The optimisation framework is aborted!')
                sys.exit(1)

            # else condition: working directory is still existing -> wait 30 seconds to retry semaphore check
            else:
                time.sleep(30)

    # check if the UNICADOworkflow configuration file of current execution is existing
    #  -> if false: -> raise an error an abort program
    if not os.path.isfile(path_to_current_working_dir + '/unicado_workflow_conf.xml'):
        if not os.path.isfile(path_to_current_working_dir + '/temp/optimizationRuntimeError.dat'):
            with open(path_to_current_working_dir + '/temp/optimizationRuntimeError.dat', 'w') as file:
                file.close()
        print('ERROR: The UNICADOworkflow configuration file of current workflow execution does not exist!\n'
              '       Optimization could not be performed! The optimisation framework is aborted!')
        sys.exit(1)

    ''' open and read parameter from unicado_workflow_conf.xml configuration file '''
    try:
        xml_tree = xml.etree.ElementTree.ElementTree(file=path_to_current_working_dir + '/unicado_workflow_conf.xml')
        root_of_workflow_tree = xml_tree.getroot()
        aircraft_exchange_file = root_of_workflow_tree.find('./control_settings/aircraft_exchange_file_name/value').text
        aircraft_project = aircraft_exchange_file[:-4]

    except OSError:
        print('ERROR: The UNICADOworkflow configuration file of current workflow execution could not be opened!\n'
              '       Optimization could not be performed! The optimization framework is aborted!')
        sys.exit(1)

    ''' read design variable-, cost function- and design requirement keys from optimization configuration file '''
    try:
        xml_tree = xml.etree.ElementTree.ElementTree(file=path_to_current_working_dir + '/optimization_conf.xml')
        root_of_optimization_tree = xml_tree.getroot()

        # read design variables from optimization configuration file
        # X
        design_variable_keys = []
        design_variable_xml_paths = []
        design_variable_system_paths = []
        number_of_design_variables = int(root_of_optimization_tree.find(
            './ProgramSettings/designParameter/designVariables').text)

        # loop across the number of design variables to read parameter xml paths
        for i in range(0, number_of_design_variables):
            design_variable_keys.append(root_of_optimization_tree.find(
                "./ProgramSettings/designParameter/designVariable[" + "@ID='" + str(i) + "'"
                + "]ParameterName").text)
            design_variable_xml_paths.append(root_of_optimization_tree.find(
                "./ProgramSettings/designParameter/designVariable[" + "@ID='" + str(i) + "'"
                + "]PathInXmlFile").text)
            design_variable_system_paths.append(root_of_optimization_tree.find(
                "./ProgramSettings/designParameter/designVariable[" + "@ID='" + str(i) + "'"
                + "]PathToFile").text)

        # read cost function keys from optimization configuration file
        # F(x)
        cost_functions_keys = []
        cost_functions_xml_paths = []
        cost_functions_system_paths = []
        number_of_cost_function_parameter = int(root_of_optimization_tree.find(
            './ProgramSettings/costFunctions/costFunctionsParameters').text)

        # loop across the number of cost function parameter to read parameter xml paths
        for i in range(0, number_of_cost_function_parameter):
            cost_functions_keys.append(root_of_optimization_tree.find(
                "./ProgramSettings/costFunctions/costFunctionsParameter[" + "@ID='" + str(i) + "'"
                + "]ParameterName").text)
            cost_functions_xml_paths.append(root_of_optimization_tree.find(
                "./ProgramSettings/costFunctions/costFunctionsParameter[" + "@ID='" + str(i) + "'"
                + "]PathInXmlFile").text)
            cost_functions_system_paths.append(root_of_optimization_tree.find(
                "./ProgramSettings/costFunctions/costFunctionsParameter[" + "@ID='" + str(i) + "'"
                + "]PathToFile").text)

        # read design requirement keys from optimization configuration file
        # G(x), H(x) --> constraints
        design_requirement_keys = []
        design_requirement_xml_paths = []
        design_requirement_system_paths = []
        number_of_design_requirements = int(root_of_optimization_tree.find(
            './ProgramSettings/designRequirementParameter/designRequirements').text)

        # loop across the number of design requirements to read parameter xml paths
        for i in range(0, number_of_design_requirements):
            design_requirement_keys.append(root_of_optimization_tree.find(
                "./ProgramSettings/designRequirementParameter/designRequirement[" + "@ID='" + str(i) + "'"
                + "]ParameterName").text)
            design_requirement_xml_paths.append(root_of_optimization_tree.find(
                "./ProgramSettings/designRequirementParameter/designRequirement[" + "@ID='" + str(i) + "'"
                + "]PathInXmlFile").text)
            design_requirement_system_paths.append(root_of_optimization_tree.find(
                "./ProgramSettings/designRequirementParameter/designRequirement[" + "@ID='" + str(i) + "'"
                + "]PathToFile").text)

    except OSError:
        print('ERROR: The optimization configuration file of current workflow execution could not be opened or is '
              'corrupt!\n'
              '       Optimization could not be performed! The optimization framework is aborted!')
        sys.exit(1)

    ''' set paths to file directory and aircraft exchange files '''
    cost_functions = {}
    design_requirement = {}
    # check if the current given input x is not None -> if true: -> perform workflow preparation and execution
    if x is not None:
        file_directory = path_to_current_working_dir + '/' + aircraft_project + '/'
        aircraft_exchange_default_file = file_directory + aircraft_project + '_Default.xml'
        aircraft_exchange_file = file_directory + aircraft_project + '.xml'

        ''' read aircraft exchange default file as element tree to write candidate values to the file '''
        tree = xml.etree.ElementTree.ElementTree(file=aircraft_exchange_default_file)
        root = tree.getroot()

        # loop across all given design varaibles to set candidate values to the aircraft exchange file
        for i in range(0, number_of_design_variables):
            root.find(design_variable_xml_paths[i]).text = str(x[i])

        tree.write(aircraft_exchange_file, encoding='utf-8')

        time.sleep(3)

        if os.path.isfile(path_to_current_working_dir + '/temp/semaphoreFlagWorkflowExecution.dat'):
            os.remove(path_to_current_working_dir + '/temp/semaphoreFlagWorkflowExecution.dat')

        ''' loop to wait for the semaphore trigger flag to perform optimization '''
        print('Attention: Optimization framework is waiting for the finishing of the current workflow loop!')
        print('Execution of UNIACDOworkflow is running for: \n')
        for i, key in enumerate(design_variable_keys):
            print('                                            ' + key + ' = ' + str(x[i]) + '\n')

        while True:
            # check if the semaphore flag is deleted by the UNICADOworkflow -> if true: -> perform optimization steps
            if not os.path.isfile(path_to_current_working_dir + '/temp/semaphoreFlagOptimization.dat'):
                with open(path_to_current_working_dir + '/temp/semaphoreFlagOptimization.dat', 'w') as file:
                    file.close()
                break
            # else condition: -> semaphore flag is existing -> wait 10 seconds and retry the check
            else:
                # check if the working directory of current workflow execution is not existing
                #  -> if true: -> raise an error and abort optimization framework
                if not os.path.isdir(path_to_current_working_dir):
                    print('ERROR: Temporary working directory of current workflow execution does not exist!\n'
                          '       Optimization framework is shut down, due to a runtime error!')
                    sys.exit(1)

                # else condition: working directory is still existing -> wait 10 seconds to retry semaphore check
                else:
                    time.sleep(10)

        # check if the current workflow loop is failed -> if true: -> return error code 1
        if os.path.isfile(path_to_current_working_dir + '/temp/workflowExecutionError.dat'):
            os.remove(path_to_current_working_dir + '/temp/workflowExecutionError.dat')
            return_code = 1

        if os.path.isfile(path_to_current_working_dir + '/temp/convergenceError.dat'):
            os.remove(path_to_current_working_dir + '/temp/convergenceError.dat')
            return_code = 2

    # Else condition: The given input x is None -> Set rerun code to 3 and skipp workflow execution for current input
    else:
        return_code = 3

    # check if the workflow execution is not successfully -> if true: -> set status flag to 'Failure'
    if return_code != 0:
        status = 'Failure'
        print('Current loop of UNICADOworkflow failed! \n'
              'Results will not be used for optimization!')

        print('Execution of UNIACDOworkflow failed for: \n')
        for key in design_variable_keys:
            print('                                         ' + key + ' =  "None" ' + '\n')

        print(cost_functions_keys)
        for key in cost_functions_keys:
            cost_functions[key] = None
        for key in design_requirement_keys:
            design_requirement[key] = None

    # Else condition: The workflow execution is successfully -> set status flag to 'Success'
    else:
        status = 'Success'
        print('Execution of UNIACDOworkflow successful for: \n')
        for i, key in enumerate(design_variable_keys):
            print('                                            ' + key + ' = ' + str(x[i]) + '\n')

        # --- output (cost functions )---
        for i in range(0, number_of_cost_function_parameter):
            tree = xml.etree.ElementTree.ElementTree(file=cost_functions_system_paths[i])
            root_of_cost_functions_file = tree.getroot()
            cost_functions[cost_functions_keys[i]] = float(root_of_cost_functions_file.find(
                cost_functions_xml_paths[i]).text)

        # --- output (constraints) ---
        for i in range(0, number_of_design_requirements):
            tree = xml.etree.ElementTree.ElementTree(file=design_requirement_system_paths[i])
            root_of_design_requirement_file = tree.getroot()
            design_requirement[design_requirement_keys[i]] = float(root_of_design_requirement_file.find(
                design_requirement_xml_paths[i]).text)

    print('Attention: Optimization framework is running to perform optimization steps!\n'
          ' The current workflow execution is waiting for the finishing of optimization steps of current run!')

    ''' your definition of a scalar cost function '''
    # define one scalar value output function, f: scaler
    #  (e.g. linear combination of several output values such mission energy, ...)
    for value in cost_functions.values():
        print('cost_functions_value is: ' + str(value))

    f = sum(value if value is not None else np.inf for value in cost_functions.values())
    print('f is: ' + str(f))
    # Write to a text file
    if os.path.isfile(path_to_output_dir + 'Y-values.csv'):
        with open(path_to_output_dir + 'Y-values.csv', "a", newline='') as csvfile:
            csv_writer = csv.writer(csvfile)
            # Write a single row containing the float value
            csv_writer.writerow([f])

    g = [value if value is not None else np.inf for value in design_requirement.values()]
    print('g is: ' + str(g))
    # Write to a text file
    if os.path.isfile(path_to_output_dir + 'G-values.csv'):
        with open(path_to_output_dir + 'G-values.csv', "a", newline='') as csvfile:
            # Convert the list to a string where values are separated by spaces
            line = ' '.join(map(str, g))  # Convert each element to a string and join with a space
            csvfile.write(line + "\n")

    return np.array([f]+g), status


"""
User-Input 2:

Define "formulation" of your optimization problem
 - Write down f and g in the module main.function (in the beginning of this file)
 - Define the bounds of x (input) and g (constraints)
"""


# --- example of User-Input 1 ---
"""
Example: 1D function
"""
xbounds1D = [[0., 1.]]               # for 1D test function
gbounds1D = [[-np.inf, -15.]]#[]     # unconstrained otimization can be defined by gbounds = []                  # TLARs itself in UNICADO


"""
Example: 2D function
"""
xbounds2D = [[-2., 2.], [-1., 3.]]       # for 2D Rosenbrock function
gbounds2D = []


"""
Example: 1D function constraints
"""
xbounds1Dconstraints = [[0., 1.]]
gbounds1Dconstraints = [[-1., 1.], [0., 0.75]]


"""
Example: 2D function constraints
"""
#xbounds2D = [ [-5., 10.], [0., 15.] ]       # for the Braine Test Function
xbounds2Dconstraints = [[0., 1.], [0., 1.]]       # for the Braine Test Function
gbounds2Dconstraints = [[0.2, np.inf]]
