# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def read_parameter_study_settings(install_path_directory, log_file_list, current_workflow_name='UNICADOworkflow'):
    """ Function to read parameter from parameter study configuration file.

        The input string 'install_path_directory' contains the absolute path to the UNICADO install directory.

        The input list "log_file_list" contains all system prints of function: perform_parameter_study.py.

        The input string "current_workflow_name" contains the name of the current workflow execution.
            * If no input is given, default sting 'UNICADOworkflow' is used.

        The output list of lists "parameter_study_list" contains the following values:
            *  [0][0] string: 'exit_if_subprogram_exits',                [0][1] integer: 'switch to handle tool error during execution'
            *  [1][0] string: 'start_every_run_with_ref_file',           [1][1] integer: 'switch to to select if each run starts with reference files'
            *  [2][0] string: 'reset_variables_after_run',               [2][1] integer: 'switch to reset variables after finishing the total parameter study'
            *  [3][0] string: 'write_failed_req_checks_results',         [3][1] integer: 'switch to handle output writing after tool execution error'
            *  [4][0] string: 'parameter_study_methode',                 [4][1] integer: 'switch to select witch parameter study methode should be used'
            *  [5][0] string: 'path_to_input_parameter_values_file',     [5][1] string:  'absolute system path to the input parameter value file'
            *  [6][0] string: 'save_results',                            [6][1] integer: 'switch to activate or deactivate output saving'
            *  [7][0] string: 'save_only_aircraft_exchange_file',        [7][1] integer: 'switch to activate only storing of aircraft exchange files'
            *  [8][0] string: 'max_folder_size',                         [8][1] float:   'the maximum allowed size of parameter study directory in GB'
            *  [9][0] string: 'reproduce_output_file_from_old_study',    [9][1] integer: 'switch to handle the use of old output files in current parameter study'
            * [10][0] string: 'path_to_current_study',                  [10][1] string:  'absolute path to the working directory of current parameter study'
            * [11][0] string: 'path_to_folder_old_study',               [11][1] string:  'absolute path to folder of old study results
            * [12][0] string: 'number_of_input_parameter',              [12][1] integer: 'number of parameter study inputs to iterate'
            * [13][0] string: 'number_of_output_parameter',             [13][1] integer: 'number of output parameter to write after each finished run'

        The output list "log_file_list" contains all system prints of functions: perform_parameter_study.py and read_parameter_study_settings.py.

        :param: install_path_directory: input string
        :param: log_file_list: input list
        :param: current_workflow_name: input string
        :return: parameter_study_list: output list of lists; log_file_list: output list
    """

    ''' imports for python '''
    import shutil
    from inspect import currentframe, getframeinfo
    from sub_function.read_xml_file import read_xml_file

    ''' initialize local parameter '''
    function_name = getframeinfo(currentframe()).function
    frame_info = getframeinfo(currentframe())
    path_of_working_directory_rce = install_path_directory + 'workingDirectoryRCE/'
    path = path_of_working_directory_rce + current_workflow_name + '/parameter_study_conf.xml'

    # copy back-up parameter study configuration file to working directory of rce
    #  to avoid realtime workflow manipulation
    root_of_parameter_study_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'parameter_study',
                                                           function_name, frame_info.lineno, log_file_list,
                                                           current_workflow_name)

    path_to_current_study = str(root_of_parameter_study_tree.find(
        "./ProgramSettings/SensitivityStudySettings/PathToCurrentStudy").text)
    shutil.copyfile(path_to_current_study + '/reference_aircraft/parameter_study_conf.xml', path)

    ''' open and read parameter from parameter_study_conf.xml configuration file to element tree '''
    frame_info = getframeinfo(currentframe())
    root_of_parameter_study_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'parameter_study',
                                                           function_name, frame_info.lineno, log_file_list,
                                                           current_workflow_name)

    ''' read program-specific control settings from element tree '''
    exit_if_subprogram_exits = int(root_of_parameter_study_tree.find(
        "./ProgramSpecific/ExitIfSubProgramExits").text)
    start_every_run_with_ref_file = int(root_of_parameter_study_tree.find(
        "./ProgramSpecific/StartEveryRunWithRefFile").text)
    reset_variables_after_run = int(root_of_parameter_study_tree.find(
        "./ProgramSpecific/ResetVariablesAfterRun").text)
    write_failed_req_checks_results = int(root_of_parameter_study_tree.find(
        "./ProgramSpecific/WriteFailedReqChecksResults").text)

    ''' read program settings from element tree '''
    parameter_study_methode = int(root_of_parameter_study_tree.find(
        "./ProgramSettings/Mode").text)
    path_to_input_parameter_values_file = str(root_of_parameter_study_tree.find(
        "./ProgramSettings/SensitivityStudySettings/PathToInputParameterValuesFile").text)
    save_results = int(root_of_parameter_study_tree.find(
        "./ProgramSettings/SensitivityStudySettings/SaveResults").text)
    save_only_aircraft_exchange_file = int(root_of_parameter_study_tree.find(
        "./ProgramSettings/SensitivityStudySettings/SaveResults").get('OnlyAiXFile'))
    max_folder_size = int(root_of_parameter_study_tree.find(
        "./ProgramSettings/SensitivityStudySettings/MaxFolderSize").text)
    reproduce_output_file_from_old_study = int(root_of_parameter_study_tree.find(
        "./ProgramSettings/SensitivityStudySettings/ReproduceOutputFileFromOldStudy").text)
    path_to_current_study = str(root_of_parameter_study_tree.find(
        "./ProgramSettings/SensitivityStudySettings/PathToCurrentStudy").text)
    path_to_folder_old_study = str(root_of_parameter_study_tree.find(
        "./ProgramSettings/SensitivityStudySettings/PathToFolderOldStudy").text)

    ''' read parameter settings from element tree '''
    number_of_input_parameter = int(root_of_parameter_study_tree.find(
        "./ProgramSettings/ParameterSettings/InputParameters").text)
    number_of_output_parameter = int(root_of_parameter_study_tree.find(
        "./ProgramSettings/ParameterSettings/OutputParameters").text)

    ''' save configuration parameter to parameter_study_list '''
    parameter_study_list = [['exit_if_subprogram_exits', exit_if_subprogram_exits],
                            ['start_every_run_with_ref_file', start_every_run_with_ref_file],
                            ['reset_variables_after_run', reset_variables_after_run],
                            ['write_failed_req_checks_results', write_failed_req_checks_results],
                            ['parameter_study_methode', parameter_study_methode],
                            ['path_to_input_parameter_values_file', path_to_input_parameter_values_file],
                            ['save_results', save_results],
                            ['save_only_aircraft_exchange_file', save_only_aircraft_exchange_file],
                            ['max_folder_size', max_folder_size],
                            ['reproduce_output_file_from_old_study', reproduce_output_file_from_old_study],
                            ['path_to_current_study', path_to_current_study],
                            ['path_to_folder_old_study', path_to_folder_old_study],
                            ['number_of_input_parameter', number_of_input_parameter],
                            ['number_of_output_parameter', number_of_output_parameter]]

    return parameter_study_list, log_file_list
