# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def reset_parameter_study_files(install_path_directory, path_to_current_study, perform_workflow_execution,
                                log_file_list, current_workflow_name='UNICADOworkflow'):
    """ Function to reset all files after finishing each input parameter study.

        The input string "install_path_directory" contains the absolute path to the UNICADO install directory.

        The input string 'path_to_current_study' contains the absolute path to the working directory of current parameter study.

        The input integer "perform_workflow_execution" contains the status to perform the parameter study or not.

        The input list "log_file_list" contains all system prints of function: perform_parameter_study.py.

        The input string "current_workflow_name" contains the name of the current workflow execution.
            * If no input is given, default sting 'UNICADOworkflow' is used.

        The output list "log_file_list" contains all system prints of function: set_configuration_parameter and reset_parameter_study_files.py.

        :param: install_path_directory: input string
        :param: path_to_current_study: input integer
        :param: perform_workflow_execution: input integer
        :param: log_file_list: input list
        :param: current_workflow_name: input string
        :return: log_file_list: output list
    """

    ''' imports of python '''
    import os
    import shutil
    from datetime import datetime

    ''' initialize local parameter '''
    path_to_workflow_dir = install_path_directory + 'workingDirectoryRCE/' + current_workflow_name + '/'
    i = 0
    # find position of project and aircraft name in given path string
    index = path_to_current_study.rfind('_parameter_study')

    index_cut = path_to_current_study[:index].rfind('_')
    project_name = path_to_current_study[index_cut+1:index]

    ''' reset workflow configuration file to avoid outer workflow manipulation '''
    if perform_workflow_execution == 1:
        if os.path.isfile(path_to_current_study + '/reference_aircraft/unicado_workflow_conf.xml'):
            shutil.copyfile(path_to_current_study + '/reference_aircraft/unicado_workflow_conf.xml',
                            path_to_workflow_dir + 'unicado_workflow_conf.xml')
            print('Workflow configuration file successfully reset!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Workflow configuration file successfully reset!')

    else:
        if os.path.isfile(path_to_current_study + '/reference_aircraft/unicado_workflow_conf_start.xml'):
            shutil.copyfile(path_to_current_study + '/reference_aircraft/unicado_workflow_conf_start.xml',
                            path_to_workflow_dir + 'unicado_workflow_conf.xml')
            print('Workflow configuration file successfully reset!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Workflow configuration file successfully reset!')

    ''' reset range type specific factors xml-file to avoid outer workflow manipulation '''
    if os.path.isfile(path_to_current_study + '/reference_aircraft/range_type_specific_factors.xml'):
        shutil.copyfile(path_to_current_study + '/reference_aircraft/range_type_specific_factors.xml',
                        path_to_workflow_dir + 'range_type_specific_factors.xml')
        print('Range type specific factors xml-file successfully reset!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'Range type specific factors xml-file successfully reset!')

    ''' reset aircraft exchange file to avoid outer workflow manipulation '''
    if os.path.isfile(path_to_current_study + '/reference_aircraft/' + project_name + '_start.xml'):
        shutil.copyfile(path_to_current_study + '/reference_aircraft/' + project_name + '_start.xml',
                        path_to_workflow_dir + project_name + '/' + project_name + '.xml')
        print('Aircraft exchange file successfully reset!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'Aircraft exchange file successfully reset!')

    ''' reset CPACS file to avoid outer workflow manipulation '''
    if os.path.isfile(path_to_current_study + '/reference_aircraft/' + project_name + '_CPACS.xml'):
        shutil.copyfile(path_to_current_study + '/reference_aircraft/' + project_name + '_CPACS.xml',
                        path_to_workflow_dir + project_name + '/' + project_name + '_CPACS.xml')
        print('CPACS file successfully reset!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'CPACS file successfully reset!')

    ''' reset engine data if exist to avoid outer workflow manipulation '''
    if os.path.isdir(path_to_current_study + '/reference_aircraft/engine_data/'):
        if os.path.isdir(path_to_workflow_dir + project_name + '/engine_data/'):
            shutil.rmtree(path_to_workflow_dir + project_name + '/engine_data/')
        shutil.copytree(path_to_current_study + '/reference_aircraft/engine_data/',
                        path_to_workflow_dir + project_name + '/engine_data/')
        print('Engine data successfully reset!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'Engine data successfully reset!')

    ''' reset aero data if exist to avoid outer workflow manipulation '''
    if os.path.isdir(path_to_current_study + '/reference_aircraft/aero_data/'):
        if os.path.isdir(path_to_workflow_dir + project_name + '/aero_data/'):
            shutil.rmtree(path_to_workflow_dir + project_name + '/aero_data/')
        shutil.copytree(path_to_current_study + '/reference_aircraft/aero_data/',
                        path_to_workflow_dir + project_name + '/aero_data/')
        print('Aero data successfully reset!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'Aero data successfully reset!')

    ''' reset geometry data if exist to avoid outer workflow manipulation '''
    if os.path.isdir(path_to_current_study + '/reference_aircraft/geometry_data/'):
        if os.path.isdir(path_to_workflow_dir + project_name + '/geometry_data/'):
            shutil.rmtree(path_to_workflow_dir + project_name + '/geometry_data/')
        shutil.copytree(path_to_current_study + '/reference_aircraft/geometry_data/',
                        path_to_workflow_dir + project_name + '/geometry_data/')
        print('Geometry data successfully reset!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'Geometry data successfully reset!')

    ''' reset mission data if exist to avoid outer workflow manipulation '''
    if os.path.isdir(path_to_current_study + '/reference_aircraft/mission_data/'):
        if os.path.isdir(path_to_workflow_dir + project_name + '/mission_data/'):
            shutil.rmtree(path_to_workflow_dir + project_name + '/mission_data/')
        shutil.copytree(path_to_current_study + '/reference_aircraft/mission_data/',
                        path_to_workflow_dir + project_name + '/mission_data/')
        print('Mission data successfully reset!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'Mission data successfully reset!')

    ''' reset all module configuration files to avoid outer workflow manipulation '''
    if os.path.isdir(path_to_current_study + '/reference_aircraft/config_files/'):
        list_of_working_directory_content = os.listdir(install_path_directory)
        path_to_config = path_to_current_study + '/reference_aircraft/config_files/'
        list_of_directory_content = os.listdir(path_to_config)
        for element in list_of_directory_content:
            if os.path.isdir(path_to_config + element):
                list_of_sub_folder_content = os.listdir(path_to_config + element)
                for sub_element in list_of_sub_folder_content:
                    # extract the tool name from current sub_element string -> toolName_conf.xml[:-9] returns toolName
                    tool_name_list = os.listdir(path_to_config + element + '/' + sub_element)
                    for config_file in tool_name_list:
                        tool_name = config_file[:-9]
                        for content in list_of_working_directory_content:
                            if content == tool_name:
                                if os.path.isdir(install_path_directory + content):
                                    if content == 'aerodynamic_assessment':
                                        if os.path.isfile(path_to_config + element + '/' + sub_element
                                                          + '/liftingLine_conf.xml'):
                                            shutil.copyfile(path_to_config + element + '/' + sub_element
                                                            + '/liftingLine_conf.xml',
                                                            path_to_workflow_dir + 'liftingLine_conf.xml')
                                            print('Reset of liftingLine configuration file successfully done!')
                                            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                                                                 + ': Reset of liftingLine configuration file '
                                                                   'successfully done!')
                                    shutil.copyfile(path_to_config + element + '/' + sub_element + '/' + config_file,
                                                    path_to_workflow_dir + config_file)
                                    print('Reset of ' + tool_name + ' configuration file successfully done!')
                                    log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                                         'Reset of ' + tool_name + ' configuration file '
                                                         'successfully done!')

    return log_file_list
