# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def set_parameter_for_resume_parameter_study(paths_and_names, path_to_folder_old_study, log_file_list):
    """ Function to check which parameter to analyze next and create a file to perform the parameter study.

        The input list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        The input string "path_to_folder_old_study" contains the absolute path to the results of old parameter study to resume.

        The input list "log_file_list" contains all system prints of function: set_configuration_parameter.py.

        The output bool "abort_parameter_study_flag" contains the status to abort the parameter study or not.

        The output integer "number_of_total_steps" contains the number of parameter study steps still to be performed.

        The output list "log_file_list" contains all system prints of functions: set_configuration_parameter.py and initialize_parameter_study.py.

        :param: paths_and_names: list of lists
        :param: path_to_folder_old_study: input string
        :param: log_file_list: input list
        :return: abort_parameter_study_flag: output bool, number_of_total_steps: output integer, log_file_list: output list
    """

    ''' imports of python '''
    import os
    import shutil
    from datetime import datetime
    from inspect import currentframe, getframeinfo
    from sub_function.path_check import path_check
    from sub_function.read_xml_file import read_xml_file

    ''' read data for script execution '''
    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory_rce = \
        ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]

    ''' initialize local parameter '''
    index_list = []
    output_list = []
    count_of_line = 0
    number_of_total_steps = 0
    number_of_parameter_to_run = 1
    abort_parameter_study_flag = False
    function_name = getframeinfo(currentframe()).function

    ''' try to open output csv-file from old study '''
    # check path of old parameter study
    _, path_to_folder_old_study = path_check(path_to_folder_old_study)

    # check if the output csv-file of old parameter study exist
    if os.path.isfile(path_to_folder_old_study + '/output_csv/parameter_study_output.csv'):
        # try to open the output csv-file from old parameter study
        try:
            output_csv_file = open(path_to_folder_old_study + '/output_csv/parameter_study_output.csv', 'r')
            # loop to count the header strings of parameter to estimate last parameter of old parameter study
            for line in output_csv_file:
                count_of_line += 1
                output_list.append(line)
                if '# ID' in line:
                    index_list.append(count_of_line)
            output_csv_file.close()

            # check if not only the reference case exists in the old parameter study results -> if true:
            #  -> perform parameter study from the last parameter
            if len(index_list) > 1:
                # -2 eliminates the reference case and the last parameter of study from counting list
                number_of_parameter_to_run = len(index_list) - 2

                # check if the temp directory exist in the directory of old study
                #  -> if true: -> delete the folder and create a new one
                if os.path.isdir(path_to_folder_old_study + 'temp'):
                    shutil.rmtree(path_to_folder_old_study + 'temp')
                os.mkdir(path_to_folder_old_study + 'temp')

                parameter_file = open(path_to_folder_old_study + '/temp/number_of_current_parameter.dat', 'a')
                parameter_file.write(str(number_of_parameter_to_run))
                parameter_file.close()

                # remove old entries of last parameter from old output csv-file
                if os.path.isfile(path_to_folder_old_study + '/output_csv/parameter_study_output.csv'):
                    os.remove(path_to_folder_old_study + '/output_csv/parameter_study_output.csv')

                output_csv_file = open(path_to_folder_old_study + '/output_csv/parameter_study_output.csv', 'a')
                i = 0
                for line in output_list:
                    if i < index_list[-1]-1:
                        output_csv_file.write(line)
                        i += 1
                    else:
                        break
                output_csv_file.close()

            ''' open and read parameter from parameter_study_conf.xml configuration file to element tree '''
            frame_info = getframeinfo(currentframe())
            path = path_of_working_directory_rce + current_workflow_name + '/parameter_study_conf.xml'
            root_of_parameter_study_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce,
                                                                   'parameter_study',
                                                                   function_name, frame_info.lineno, log_file_list,
                                                                   current_workflow_name)

            number_of_input_parameter = int(root_of_parameter_study_tree.find(
                "./ProgramSettings/ParameterSettings/InputParameters").text)

            # loop across the number of input parameters to estimate total number of necessary workflow loops
            i = number_of_parameter_to_run + 1
            while i <= number_of_input_parameter:
                number_of_steps_up = int(root_of_parameter_study_tree.find(
                    "./ProgramSettings/ParameterSettings/InputParameter[" + "@ID='" + str(i) + "'"
                    + "]ParametricStudy/input_selector_specific_settings/mode_0/NumberOfStepsUp").text)
                number_of_steps_down = int(root_of_parameter_study_tree.find(
                    "./ProgramSettings/ParameterSettings/InputParameter[" + "@ID='" + str(i) + "'"
                    + "]ParametricStudy/input_selector_specific_settings/mode_0/NumberOfStepsDown").text)
                number_of_total_steps = number_of_total_steps + number_of_steps_up + number_of_steps_down
                i += 1

            # Add + 1 to number of total steps for reference calculation
            number_of_total_steps += 1

        # exception handling if the old output csv-file could not be opened
        except OSError:
            abort_parameter_study_flag = True
            print('Error: The existing output.csv file from old parameter study could not be opened! \n'
                  'After finishing the reference aircraft estimation the parameter study will be aborted!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                                 + ': Error: The existing output.csv file from old parameter study could not be opened!'
                                   '\n'
                                   'After finishing the reference aircraft estimation the parameter study will be'
                                   ' aborted!')

    # else condition: output csv-file of old parameter study does not exist -> set abort parameter study flag to 'True'
    else:
        abort_parameter_study_flag = True

    print('number_of_total_steps is: ' + str(number_of_total_steps))

    return abort_parameter_study_flag, number_of_total_steps, log_file_list
