# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def prepare_optimization(install_path_directory, current_workflow_name, global_loop_counter):
    """ set_configuration_parameter read workflow configuration parameter from workflow configuration file.
        Set values to output lists for following workflow components.

        The input string "install_path_directory" contains the absolute path to the installation directory of UNICADO.

        The input string "current_workflow_name" contains the name of current workflow execution.

        The output integer "global_loop_counter" contains the number of current outer workflow loop.

        The output integer "preparation_status" contains the status of preparation.

        :param: install_path_directory: input string
        :param: current_workflow_name: input string
        :param: global_loop_counter: input integer
        :return: preparation_status: output integer
    """

    ''' imports for python '''
    import os
    import shutil
    from inspect import currentframe, getframeinfo
    from sub_function.read_xml_file import read_xml_file
    from sub_function.xml_path_check import xml_path_check

    ''' initialize local parameter '''
    log_file_list = []
    preparation_status = 1
    path_of_working_directory_rce = install_path_directory + 'workingDirectoryRCE/'
    path_to_temporary_working_directory = install_path_directory + 'workingDirectoryRCE/' + current_workflow_name + '/'

    function_name = getframeinfo(currentframe()).function

    ''' open and read parameter from unicado_workflow_conf.xml configuration file '''
    frame_info = getframeinfo(currentframe())
    path = path_to_temporary_working_directory + 'unicado_workflow_conf.xml'
    root_of_workflow_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'unicado_workflow',
                                                    function_name, frame_info.lineno, log_file_list,
                                                    current_workflow_name)

    aircraft_project = root_of_workflow_tree.find('./control_settings/aircraft_exchange_file_name/value').text[:-4]
    path_to_reference_project_directory = install_path_directory + 'workflowResults/' + current_workflow_name + '_' \
        + aircraft_project + '_optimization/referenceCase/'

    # check if the current outer loop counter is equal to 2
    #  -> if true: -> prepare UNICADOworkflow configuration file to perform optimization
    #   -> disable all update and copy modes to use the same files even during optimization
    if global_loop_counter == 2:
        # delete old default file if it is existing
        if os.path.isfile(path_to_temporary_working_directory + aircraft_project + '/' + aircraft_project
                          + '_Default.xml'):
            os.remove(path_to_temporary_working_directory + aircraft_project + '/' + aircraft_project + '_Default.xml')

        try:
            shutil.copyfile(path_to_reference_project_directory + aircraft_project + '.xml',
                            path_to_temporary_working_directory + aircraft_project + '/' + aircraft_project
                            + '_Default.xml')

            # set clean up parameter to zero and activate retrofit mode in UNICADOworkflow configuration file
            root_of_workflow_tree.find(
                './control_settings/program_specific_settings/'
                'pre_workflow_operation_settings/set_project_folder_to_default/value').text = str(bool(0)).lower()
            root_of_workflow_tree.find(
                './control_settings/program_specific_settings/'
                'pre_workflow_operation_settings/delete_engines_before_run/value').text = str(bool(0)).lower()
            root_of_workflow_tree.find(
                './control_settings/program_specific_settings/'
                'iteration_settings/use_configs_from_existing_project/config_copy_mode/value').text = "mode_0"
            root_of_workflow_tree.find(
                './control_settings/program_specific_settings/'
                'iteration_settings/use_engine_from_existing_project/engine_copy_mode/value').text = "mode_0"
            root_of_workflow_tree.find(
                './program_settings/design_case_settings/design_mode/value').text = "mode_2"
            xml_tree.write(path_to_temporary_working_directory + 'unicado_workflow_conf.xml', encoding='utf-8')

        except OSError:
            print('Error: clean up parameter could not be set to zero in unicado_workflow_conf.xml!\n'
                  'Optimization framework will be aborted!')
            preparation_status = 0

        # check if the optimization configuration file exist -> if true: -> check the xml paths of all parameter
        if os.path.isfile(path_to_temporary_working_directory + 'optimization_conf.xml'):
            frame_info = getframeinfo(currentframe())
            path = path_to_temporary_working_directory + 'optimization_conf.xml'
            root_of_optimization_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'optimization',
                                                                function_name, frame_info.lineno, log_file_list,
                                                                current_workflow_name)
            try:
                # set system path of temporary working directory of current workflow execution
                #  to optimization configuration file
                root_of_optimization_tree.find(
                    './ProgramSettings/SensitivityStudySettings/PathToCurrentWorkingDir').text = \
                    path_to_temporary_working_directory
                # read and check xml paths of design variables and set system path to xml file
                number_of_design_variables = int(root_of_optimization_tree.find(
                    './ProgramSettings/designParameter/designVariables').text)
                for i in range(0, number_of_design_variables):
                    xml_path_to_check = root_of_optimization_tree.find(
                        "./ProgramSettings/designParameter/designVariable[" + "@ID='" + str(i) + "'" + "]"
                        "PathInXmlFile").text
                    corrected_xml_path = xml_path_check(xml_path_to_check)
                    root_of_optimization_tree.find(
                        "./ProgramSettings/designParameter/designVariable[" + "@ID='" + str(i) + "'" + "]"
                        "PathInXmlFile").text = corrected_xml_path
                    xml_file_name = root_of_optimization_tree.find(
                        "./ProgramSettings/designParameter/designVariable[" + "@ID='" + str(i) + "'" + "]"
                        "FileName").text
                    if '_conf' in xml_file_name:
                        path_to_file = path_to_temporary_working_directory + xml_file_name
                    else:
                        path_to_file = path_to_temporary_working_directory + aircraft_project + '/' + xml_file_name
                    root_of_optimization_tree.find(
                        "./ProgramSettings/designParameter/designVariable[" + "@ID='" + str(i) + "'" + "]"
                        "PathToFile").text = path_to_file

                # read and check xml paths of cost functions and set system path to xml file
                number_of_cost_function_parameter = int(root_of_optimization_tree.find(
                    './ProgramSettings/costFunctions/costFunctionsParameters').text)
                for i in range(0, number_of_cost_function_parameter):
                    xml_path_to_check = root_of_optimization_tree.find(
                        "./ProgramSettings/costFunctions/costFunctionsParameter[" + "@ID='" + str(i) + "'" + "]"
                        "PathInXmlFile").text
                    corrected_xml_path = xml_path_check(xml_path_to_check)
                    root_of_optimization_tree.find(
                        "./ProgramSettings/costFunctions/costFunctionsParameter[" + "@ID='" + str(i) + "'" + "]"
                        "PathInXmlFile").text = corrected_xml_path
                    xml_file_name = root_of_optimization_tree.find(
                        "./ProgramSettings/costFunctions/costFunctionsParameter[" + "@ID='" + str(i) + "'" + "]"
                        "FileName").text
                    if '_conf' in xml_file_name:
                        path_to_file = path_to_temporary_working_directory + xml_file_name
                    else:
                        path_to_file = path_to_temporary_working_directory + aircraft_project + '/' + xml_file_name
                    root_of_optimization_tree.find(
                        "./ProgramSettings/costFunctions/costFunctionsParameter[" + "@ID='" + str(i) + "'" + "]"
                        "PathToFile").text = path_to_file

                # read and check xml paths of requirement parameter and set system path to xml file
                number_of_design_requirements = int(root_of_optimization_tree.find(
                    './ProgramSettings/designRequirementParameter/designRequirements').text)
                for i in range(0, number_of_design_requirements):
                    xml_path_to_check = root_of_optimization_tree.find(
                        "./ProgramSettings/designRequirementParameter/designRequirement[" + "@ID='" + str(i) + "'" + "]"
                        "PathInXmlFile").text
                    corrected_xml_path = xml_path_check(xml_path_to_check)
                    root_of_optimization_tree.find(
                        "./ProgramSettings/designRequirementParameter/designRequirement[" + "@ID='" + str(i) + "'" + "]"
                        "PathInXmlFile").text = corrected_xml_path
                    xml_file_name = root_of_optimization_tree.find(
                        "./ProgramSettings/designRequirementParameter/designRequirement[" + "@ID='" + str(i) + "'" + "]"
                        "FileName").text
                    if '_conf' in xml_file_name:
                        path_to_file = path_to_temporary_working_directory + xml_file_name
                    else:
                        path_to_file = path_to_temporary_working_directory + aircraft_project + '/' + xml_file_name
                    root_of_optimization_tree.find(
                        "./ProgramSettings/designRequirementParameter/designRequirement[" + "@ID='" + str(i) + "'" + "]"
                        "PathToFile").text = path_to_file

                xml_tree.write(path, encoding='utf-8')

            except OSError:
                print('Error: Optimization configuration file could not opened or is corrupt!\n'
                      'Optimization framework will be aborted!')
                preparation_status = 0

        else:
            print('Error: Optimization configuration file could not find in the temporary working directory of current '
                  'workflow execution!\n'
                  'Optimization framework will be aborted!')
            preparation_status = 0

    ''' reset module configuration files '''
    if os.path.isdir(path_to_reference_project_directory + 'config_files/preExecution'):
        list_of_directory_content = os.listdir(path_to_reference_project_directory + 'config_files/preExecution')

        for directory in list_of_directory_content:
            path_to_config_files = path_to_reference_project_directory + 'config_files/preExecution/' + directory
            list_of_sub_folder_content = os.listdir(path_to_config_files)
            for config_file in list_of_sub_folder_content:
                try:
                    shutil.copyfile(path_to_config_files + '/' + config_file,
                                    path_to_temporary_working_directory + config_file)
                    print('Reset of ' + config_file[:-9] + ' configuration file successful.')
                except OSError:
                    print('Reset of ' + config_file[:-9] + ' configuration file failed.')
                    preparation_status = 0

    else:
        print('Error: Could not find ' + path_to_reference_project_directory + 'config_files/preExecution\n'
              'Optimization framework will be aborted!')
        preparation_status = 0

    return preparation_status
