# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def reset_of_important_values(paths_and_names, control_settings, program_mode):
    """ reset_of_important_values resets all important switches in the aircraft exchange file to perform design sizing loop.
        Attention: If the selected design mode is not "clean sheet design", switches will not reset.

        The input list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        The input list of lists "control_settings" contains the following values:
            *  [0][0] string: 'delete_plots_before_run',                              [0][1] int: 'switch to delete old plots before running'
            *  [1][0] string: 'delete_engines_before_run',                            [1][1] int: 'switch to delete old engine data before running'
            *  [2][0] string: 'clear_old_log_files_at_start',                         [2][1] int: 'switch to delete old log-files before running'
            *  [3][0] string: 'plot_output_on',                                       [3][1] int: 'switch to enable or disable plot generation'
            *  [4][0] string: 'switch_off_plots',                                     [4][1] int: 'switch to enable generation of plots of each iteration step'
            *  [5][0] string: 'report_output_on'                                      [5][1] int: 'switch to enable or disable report generation'
            *  [6][0] string: 'switch_off_reports',                                   [6][1] int: 'switch to enable generation of reports of each iteration step'
            *  [7][0] string: 'tex_report_on'                                         [7][1] int: 'switch to enable or disable tex-report generation'
            *  [8][0] string: 'switch_off_tex',                                       [8][1] int: 'switch to enable generation of tex-reports of each iteration step'
            *  [9][0] string: 'status_flag_to_skip_design_sizing'                     [9][1] int: 'status flag for plot generation of design sizing loop - iniitalized with 0'
            * [10][0] string: 'status_flag_to_skip_study_mission',                   [10][1] int: 'status flag for plot generation of study mission loop - iniitalized with 0'
            * [11][0] string: 'check_settings_for_design_logic',                     [11][1] int: 'value to switch off and on the checks of the design logic'
            * [12][0] string: 'use_control_settings_for_sub_programs',               [12][1] int: 'value for switching off and on the override control settings of subroutines'
            * [13][0] string: 'use_range_type_specific_factors',                     [13][1] int: 'value for switch of range scenario to set range dependent parameters'
            * [14][0] string: 'use_configs_from_existing_project',                   [14][1] int: 'value to activate copy of configuration files from existing project before running'
            * [15][0] string: 'use_engine_from_existing_project',                    [15][1] int: 'value to activate copy of engine data from existing project before running'
            * [16][0] string: 'save_configs_to_result_folder',                       [16][1] int: 'value to switch off and on the saving of configuration files in the result folder'
            * [17][0] string: 'save_log_files_to_result_folder',                     [17][1] int: 'value to switch off and on the saving of log files in the result folder'
            * [18][0] string: 'save_acft_xml_to_result_folder',                      [18][1] int: 'value to switch off and on the saving of aircraft exchange file in the result folder'
            * [19][0] string: 'save_acft_xml_after_each_iteration',                  [19][1] int: 'value to switch off and on the saving of aircraft exchange file in the result folder after each step of iteration'
            * [20][0] string: 'write_info_files',                                    [20][1] int: 'switch to activate the writing of info files after each tool execution'
            * [21][0] string: 'write_info_files_only_at_last_step_of_iteration',     [21][1] int: 'switch to activate the writing of info files only after last step of sizing loop'
            * [22][0] string: 'program_mode',                                        [22][1] string: 'Value of program execution mode - either standard_aircraft_design, or parameter_study, or optimization'

        :param: paths_and_names: input list of lists
        :param: control_settings: input list of lists
        :return: none
    """

    ''' imports for python '''
    from datetime import datetime
    from inspect import currentframe, getframeinfo
    from sub_function.read_xml_file import read_xml_file
    from sub_function.write_to_log_file import write_to_log_file
    from pre_condition.set_range_type_specific_factors import set_range_type_specific_factors

    '''create list for log-file string '''
    log_file_list = ['',
                     '******************************************** reset Of important values '
                     '********************************************']

    ''' read data for script execution '''
    aircraft_exchange_file = ([item for item in paths_and_names if 'aircraft_exchange_file' in item])[-1][-1]
    aircraft_project = ([item for item in paths_and_names if 'aircraft_project' in item])[-1][-1]
    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory_rce = \
        ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]

    use_range_type_specific_factors = \
        int(([item for item in control_settings if 'use_range_type_specific_factors' in item])[-1][-1])

    ''' initialize local parameter'''
    function_name = getframeinfo(currentframe()).function

    ''' open and reset requirement switches to zero aircraft exchange file '''
    # try to open configuration file as element tree
    frame_info = getframeinfo(currentframe())
    path = path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/' \
        + aircraft_exchange_file
    root_of_aircraft_exchange_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce,
                                                             aircraft_exchange_file, function_name,
                                                             frame_info.lineno, log_file_list, current_workflow_name)

    ''' open and read range dependent parameter from range_type_specific_factors.xml file '''
    if not use_range_type_specific_factors == 0:
        ''' initialize public parameter '''
        scenario_range = str('no range specified')
        design_range = float(-1.0)
        frame_info = getframeinfo(currentframe())
        path_2 = path_of_working_directory_rce + current_workflow_name + '/range_type_specific_factors.xml'

        ''' read range type specific factors from xml-file '''
        range_type_specific_factors, _ = read_xml_file(path_2, path_of_working_directory_rce,
                                                       'range_type_specific_factors', function_name,
                                                       frame_info.lineno, log_file_list, current_workflow_name)

        ''' read and set range scenario'''
        if use_range_type_specific_factors == 1:
            design_range = float(root_of_aircraft_exchange_tree.find(
                './requirements_and_specifications/requirements/top_level_aircraft_requirements/'
                'design_mission/range/value').text)

        if use_range_type_specific_factors == 2 or (0 < design_range < 5556000.): # given here in [m]; for ranges below 3000 NM
            scenario_range = 'short-range'

        if use_range_type_specific_factors == 3 or (3000 <= design_range < 10186000.): # given here in [m]; for ranges between 3000 NM and 5500 NM
            scenario_range = 'medium-range'

        if use_range_type_specific_factors == 4 or design_range >= 10186000.:
            scenario_range = 'long-range'

        ''' call function to set range type specific factors to module configuration files'''
        log_file_list = set_range_type_specific_factors(paths_and_names, range_type_specific_factors,
                                                        scenario_range, log_file_list)
        
    if not use_range_type_specific_factors == 0 or program_mode == 'optimization':
        ''' convert switches in config files of modules to perform sizing loop: '''
        # set switches in create_mission_xml_conf.xml
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/create_mission_xml_conf.xml'
        create_mission_conf_xml_root, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'create_mission_xml',
                                                               function_name, frame_info.lineno, log_file_list,
                                                               current_workflow_name)

        # Change switch to design mission in order to create a design mission
        create_mission_conf_xml_root.find('./program_settings/mission_selector/value').text = "design_mission"

        # write settings to configuration xml file
        xml_tree.write(path, encoding='utf-8')

        # set switches in systems_design_conf.xml
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/systems_design_conf.xml'
        systems_design_conf_root, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'systems_design',
                                                           function_name, frame_info.lineno, log_file_list,
                                                           current_workflow_name)

        # change design case data
        systems_design_conf_root.find('./program_settings/mission_mode/value').text = "design"
        # write settings to configuration xml file
        xml_tree.write(path, encoding='utf-8')

        # set switches in mission_analysis_conf.xml
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/mission_analysis_conf.xml'
        mission_analysis_conf_root, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'mission_analysis',
                                                             function_name, frame_info.lineno, log_file_list,
                                                             current_workflow_name)

        # change mTOM estimation method
        mission_analysis_conf_root.find('./program_settings/mode/mission_methods/mission_type/value').text = \
            "design_mission"
        # change attribute "Mode" of polar switch
        # mission_analysis_conf_root.find('./ProgramSettings/General/PolarSwitchMissionPoint').set('Mode', '1')
        # write settings to configuration xml file
        xml_tree.write(path, encoding='utf-8')

        print('Important values set to default to perform design sizing loop!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'Important values set to default to perform design sizing loop!')

    ''' write log-file to system '''
    log_file_list.append('****************************************** end reset Of important values '
                         '******************************************')
    write_to_log_file(path_of_working_directory_rce, current_workflow_name, log_file_list)

    return
