# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def set_configuration_parameter(paths_and_names, global_loop_counter):
    """ set_configuration_parameter read workflow configuration parameter from workflow configuration file.
        Set values to output lists for following workflow components.

        The input list of lists "paths_and_names" contains the following values:
            * [0][0] string: 'path_to_python_scripts',        [0][1] string: 'the system path to python scripts'
            * [1][0] string: 'path_of_working_directory',     [1][1] string: 'the system path of working directory'
            * [2][0] string: 'path_of_working_directory_rce', [2][1] string: 'the system path of working directory for rce'
            * [3][0] string: 'path_of_origin_tool_directory', [3][1] string: 'the system path to the root directory of UNICADO repositories'
            * [4][0] string: 'current_workflow_name',         [4][1] string: 'workflow name of the current workflow execution'
            * [5][0] string: 'path_to_aircraft_projects',     [5][1] string: 'the system path to the repository of aircraft design projects'
            * [6][0] string: 'path_to_aircraft_design_tools', [6][1] string: 'the system path to the repository of aircraft design tools'
            * [7][0] string: 'path_to_software_tools',        [7][1] string: 'the system path to the repository of software tools'
            * [8][0] string: 'path_to_json_files',            [8][1] string: 'the path to module json files in .rce system folder'

        The input integer "global_loop_counter" contains the number of current outer workflow loop.

        The output list of lists "accuracy_list" contains the following values:
            * [0][0] string: 'accuracy_low',    [0][1] float: 'value of low accuracy condition'
            * [1][0] string: 'accuracy_medium', [1][1] float: 'value of medium accuracy condition'
            * [2][0] string: 'accuracy_high',   [2][1] float: 'value of high accuracy condition'

        The output list of lists "control_settings" contains the following values:
            *  [0][0] string: 'delete_plots_before_run',                              [0][1] int: 'switch to delete old plots before running'
            *  [1][0] string: 'delete_engines_before_run',                            [1][1] int: 'switch to delete old engine data before running'
            *  [2][0] string: 'clear_old_log_files_at_start',                         [2][1] int: 'switch to delete old log-files before running'
            *  [3][0] string: 'plot_output_on',                                       [3][1] int: 'switch to enable or disable plot generation'
            *  [4][0] string: 'switch_off_plots',                                     [4][1] int: 'switch to enable generation of plots of each iteration step'
            *  [5][0] string: 'report_output_on'                                      [5][1] int: 'switch to enable or disable report generation'
            *  [6][0] string: 'switch_off_reports',                                   [6][1] int: 'switch to enable generation of reports of each iteration step'
            *  [7][0] string: 'tex_report_on'                                         [7][1] int: 'switch to enable or disable tex-report generation'
            *  [8][0] string: 'switch_off_tex',                                       [8][1] int: 'switch to enable generation of tex-reports of each iteration step'
            *  [9][0] string: 'status_flag_to_skip_design_sizing'                     [9][1] int: 'status flag for plot generation of design sizing loop - iniitalized with 0'
            * [10][0] string: 'status_flag_to_skip_study_mission',                   [10][1] int: 'status flag for plot generation of study mission loop - iniitalized with 0'
            * [11][0] string: 'check_settings_for_design_logic',                     [11][1] int: 'value to switch off and on the checks of the design logic'
            * [12][0] string: 'use_control_settings_for_sub_programs',               [12][1] int: 'value for switching off and on the override control settings of subroutines'
            * [13][0] string: 'use_range_type_specific_factors',                     [13][1] int: 'value for switch of range scenario to set range dependent parameters'
            * [14][0] string: 'use_configs_from_existing_project',                   [14][1] int: 'value to activate copy of configuration files from existing project before running'
            * [15][0] string: 'use_engine_from_existing_project',                    [15][1] int: 'value to activate copy of engine data from existing project before running'
            * [16][0] string: 'save_configs_to_result_folder',                       [16][1] int: 'value to switch off and on the saving of configuration files in the result folder'
            * [17][0] string: 'save_log_files_to_result_folder',                     [17][1] int: 'value to switch off and on the saving of log files in the result folder'
            * [18][0] string: 'save_acft_xml_to_result_folder',                      [18][1] int: 'value to switch off and on the saving of aircraft exchange file in the result folder'
            * [19][0] string: 'save_acft_xml_after_each_iteration',                  [19][1] int: 'value to switch off and on the saving of aircraft exchange file in the result folder after each step of iteration'
            * [20][0] string: 'write_info_files',                                    [20][1] int: 'switch to activate the writing of info files after each tool execution'
            * [21][0] string: 'write_info_files_only_at_last_step_of_iteration',     [21][1] int: 'switch to activate the writing of info files only after last step of sizing loop'

        The output list of lists "parameter_for_design_case" contains the following values:
            *  [0][0] string: 'program_mode',                     [0][1] string: 'Value of program execution mode - either standard_aircraft_design, or parameter_study, or optimization'
            *  [1][0] string: 'design_mode',                      [1][1] int:   'value to check which design mode is selected'
            *  [2][0] string: 'automatic_trim',                   [2][1] int:   'value to switch off and on the automatic trim function of the workflow'
            *  [3][0] string: 'convergence_criteria',             [3][1] float: 'value of convergence criteria of ome and mtom iteration'
            *  [4][0] string: 'number_of_max_iteration',          [4][1] int:   'value to set maximum number of design sizing iteration steps'
            *  [5][0] string: 'bridge_constraint_analyzer',       [5][1] int: 'value to switch off and on the execution of constraint_analysis module in workflow for each iteration step'
            *  [6][0] string: 'damp_mtom_iteration'               [6][1] bool:  'switch to activate damping of mtom iteration loop'
            *  [7][0] string: 'parallel_execution'                [7][1] int:   'switch to activate the parallel execution mode'
            *  [8][0] string: 'count_of_iteration',               [8][1] int:   'current number of iteration of design sizing loop'
            *  [9][0] string: 'count_of_mission_study_loop',      [9][1] int:   'current number of iteration of mission study loop'

        The output list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        The output list of lists "calibration_settings" contains the following values:
            *  [0][0] string: 'free_variable_OME_calibration',                   [0][1] int:   'switch for free variable OME calibration'
            *  [1][0] string: 'free_variable_MTOM_calibration',                  [1][1] int:   'switch for free variable MTOM calibration'
            *  [2][0] string: 'calibration_target_OME',                          [2][1] float: 'target value for OME calibration'
            *  [3][0] string: 'calibration_target_mtom',                         [3][1] float: 'target value for MTOM calibration'
            *  [4][0] string: 'reset_fuselage_mass_at_start',                    [4][1] int:   'switch to reset free OME calibration values at start'
            *  [5][0] string: 'reset_viscous_drag_counts_at_start',              [5][1] int:   'switch to reset aerodynamic MTOM calibration value at start'
            *  [6][0] string: 'reset_engine_fuel_flow_factor_at_start',          [6][1] int:   'switch to reset engine MTOM calibration value at start'
            *  [7][0] string: 'number_of_iterations_before_mtom_calibration',    [7][1] int:   'maximum number of OME calibration steps before performing a MTOM calibration'
            *  [8][0] string: 'max_iterations_before_exit_OME_calibration',      [8][1] int:   'maximum number of OME calibration steps before exit the calibration'
            *  [9][0] string: 'trunc_Error_factor',                              [9][1] int:   'value for termination criterion -> as factor regarding convergence criteria'
            * [10][0] string: 'damp_MTOM_calibration_lever',                    [10][1] int:   'switch for damping MTOM calibration lever arm'
            * [11][0] string: 'x_rel_max_for_MTOM_damp',                        [11][1] float: 'value of relative MTOM calibration lever arm'
            * [12][0] string: 'damp_OME_calibration_lever',                     [12][1] int:   'Switch for damping OME calibration lever arm'
            * [13][0] string: 'x_rel_max_for_OME_damp',                         [13][1] float: 'value of relative OME calibration lever arm'

        The output list of lists "module_list_of_sizing_loop" is a dynamic list of used sizing loop modules of current workflow execution:
            *  [:][0] string: 'sizing_loop_module',          [0][:] string: 'name of the used sizing loop module of current workflow execution'

        :param: paths_and_names: input list of lists
        :param: global_loop_counter: input integer
        :return: accuracy_list: output list of lists, control_settings: output list of lists, parameter_for_design_case: output list of lists,
                paths_and_names: output list of lists, calibration_settings: output list of lists, module_list_of_sizing_loop: list of lists
    """

    ''' imports for python '''
    import json
    import os
    import sys
    import shutil
    import time
    from datetime import datetime
    from inspect import currentframe, getframeinfo
    from sub_function.path_check import path_check
    from sub_function.read_xml_file import read_xml_file
    from sub_function.recursive_data_remove import recursive_data_remove
    from calibration.read_calibration_parameter import read_calibration_parameter
    from sub_function.select_engine_from_existing_project import select_engine_from_existing_project
    from sub_function.select_configuration_files_from_existing_project \
        import select_configuration_files_from_existing_project
    from sub_function.write_to_log_file import write_to_log_file

    ''' create list for log-file string '''
    log_file_list = ['',
                     '******************************************** set configuration parameter '
                     '********************************************']

    ''' read data for script execution '''
    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory = ([item for item in paths_and_names if 'path_of_working_directory' in item])[-1][-1]
    path_of_working_directory_rce = \
        ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]
    path_to_aircraft_projects = ([item for item in paths_and_names if 'path_to_aircraft_projects' in item])[-1][-1]
    path_to_software_tools = ([item for item in paths_and_names if 'path_to_software_tools' in item])[-1][-1]
    path_to_json_files = ([item for item in paths_and_names if 'path_to_json_files' in item])[-1][-1]

    ''' initialize local parameter '''
    function_name = getframeinfo(currentframe()).function

    ''' set accuracy condition '''
    accuracy_low = round(float(0.0001), 4)
    accuracy_medium = round(float(0.0000001), 7)
    accuracy_high = round(float(0.0000000001), 10)

    accuracy_list = [['accuracy_low', accuracy_low],
                     ['accuracy_medium', accuracy_medium],
                     ['accuracy_high', accuracy_high]]

    ''' open and read parameter from unicado_workflow_conf.xml configuration file '''
    frame_info = getframeinfo(currentframe())
    path = path_of_working_directory_rce + current_workflow_name + '/unicado_workflow_conf.xml'
    root_of_workflow_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'unicado_workflow',
                                                    function_name, frame_info.lineno, log_file_list,
                                                    current_workflow_name)

    # try to read configuration parameter from imported element tree
    try:
        frame_info = getframeinfo(currentframe())
        aircraft_exchange_file = root_of_workflow_tree.find("./control_settings/aircraft_exchange_file_name/value").text
        aircraft_project = aircraft_exchange_file[0:len(aircraft_exchange_file) - 4]
        aircraft_exchange_directory = root_of_workflow_tree.find(
            "./control_settings/aircraft_exchange_file_directory/value").text
        aircraft_exchange_directory = aircraft_exchange_directory.replace(os.sep, '/')
        path_to_project = aircraft_exchange_directory
        aircraft_type = path_to_project
        # check if the given aircraft type string have a "/" on his end
        # -> if true: -> cut the last character from string
        if aircraft_type[-1] == '/':
            aircraft_type = aircraft_type[:-1]
        # find the positions of the last "/" and "-" in given string
        ind = aircraft_type.rfind('/')
        ind2 = aircraft_type.rfind('-')
        # extract aircraft type of current project from given string
        aircraft_type = aircraft_type[ind+1:ind2]
        # check if the given project path starts with '..'
        # -> if true: -> cut the first to characters and set the correct path to project
        corrected_path_is_absolute, corrected_path = path_check(aircraft_exchange_directory)
        if corrected_path_is_absolute:
            path_to_project = corrected_path
        else:
            path_to_project = path_of_working_directory + corrected_path

        convert_mode_to_int_dict = {
            'mode_0' : int(0),
            'mode_1' : int(1),
            'mode_2' : int(2),
            'mode_3' : int(3),
            'mode_4' : int(4),
            'mode_5' : int(5)
        }

        ''' read global control settings '''
        plot_output_on = int(root_of_workflow_tree.find(
            "./control_settings/plot_output/enable/value").text.lower() in ['true', 'True', '1', 't'])
        switch_off_plots = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/iteration_settings/"
            "switch_off_plots/value").text.lower() in ['true', 'True', '1', 't'])
        report_output_on = int(root_of_workflow_tree.find(
            "./control_settings/report_output/value").text.lower() in ['true', 'True', '1', 't'])
        switch_off_reports = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/iteration_settings/"
            "switch_off_html_report/value").text.lower() in ['true', 'True', '1', 't'])
        tex_report_on = int(root_of_workflow_tree.find(
            "./control_settings/tex_report/value").text.lower() in ['true', 'True', '1', 't'])
        switch_off_tex = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/iteration_settings/"
            "switch_off_tex_report/value").text.lower() in ['true', 'True', '1', 't'])
        console_output_on = convert_mode_to_int_dict[root_of_workflow_tree.find(
                                                        "./control_settings/console_output/value").text]
        logfile_output_on = convert_mode_to_int_dict[root_of_workflow_tree.find(
                                                        "./control_settings/log_file_output/value").text]
        write_info_files = int(root_of_workflow_tree.find(
            "./control_settings/write_info_files/value").text.lower() in ['true', 'True', '1', 't'])
        write_info_files_only_at_last_step_of_iteration = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/iteration_settings/"
            "switch_off_info_files/value").text.lower() in ['true', 'True', '1', 't'])
        inkscape_path = root_of_workflow_tree.find(
            "./control_settings/inkscape_path/value").text
        gnuplot_path = root_of_workflow_tree.find(
            "./control_settings/gnuplot_path/value").text

        ''' read program-specific control settings '''
        # program specific pre_workflow_operation_settings
        set_to_default_before_run = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/pre_workflow_operation_settings/"
            "set_project_folder_to_default/value").text.lower() in ['true', 'True', '1', 't'])
        delete_plots_before_run = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/pre_workflow_operation_settings/"
            "delete_plots_before_run/value").text.lower() in ['true', 'True', '1', 't'])
        delete_engines_before_run = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/pre_workflow_operation_settings/"
            "delete_engines_before_run/value").text.lower() in ['true', 'True', '1', 't'])
        clear_old_log_files_at_start = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/pre_workflow_operation_settings/"
            "clear_old_log_files_before_run/value").text.lower() in ['true', 'True', '1', 't'])

        # program specific iteration_settings
        check_settings_for_design_logic = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/iteration_settings/"
            "check_settings_for_design_logic/value").text.lower() in ['true', 'True', '1', 't'])
        use_control_settings_for_sub_programs = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/iteration_settings/"
            "use_control_settings_for_subprograms/value").text.lower() in ['true', 'True', '1', 't'])
        use_range_type_specific_factors = convert_mode_to_int_dict[root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/iteration_settings/"
            "use_range_type_specific_factors/value").text]
        use_configs_from_existing_project = convert_mode_to_int_dict[root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/iteration_settings/"
            "use_configs_from_existing_project/config_copy_mode/value").text]
        use_engine_from_existing_project = convert_mode_to_int_dict[root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/iteration_settings/"
            "use_engine_from_existing_project/engine_copy_mode/value").text]
        save_acft_xml_after_each_iteration = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/iteration_settings/"
            "save_aircraft_exchange_files_to_result_folder/value").text.lower() in ['true', 'True', '1', 't'])

        # program specific post_workflow_operation_settings
        save_configs_to_result_folder = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/post_workflow_operation_settings/"
            "save_configs_to_result_folder/value").text.lower() in ['true', 'True', '1', 't'])
        save_log_files_to_result_folder = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/post_workflow_operation_settings/"
            "save_log_files_to_result_folder/value").text.lower() in ['true', 'True', '1', 't'])
        save_acft_xml_to_result_folder = int(root_of_workflow_tree.find(
            "./control_settings/program_specific_settings/post_workflow_operation_settings/"
            "save_final_aircraft_exchange_file_to_result_folder/value").text.lower() in ['true', 'True', '1', 't'])

        ''' create table for control settings '''
        control_settings = [['delete_plots_before_run', delete_plots_before_run],
                            ['delete_engines_before_run', delete_engines_before_run],
                            ['clear_old_log_files_at_start', clear_old_log_files_at_start],
                            ['plot_output_on', plot_output_on],
                            ['switch_off_plots', switch_off_plots],
                            ['report_output_on', report_output_on],
                            ['switch_off_reports', switch_off_reports],
                            ['tex_report_on', tex_report_on],
                            ['switch_off_tex', switch_off_tex],
                            ['status_flag_to_skip_design_sizing', int(0)],
                            ['status_flag_to_skip_study_mission', int(0)],
                            ['check_settings_for_design_logic', check_settings_for_design_logic],
                            ['use_control_settings_for_sub_programs', use_control_settings_for_sub_programs],
                            ['use_range_type_specific_factors', use_range_type_specific_factors],
                            ['use_configs_from_existing_project', use_configs_from_existing_project],
                            ['use_engine_from_existing_project', use_engine_from_existing_project],
                            ['save_configs_to_result_folder', save_configs_to_result_folder],
                            ['save_log_files_to_result_folder', save_log_files_to_result_folder],
                            ['save_acft_xml_to_result_folder', save_acft_xml_to_result_folder],
                            ['save_acft_xml_after_each_iteration', save_acft_xml_after_each_iteration],
                            ['write_info_files', write_info_files],
                            ['write_info_files_only_at_last_step_of_iteration',
                             write_info_files_only_at_last_step_of_iteration]]

        ''' read program specific settings - program execution mode '''
        # Read global program mode (either standard design / parameter_study / optimization)
        program_mode = convert_mode_to_int_dict[root_of_workflow_tree.find(
            "./program_settings/program_mode/value").text]
        if program_mode == 0:
            program_mode = "standard_aircraft_design"
        elif program_mode == 1:
            program_mode = "parameter_study"
        else:
            program_mode = "optimization"
        
        ''' read program specific settings - parameter for design case '''
        design_mode = convert_mode_to_int_dict[root_of_workflow_tree.find(
            "./program_settings/design_case_settings/design_mode/value").text]
        automatic_trim = int(0)
        convergence_criteria = float(root_of_workflow_tree.find(
            "./program_settings/design_case_settings/iteration_settings/"
            "convergence_criteria/value").text)
        number_of_max_iteration = int(root_of_workflow_tree.find(
            "./program_settings/design_case_settings/iteration_settings/"
            "max_number_of_iterations_before_exit/value").text)
        bridge_constraint_analyzer = int(root_of_workflow_tree.find(
            "./program_settings/design_case_settings/iteration_settings/"
            "bridge_constraint_analyzer/value").text.lower() in ['true', 'True', '1', 't'])
        damp_mtom_iteration = int(root_of_workflow_tree.find(
            "./program_settings/design_case_settings/iteration_settings/"
            "damp_MTOM_during_iteration/value").text.lower() in ['true', 'True', '1', 't'])
        parallel_execution = int(root_of_workflow_tree.find(
            "./program_settings/design_case_settings/parallel_execution").text.lower() in ['true', 'True', '1', 't'])

        ''' create table for design case parameter '''
        parameter_for_design_case = [['program_mode', program_mode],
                                     ['design_mode', design_mode],
                                     ['automatic_trim', automatic_trim],
                                     ['convergence_criteria', convergence_criteria],
                                     ['number_of_max_iteration', number_of_max_iteration],
                                     ['bridge_constraint_analyzer', bridge_constraint_analyzer],
                                     ['damp_mtom_iteration', damp_mtom_iteration],
                                     ['parallel_execution', parallel_execution],
                                     ['count_of_iteration', int(0)],
                                     ['count_of_mission_study_loop', int(0)]]

    # design case calibration
        calibration_settings = read_calibration_parameter(root_of_workflow_tree)

    # exception handling if read of configuration parameter failed -> raise an error and abort program
    except OSError:
        frame_info_end = getframeinfo(currentframe())
        print('Error in file: set_configuration_parameter.py \n In line: ' + str(frame_info.lineno) + ' to '
              + str(frame_info_end.lineno - 3) + ' \n Read configuration parameter from imported element tree failed!'
                                                 '  \n Program aborted!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'Error in file: set_configuration_parameter.py \n In line: ' + str(frame_info.lineno)
                             + ' to ' + str(frame_info_end.lineno - 3) +
                             ' \n Read configuration parameter from imported element tree failed!  \n Program aborted!')
        write_to_log_file(path_of_working_directory_rce, current_workflow_name, log_file_list)
        sys.exit(1)

    # execute operations for initial settings
    ''' create project folder if not exist '''
    module_list_of_sizing_loop = []
    # project directory does not exist
    if not os.path.isdir(path_to_project):
        if os.path.isdir(path_to_aircraft_projects + aircraft_type + '/' + aircraft_project):
            while_count = 0
            while True:
                try:
                    shutil.copytree(path_to_aircraft_projects + aircraft_type + '/' + aircraft_project, path_to_project)
                    print('Working copy of aircraft project ' + aircraft_project +
                          ' has been successfully created in the working directory!')
                    log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                         'Working copy of aircraft project ' + aircraft_project
                                         + ' has been successfully created in the working directory!')
                    break
                except OSError:
                    if while_count == 10:
                        print('Warning: Working copy of aircraft project ' + aircraft_project +
                              ' could not be created in the working directory!')
                        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                             'Warning: Working copy of aircraft project ' + aircraft_project +
                                             ' could not be created in the working directory!')
                        break
                    while_count += 1
                    print('Warning: Try to copy of aircraft project ' + aircraft_project +
                          ' to the temporary working directory, but no response from the system.')
                    print('Retry ' + str(while_count) + ' of 10!')
                    time.sleep(3)

        # repository folder not found -> project could not be created -> abort program
        else:
            print('Error: Working copy of aircraft project ' + aircraft_project
                  + ' could not be created, no repository folder found! \n Program aborted!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Error: Working copy of aircraft project ' + aircraft_project
                                 + ' could not be created, no repository folder found! \n Program aborted!')
            write_to_log_file(path_of_working_directory_rce, current_workflow_name, log_file_list)
            sys.exit(1)

    elif set_to_default_before_run:
        content_list_of_project_folder = os.listdir(path_to_project)
        starting_file = aircraft_project + '_start.xml'
        cpacs_file = aircraft_project + '_CPACS.xml'
        for element in content_list_of_project_folder:
            # delete current file from project folder
            element_flag = False
            while_count = 0
            while True:
                if os.path.isfile(path_to_project + element) and not element == starting_file \
                        and not element == cpacs_file:
                    try:
                        os.remove(path_to_project + element)
                        element_flag = True
                        break
                    except OSError:
                        if while_count == 10:
                            print('Warning: Reset of old project files failed. Delete operation is skipped')
                            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                                 'Warning: Reset of old project files failed. Delete operation '
                                                 'is skipped')
                            break
                        while_count += 1
                        print('Warning: Try to reset old project files, but no response from the system!')
                        print('Retry ' + str(while_count) + ' of 10!')
                        time.sleep(3)
                else:
                    break

            # delete current folder from project folder
            while_count = 0
            while True:
                if os.path.isdir(path_to_project + element) and not element == 'cleanSheetDesign' \
                        and not element == 'retrofitDesign':
                    try:
                        shutil.rmtree(path_to_project + element)
                        element_flag = True
                        break
                    except OSError:
                        if while_count == 10:
                            print('Warning: Delete of old project directories failed. Delete operation is skipped')
                            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                                 'Warning: Delete of old project directories failed. Delete operation '
                                                 'is skipped')
                            break
                        while_count += 1
                        print('Warning: Try to delete old project directories, but no response from the system!')
                        print('Retry ' + str(while_count) + ' of 10!')
                        time.sleep(3)
                else:
                    break

            if element_flag:
                print(element + ' has been successfully deleted from project folder!')
                log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': ' + element +
                                     ' has been successfully deleted from project folder!')

    else:
        print('Current aircraft design project is ' + aircraft_project + '!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'Current aircraft design project is ' + aircraft_project + '!')

    ''' copy project file to temporary workflow working directory of current execution '''
    while_count = 0
    directory_of_file_to_copy = str()
    while True:
        if not os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project):
            try:
                shutil.copytree(path_to_project, path_of_working_directory_rce + current_workflow_name + '/'
                                + aircraft_project)
                if design_mode == 2:
                    directory_of_file_to_copy = '/retrofit_design/without_calibration/'
                    if os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                                     + '/clean_sheet_design'):
                        shutil.rmtree(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                                      + '/clean_sheet_design')
                else:
                    directory_of_file_to_copy = '/clean_sheet_design/'
                    if os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                                     + '/retrofit_design'):
                        shutil.rmtree(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                                      + '/retrofit_design')
                ''' check if the the geometry_data directory inside of current workflow folder '''
                if not os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/'
                                     + aircraft_project + '/geometry_data/existing_component_geometries'):
                    print('Warning: The existing_component_geometries directory could not be found inside of the geometry_data folder of current project! \n'
                          '         The use_exisiting_geometry mode of the component sizing tools cannot be executed. Affected modules will raise an error.')
                    log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                     'Warning: The existing_component_geometries directory could not be found inside of the geometry_data folder of current project! \n'
                                     '                     The use_exisiting_geometry mode of the component sizing tools cannot be executed. Affected modules will raise an error.')

                print('Copy project file to temporary working directory successfully done!')
                log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                     'Copy project file to temporary working directory successfully done!')
                break
            except OSError:
                if while_count == 10:
                    print('Warning: Copy operation of project files to temporary working directory. '
                          'Copy operation is skipped.')
                    log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                         'Warning: Copy operation of project files to temporary working directory.'
                                         ' Copy operation is skipped.')
                    break
                while_count += 1
                print('Warning: Try to copy project files to temporary working directory')
                print('Retry ' + str(while_count) + ' of 10!')
                time.sleep(3)
        else:
            break

    # Copy operating company xml file to temporary working directory.
    if os.path.isfile(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                      + directory_of_file_to_copy + 'operating_company.xml'):
        shutil.copy(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                    + directory_of_file_to_copy + 'operating_company.xml',
                    path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                    + '/operating_company.xml')
    else:
        print('Warning: The file operating_company.xml could not be found, '
              'it is created by the Emissions module itself.')

    # Copy scenario xml file to temporary working directory.
    if os.path.isfile(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                      + directory_of_file_to_copy + 'scenario.xml'):
        shutil.copy(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                    + directory_of_file_to_copy + 'scenario.xml',
                    path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                    + '/scenario.xml')
    else:
        print('Warning: The file scenario.xml could not be found, '
              'it is created by the Emissions module itself.')

    ''' path to temporary aircraft exchange files of sizing loop '''
    path_of_aircraft_exchange_files = path_of_working_directory_rce + current_workflow_name \
        + '/temporaryResults/config_files/preExecution/'
    if not os.path.isdir(path_of_aircraft_exchange_files):
        os.makedirs(path_of_aircraft_exchange_files)
    # create results folder for configuration files
    if not os.path.isdir(path_of_aircraft_exchange_files + 'setup/'):
        os.mkdir(path_of_aircraft_exchange_files + 'setup/')
    if not os.path.isdir(path_of_aircraft_exchange_files + 'sizingLoop/'):
        os.mkdir(path_of_aircraft_exchange_files + 'sizingLoop/')
    if not os.path.isdir(path_of_aircraft_exchange_files + 'postProcessing/'):
        os.mkdir(path_of_aircraft_exchange_files + 'postProcessing/')

    # read tool list of used design modules
    status_flag = False
    list_of_json_tools = os.listdir(path_to_json_files)

    ''' check if configuration files from clean sheet folder or from calibrated engine folder are to be used '''
    # configuration files from clean sheet folder are to be used
    if not use_configs_from_existing_project == 0:
        # call function to use engine from existing project
        status_flag, log_file_list = select_configuration_files_from_existing_project(path_of_working_directory_rce,
                                                                                      path_of_working_directory,
                                                                                      path_to_aircraft_projects,
                                                                                      aircraft_project,
                                                                                      aircraft_type,
                                                                                      use_configs_from_existing_project,
                                                                                      log_file_list,
                                                                                      current_workflow_name)

    ''' create list of files in working directory '''
    files_in_working_directory = os.listdir(path_of_working_directory)
    # loop for all elements in list of files
    for file_name in files_in_working_directory:
        # set skipping flag of initial_sizing to false -> if retrofit mode is selected,
        #  the flag is set to true in the following lines of code to skip initial_sizing
        skip_initial_sizing_flag = False
        # check if current list element not 'workingDirectoryRCE' or 'workflowResults', then save configuration xml
        if not (file_name == 'workingDirectoryRCE') and not (file_name == 'workflowResults') \
            and not (file_name == 'projects'):
            # check if current list element is a directory
            if os.path.isdir(path_of_working_directory + file_name):
                xml_file_name = file_name + '/' + file_name + '_conf.xml'
                # check whether a configuration xml file exists within the current list element
                if os.path.isfile(path_of_working_directory + xml_file_name):
                    # copy tool configuration file to temporary working workflow directory of current execution
                    if not status_flag and global_loop_counter == 1:
                        while_count = 0
                        while True:
                            try:
                                shutil.copyfile(path_of_working_directory + xml_file_name,
                                                path_of_working_directory_rce + current_workflow_name + '/' + file_name
                                                + '_conf.xml')
                                if file_name == 'fuselage_design':
                                    shutil.copytree(path_of_working_directory + '/' + file_name + '/fuselage_design_lib',
                                                    path_of_working_directory_rce + current_workflow_name
                                                    + '/fuselage_design_lib')
                                if file_name == 'landing_gear_design':
                                    shutil.copytree(path_of_working_directory + '/' + file_name + '/landing_gear_lib',
                                                    path_of_working_directory_rce + current_workflow_name
                                                    + '/landing_gear_lib')
                                if file_name == 'aerodynamic_analysis':
                                    shutil.copyfile(path_of_working_directory + '/' + file_name + '/'
                                                    + 'LiftingLine/liftingLine_conf.xml',
                                                    path_of_working_directory_rce
                                                    + current_workflow_name
                                                    + '/' + 'liftingLine_conf.xml')
                                if file_name == 'cpacs_interface':
                                    shutil.copyfile(path_of_working_directory + '/' + file_name + '/'
                                                    + 'convertUNICADO2CPACS/convertUNICADO2CPACS_conf.xml',
                                                    path_of_working_directory_rce
                                                    + current_workflow_name
                                                    + '/' + 'convertUNICADO2CPACS_conf.xml')
                                break

                            # exception handling or copy of module configuration files
                            except OSError:
                                if while_count == 10:
                                    print('Warning: Copy of ' + file_name + ' configuration file to temporary working '
                                          'failed. Copy operation is skipped')
                                    log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                                         'Warning: Copy of ' + file_name + ' configuration file to '
                                                         'temporary working failed. Copy operation is skipped')
                                    break
                                while_count += 1
                                print('Warning: Try to copy ' + file_name + ' configuration file to temporary working '
                                      'directory')
                                print('Retry ' + str(while_count) + ' of 10!')
                                time.sleep(3)

                    # copy tool configuration file to config files/preExecution folder in temporary working workflow directory of current execution
                    module_list_of_sizing_loop.append(['sizing_loop_module', file_name])
                    # read group name of module from json file
                    for json_tool in list_of_json_tools:
                        if json_tool == file_name:
                            frame_info = getframeinfo(currentframe())
                            while_count = 0
                            while True:
                                if os.path.isfile(path_to_json_files + file_name + '/configuration.json'):
                                    try:
                                        with open(path_to_json_files + file_name + '/configuration.json', 'r') as file:
                                            json_tree = json.loads(file.read())
                                        group_name = json_tree['groupName']
                                        file.close()
                                        # copy current configuration file of list element to temporary working results
                                        if group_name == 'preSizing':
                                            if not design_mode == 2:
                                                shutil.copy(path_of_working_directory_rce + current_workflow_name + '/'
                                                            + file_name + '_conf.xml',
                                                            path_of_aircraft_exchange_files + 'setup/'
                                                            + file_name + '_conf.xml')
                                            else:
                                                if not file_name == 'initial_sizing':
                                                    shutil.copy(path_of_working_directory_rce + current_workflow_name
                                                                + '/' + file_name + '_conf.xml',
                                                                path_of_aircraft_exchange_files + 'setup/'
                                                                + file_name + '_conf.xml')
                                                else:
                                                    # the flag is set to true to skip initial_sizing
                                                    skip_initial_sizing_flag = True

                                        elif group_name == 'sizingLoop' or group_name == 'visualization':
                                            shutil.copy(path_of_working_directory_rce + current_workflow_name + '/'
                                                        + file_name + '_conf.xml',
                                                        path_of_aircraft_exchange_files + 'sizingLoop/'
                                                        + file_name + '_conf.xml')
                                            if file_name == 'aerodynamic_analysis':
                                                if os.path.isfile(path_of_working_directory_rce + current_workflow_name
                                                                  + '/liftingLine_conf.xml'):
                                                    shutil.copy(path_of_working_directory_rce + current_workflow_name
                                                                + '/liftingLine_conf.xml',
                                                                path_of_aircraft_exchange_files + 'sizingLoop/'
                                                                + 'liftingLine_conf.xml')
                                            if file_name == 'cpacs_interface':
                                                if os.path.isfile(path_of_working_directory_rce + current_workflow_name
                                                                  + '/convertUNICADO2CPACS_conf.xml'):
                                                    shutil.copy(path_of_working_directory_rce + current_workflow_name
                                                                + '/convertUNICADO2CPACS_conf.xml',
                                                                path_of_aircraft_exchange_files + 'sizingLoop/'
                                                                + 'convertUNICADO2CPACS_conf.xml')

                                        elif group_name == 'postProcessing':
                                            shutil.copy(path_of_working_directory_rce + current_workflow_name + '/'
                                                        + file_name + '_conf.xml',
                                                        path_of_aircraft_exchange_files + 'postProcessing/'
                                                        + file_name + '_conf.xml')
                                        break
                                    # exception handling or copy of module configuration files
                                    except OSError:
                                        if while_count == 10:
                                            print('Warning: Copy of ' + file_name + ' configuration file to temporary '
                                                  'working failed. Copy operation is skipped')
                                            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) +
                                                                 ': Warning: Copy of ' + file_name +
                                                                 ' configuration file to temporary working failed.'
                                                                 ' Copy operation is skipped')
                                            break
                                        while_count += 1
                                        print('Warning: Try to copy ' + file_name + ' configuration file to temporary '
                                              'working directory')
                                        print('Retry ' + str(while_count) + ' of 10!')
                                        time.sleep(3)

                                # Else condition: -> json file does not exist
                                #  -> raise an error and skip copy of current module configuration file
                                else:
                                    frame_info_end = getframeinfo(currentframe())
                                    print('Error in file: set_configuration_parameter.py \n In line: '
                                          + str(frame_info.lineno) + 'to ' + str(frame_info_end.lineno - 3) +
                                          ' \n Read configuration.json file of module: ' + file_name + ' failed! \n '
                                          + file_name + 'configuration file could not be copied to results!')
                                    log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                                                         + ': Error in file: set_configuration_parameter.py '
                                                           '\n In line: ' + str(frame_info.lineno) + 'to '
                                                         + str(frame_info_end.lineno - 3)
                                                         + ' \n  Read configuration.json file of module: ' + file_name
                                                         + ' failed! \n ' + file_name
                                                         + 'configuration file could not be copied to results!')
                                    write_to_log_file(path_of_working_directory_rce, current_workflow_name,
                                                      log_file_list)
                                    break

                    # check if retrofit design is selected
                    #  -> if true: -> skipp set parameter to initial_sizing configuration file
                    file_name_list = []
                    if not skip_initial_sizing_flag and not file_name == "cpacs_interface":
                        file_name_list.append(file_name)
                        if file_name == 'cpacs_interface':
                            file_name_list.append('convertUNICADO2CPACS')
                        for f_name in file_name_list:
                            # try to open module configuration file
                            frame_info = getframeinfo(currentframe())
                            path = path_of_working_directory_rce + current_workflow_name + '/' + f_name + '_conf.xml'
                            file_name_tree_root, xml_tree = read_xml_file(path, path_of_working_directory_rce, f_name,
                                                                          function_name, frame_info.lineno,
                                                                          log_file_list, current_workflow_name)

                            # set aircraftExchangeFile to aircraft_exchange_file_name/value and aircraftExchangeDirectory
                            # to aircraft_exchange_file_directory/value of configuration file of current list element
                            file_name_tree_root.find('./control_settings/aircraft_exchange_file_name/value').text = \
                            aircraft_exchange_file
                            file_name_tree_root.find('./control_settings/aircraft_exchange_file_directory/value').text = \
                                path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/'

                            # set paths for log files and plt files to module configuration files
                            file_name_tree_root.find('./control_settings/log_file/value').text = path_of_working_directory_rce \
                                + current_workflow_name + '/' + f_name + '.log'

                            # check if control settings for sub programs is true
                            if use_control_settings_for_sub_programs == 1:
                                convert_mode_to_int_dict = {
                                    "0" : 'mode_0',
                                    "1" : 'mode_1',
                                    "2" : 'mode_2',
                                    "3" : 'mode_3',
                                    "4" : 'mode_4',
                                    "5" : 'mode_5'
                                }

                                # set switches for report and plot generation for modules of sizing loops
                                file_name_tree_root.find('./control_settings/console_output/value').text = \
                                    str(convert_mode_to_int_dict[str(console_output_on)])
                                file_name_tree_root.find('./control_settings/log_file_output/value').text = \
                                    str(convert_mode_to_int_dict[str(logfile_output_on)])
                                file_name_tree_root.find('./control_settings/plot_output/enable/value').text = \
                                    str(bool(plot_output_on)).lower()
                                file_name_tree_root.find('./control_settings/report_output/value').text = \
                                    str(bool(report_output_on)).lower()
                                file_name_tree_root.find('./control_settings/tex_report/value').text = \
                                    str(bool(tex_report_on)).lower()

                                # check if write info files is enabled
                                #  -> if true: -> enable write info files in all submodules
                                if write_info_files == 1 and write_info_files_only_at_last_step_of_iteration == 0:
                                    file_name_tree_root.find('./control_settings/write_info_files/value').text = \
                                        str(bool(1)).lower()
                                elif write_info_files == 1 and write_info_files_only_at_last_step_of_iteration == 1:
                                    file_name_tree_root.find('./control_settings/write_info_files/value').text = \
                                        str(bool(0)).lower()
                                    if f_name == 'initial_sizing' \
                                            and write_info_files_only_at_last_step_of_iteration == 1:
                                        file_name_tree_root.find('./control_settings/write_info_files/value').text = \
                                            str(bool(1)).lower()
                                        print('Switch of WriteInfoFile only for last step of iteration in '
                                              + f_name + '_conf.xml successfully set to active!')
                                        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                                                             + ': Switch of WriteInfoFile only for last step of'
                                                               ' iteration in ' + f_name + '_conf.xml successfully set'
                                                               ' to active!')
                                elif write_info_files == 0:
                                    file_name_tree_root.find('./control_settings/write_info_files/value').text = \
                                        str(bool(0)).lower()
                                file_name_tree_root.find('./control_settings/inkscape_path/value').text = inkscape_path
                                file_name_tree_root.find('./control_settings/gnuplot_path/value').text = gnuplot_path

                            if f_name == 'cpacs_interface':
                                cpacs_file_name = aircraft_exchange_file[:-4] + '_CPACS.xml'
                                file_name_tree_root.find(
                                    './control_settings/program_specific/cpacs_filename').text = cpacs_file_name
                                file_name_tree_root.find(
                                    './control_settings/program_specific/cpacs_dir').text = path_of_working_directory_rce \
                                    + current_workflow_name + '/' + aircraft_project + '/'

                            # write settings to configuration xml file
                            xml_tree.write(path, encoding='utf-8')

                # check if old log-files should be cleared
                if clear_old_log_files_at_start == 1:
                    if os.path.isfile(path_of_working_directory_rce + current_workflow_name + '/' + file_name + '.log'):
                        os.remove(path_of_working_directory_rce + current_workflow_name + '/' + file_name + '.log')

    ''' check if old plots and reports should be deleted '''
    if delete_plots_before_run == 1:
        if os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/reporting'):
            recursive_data_remove(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                                  + '/reporting')
            print('Old reporting data has been successfully deleted!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Old reporting data has been successfully deleted!')

        if os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/plots'):
            recursive_data_remove(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                                  + '/plots')
            print('Old plots has been successfully deleted!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Old plots has been successfully deleted!')

        if os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                         + '/report_html'):
            recursive_data_remove(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                                  + '/report_html')
            print('Old html reports has been successfully deleted!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Old html reports has been successfully deleted!')

        if os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/report_tex'):
            recursive_data_remove(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                                  + '/report_tex')
            print('Old TeX reports has been successfully deleted!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Old TeX reports has been successfully deleted!')

        if os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                         + '/csvFilesForPlots'):
            recursive_data_remove(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                                  + '/csvFilesForPlots')
            print('Old csv files for plots has been successfully deleted!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Old csv files for plots has been successfully deleted!')

    ''' check if old engines should be deleted; this should not be done for design_mode 3'''
    if delete_engines_before_run == 1 and not design_mode == 3:
        if os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                         + '/engine_data'):
            recursive_data_remove(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                                  + '/engine_data')
            print('Old engine data has been successfully deleted!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Old engine data has been successfully deleted!')

    ''' check if engine from existing project should be used '''
    if not use_engine_from_existing_project == 0:
        # call function to use engine from existing project
        _, sparse_checkout_flag, folder_name, log_file_list = \
            select_engine_from_existing_project(path_of_working_directory_rce,
                                                path_to_aircraft_projects,
                                                aircraft_exchange_file,
                                                aircraft_type,
                                                aircraft_project,
                                                use_engine_from_existing_project,
                                                log_file_list,
                                                current_workflow_name)

        # set parameter of used engine to propulsion_design configuration file
        if not sparse_checkout_flag:
            print('Warning: Engine data could not found in ' + folder_name
                  + ' folder! \n Current engine data will be used!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                                 + 'Warning: Engine data could not found in ' + folder_name
                                 + ' folder! \n Current engine data will be used!')

    ''' extend table for paths and names '''
    paths_and_names.append(['aircraft_exchange_directory', aircraft_exchange_directory])
    paths_and_names.append(['aircraft_exchange_file', aircraft_exchange_file])
    paths_and_names.append(['aircraft_project', aircraft_project])
    paths_and_names.append(['path_to_project', path_to_project])
    paths_and_names.append(['aircraft_type', aircraft_type])

    print('All configuration parameters for the aircraft design process successfully set!')
    log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                         'All configuration parameters for the aircraft design process successfully set!')

    ''' write log-file to system '''
    log_file_list.append('****************************************** end set configuration parameter '
                         '******************************************')
    write_to_log_file(path_of_working_directory_rce, current_workflow_name, log_file_list)
        
    return accuracy_list, control_settings, parameter_for_design_case, paths_and_names, calibration_settings,\
        module_list_of_sizing_loop
