# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def set_range_type_specific_factors(paths_and_names, range_type_specific_factors, scenario_range, log_file_list):
    """ set_range_type_specific_factors sets scenario range dependent parameters to module configuration files.

        The input list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        The input elementTree "range_type_specific_factors" contains the data from range_type_specific_factors.xml file.

        The input string "scenario_range" contains the name of range scenario.

        The input list "log_file_list" contains the log file data from caller function

        :param: paths_and_names: input list of lists
        :param: range_type_specific_factors: input elementTree
        :param: scenario_range: input string
        :param: log_file_list: input list
        :return: status_flag: output bool, log_file_list: output list
    """

    ''' imports for python '''
    from datetime import datetime
    from inspect import currentframe, getframeinfo
    from sub_function.read_xml_file import read_xml_file

    ''' read data for script execution '''
    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory_rce =\
        ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]

    ''' initialize local parameter'''
    function_name = getframeinfo(currentframe()).function

    ''' loop across all elements in range_type_specific_factors elementTree to read and set module config parameter '''
    for elements in range_type_specific_factors:
        ''' initialize private loop parameter '''
        i = 1
        name_of_module = str(elements.tag)
        number_of_factors = int(range_type_specific_factors.find('./' + name_of_module + '/number_of_factors').text)
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': Set ' + str(number_of_factors)
                             + ' range dependent parameter in module: ' + name_of_module)
        print('Set ' + str(number_of_factors) + ' range dependent parameter in module: ' + name_of_module)

        ''' call function to read module configuration file as elementTree '''
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/' + name_of_module + '_conf.xml'
        root_of_design_tool, xml_tree = read_xml_file(path, path_of_working_directory_rce,
                                                      name_of_module, function_name, frame_info.lineno,
                                                      log_file_list, current_workflow_name)

        ''' loop across all factors of current design module '''
        while i <= number_of_factors:
            # read current path of parameter from range type specific factors xml-file
            path_to_parameter = str(range_type_specific_factors.find("./" + name_of_module + "/factor[" + "@ID='"
                                                                     + str(i) + "'" + "]/path").text)

            # check if the current path of parameter starts with a slash
            index_of_slash = path_to_parameter.find('/')
            if index_of_slash == 0:
                path_to_parameter = path_to_parameter[1:]

            # check if the string 'module_configuration_file' exist in current path of parameter
            index_of_cfg = path_to_parameter.find('module_configuration_file')
            if index_of_cfg > -1:
                path_to_parameter = path_to_parameter[index_of_cfg + 11:]

            # check if the string '@' exist in current path of parameter
            index_of_at = path_to_parameter.find('@')
            if index_of_at > -1:
                path_part_one = path_to_parameter[:index_of_at]
                path_part_two = path_to_parameter[index_of_at:]

                # check if the string '=' exist in current path of parameter
                index_of_equal = path_part_two.find('=')
                if index_of_equal > -1:
                    # check if the string '/' exist in the second part of path of parameter
                    index_of_new_slash = path_part_two.find('/')
                    if index_of_new_slash > -1:
                        value_of_id = path_part_two[index_of_equal+1:index_of_new_slash]
                        path_part_tree = path_part_two[index_of_new_slash:]
                        path_to_parameter = './' + path_part_one + '[' + path_part_two[:index_of_equal+1] + "'" \
                                            + str(value_of_id) + "'" + ']' + path_part_tree

                    else:
                        value_of_id = path_part_two[index_of_equal+1:]
                        path_to_parameter = './' + path_part_one + '[' + path_part_two[:index_of_equal+1] + "'" \
                                            + str(value_of_id) + "'" + ']'

            log_file_list.append('Path to parameter: ' + path_to_parameter)
            print('Path to parameter: ' + path_to_parameter)

            # read range type specific factor of current module from elementTree
            value_of_cfg_file = str(range_type_specific_factors.find("./" + name_of_module + "/factor[" + "@ID='"
                                                                     + str(i) + "'" + "]/value/" + scenario_range).text)

            # set range type specific factor to module configuration file
            root_of_design_tool.find(path_to_parameter).text = str(value_of_cfg_file)
            xml_tree.write(path_of_working_directory_rce + current_workflow_name + '/'
                           + name_of_module + '_conf.xml', encoding='utf-8')

            log_file_list.append('Set parameter value to: ' + str(value_of_cfg_file))
            print('Set parameter value to: ' + str(value_of_cfg_file))

            ''' add +1 to iteration counter '''
            i += 1

        log_file_list.append(' ')
        print('')

    return log_file_list
