# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def auto_update_additional_software(path_of_working_directory, path_to_origin, file_name, config_back_up_flag,
                                    log_file_list):
    """ Function to auto update the additional software tools in the UNICADO working directory.

        The input string "path_of_working_directory" contains the absolute path to installed working directory of
         UNICADO.

        The input string "path_to_origin" contains the absolute system path to the repository of additional software.

        The input string "fileName" contains the folder name of aircraft projects.

        The input bool "config_back_up_flag" contains the status whether the configuration files of the modules have to
         be backup.

        The input list "log_file_list" contains all system prints of function "set_configuration_parameter.py".

        The output list "log_file_list" contains all system prints of function "set_configuration_parameter.py" and
         "auto_update_additional_software.py".

        :param: path_of_working_directory: input string
        :param: path_to_origin: input string
        :param: file_name: input string
        :param: config_back_up_flag: input bool
        :param: log_file_list: input list
        :return: log_file_list: output list
    """

    ''' imports for python '''
    import os
    import shutil
    from datetime import datetime

    # check for origin tool data and update only this files in working directory
    if os.path.isdir(path_to_origin + file_name):
        files_in_origin_tool_directory = os.listdir(path_to_origin + file_name)

        for data in files_in_origin_tool_directory:
            # check if data a file, then replace with file from origin tool directory
            if os.path.isfile(path_of_working_directory + file_name + '/' + data):
                # check if current file the module configuration file and switch for back-up of configuration files
                # is true, then create back-up copy in working directory of module
                if data == file_name + '_conf.xml' and config_back_up_flag:
                    shutil.copyfile(path_of_working_directory + file_name + '/' + data,
                                    path_of_working_directory + file_name + '/' + file_name + '_backUp_conf.xml')

                os.remove(path_of_working_directory + file_name + '/' + data)
                shutil.copyfile(path_to_origin + file_name + '/' + data,
                                path_of_working_directory + file_name + '/' + data)

            # check if data a directory, then replace with directory from origin tool directory
            if os.path.isdir(path_of_working_directory + file_name + '/' + data):
                shutil.rmtree(path_of_working_directory + file_name + '/' + data)
                # check if data is not equal to src or convertUNICADO2CPACS
                # -> if true: -> copy data from origin to working directory
                if not data == 'src' and not data == 'convertUNICADO2CPACS':
                    shutil.copytree(path_to_origin + file_name + '/' + data,
                                    path_of_working_directory + file_name + '/' + data)
                # check if data is equal to convertUNICADO2CPACS
                # -> if true: -> copy all content inside the origin directory out of the src directory to working copy
                if data == 'convertUNICADO2CPACS':
                    data_inside = os.listdir(path_to_origin + file_name + '/' + data)
                    os.mkdir(path_of_working_directory + file_name + '/' + data)
                    for element in data_inside:
                        if not element == 'src':
                            if os.path.isfile(path_to_origin + file_name + '/' + data + '/' + element):
                                shutil.copyfile(path_to_origin + file_name + '/' + data + '/' + element,
                                                path_of_working_directory + file_name + '/' + data + '/' + element)
                            else:
                                shutil.copytree(path_to_origin + file_name + '/' + data + '/' + element,
                                                path_of_working_directory + file_name + '/' + data + '/' + element)

            # check if data exist in working directory, if not, then copy data to working directory
            if not os.path.exists(path_of_working_directory + file_name + '/' + data):
                if os.path.isfile(path_to_origin + file_name + '/' + data):
                    shutil.copyfile(path_to_origin + file_name + '/' + data,
                                    path_of_working_directory + file_name + '/' + data)
                # check if data a directory, then replace with directory from origin tool directory
                if os.path.isdir(path_to_origin + file_name + '/' + data):
                    # check if data is not equal to src or convertUNICADO2CPACS
                    # -> if true: -> copy data from origin to working directory
                    if not data == 'src' and not data == 'convertUNICADO2CPACS':
                        shutil.copytree(path_to_origin + file_name + '/' + data,
                                        path_of_working_directory + file_name + '/' + data)
                    # check if data is equal to convertUNICADO2CPACS
                    # -> if true: -> copy all content inside the origin directory out of the src folder to working copy
                    if data == 'convertUNICADO2CPACS':
                        data_inside = os.listdir(path_to_origin + file_name + '/' + data)
                        os.mkdir(path_of_working_directory + file_name + '/' + data)
                        for element in data_inside:
                            if not element == 'src':
                                if os.path.isfile(path_to_origin + file_name + '/' + data + '/' + element):
                                    shutil.copyfile(path_to_origin + file_name + '/' + data + '/' + element,
                                                    path_of_working_directory + file_name + '/' + data + '/' + element)
                                else:
                                    shutil.copytree(path_to_origin + file_name + '/' + data + '/' + element,
                                                    path_of_working_directory + file_name + '/' + data + '/' + element)

        print('Working copy of ' + file_name + ' has been successfully updated!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'Working copy of ' + file_name + ' has been successfully updated!')

    return log_file_list
