# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def prepare_single_mission_analysis(path_of_working_directory_rce, aircraft_project, file_name, log_file_list,
                                    current_workflow_name='UNICADOworkflow'):
    """ prepare_single_mission_analysis checks if all necessary files exist to perform a study mission analysis without
         design sizing loop.

        The input string path_of_working_directory_rce contains the system path to the working directory of rce.

        The input string aircraft_project contains the name of the current aircraft project.

        The input string file_name contains the name to the aircraft exchange file of current project.

        The input list log_file_list contains the strings of workflow log-file from the caller function.

        The input string "current_workflow_name" contains the name of the current workflow execution.
            * If no input is given, default sting 'UNICADOworkflow' is used.

        The output list log_file_list contains the strings of workflow log-file from the caller function and the added
         strings from this function.

        :param: path_of_working_directory_rce: input string
        :param: aircraft_project: input string
        :param: file_name: input string
        :param: log_file_list: input list
        :param: current_workflow_name: input string
        :return: log_file_list: output list
    """

    ''' imports for python '''
    import os
    import sys
    import shutil
    from inspect import currentframe, getframeinfo
    from datetime import datetime
    from sub_function.read_xml_file import read_xml_file
    from sub_function.write_to_log_file import write_to_log_file

    ''' initialize local parameter '''
    error_flag = False
    error_string = str()
    function_name = getframeinfo(currentframe()).function

    ''' check if converged aircraft exchange file exist -> if false: -> raise an error and abort program '''
    if not os.path.isfile(path_of_working_directory_rce + current_workflow_name
                          + '/' + aircraft_project + '/' + file_name):
        if os.path.isfile(path_of_working_directory_rce + current_workflow_name
                          + '/' + aircraft_project + '/cleanSheetDesign/' + file_name):
            shutil.copyfile(path_of_working_directory_rce + current_workflow_name
                            + '/' + aircraft_project + '/cleanSheetDesign/' + file_name,
                            path_of_working_directory_rce + current_workflow_name
                            + '/' + aircraft_project + '/' + file_name)

            print('Warning: No converged ' + file_name + ' in temporary working directory found!  \n '
                  '        Converged ' + file_name + ' from clean sheet design folder is used!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Warning: No converged '
                                 + file_name + ' in temporary working directory found! \n '
                                 '        Converged ' + file_name + 'from clean sheet design folder is used!')
        else:
            error_string = 'Error: Converged ' + file_name \
                           + ' file not available! Study mission analysis without design sizing ' \
                             'loop cannot be performed! \n Program aborted!'
            error_flag = True

    # check if the necessary geometry data exist in the temporary working directory
    if not os.path.isdir(path_of_working_directory_rce + current_workflow_name
                         + '/' + aircraft_project + '/geometry_data/'):
        if os.path.isdir(path_of_working_directory_rce + current_workflow_name
                         + '/' + aircraft_project + '/cleanSheetDesign/geometry_data/'):
            shutil.copytree(path_of_working_directory_rce + current_workflow_name
                            + '/' + aircraft_project + '/cleanSheetDesign/geometry_data/',
                            path_of_working_directory_rce + current_workflow_name
                            + '/' + aircraft_project + '/geometry_data/')
            print('Warning: No geometry data in temporary working directory found!  \n '
                  '        Geometry data from clean sheet design folder is used!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Warning: No geometry data in temporary working directory found!  \n '
                                 '        Geometry data from clean sheet design folder is used!')
        else:
            error_string = 'Error: No geometry data in temporary working directory found! ' \
                           'Study mission analysis without design sizing loop cannot be performed! \n Program aborted!'
            error_flag = True

    # check if the necessary aero data exist in the temporary working directory
    if not os.path.isdir(path_of_working_directory_rce + current_workflow_name
                         + '/' + aircraft_project + '/aero_data/'):
        if os.path.isdir(path_of_working_directory_rce + current_workflow_name
                         + '/' + aircraft_project + '/cleanSheetDesign/aero_data/'):
            shutil.copytree(path_of_working_directory_rce + current_workflow_name
                            + '/' + aircraft_project + '/cleanSheetDesign/aero_data/',
                            path_of_working_directory_rce + current_workflow_name
                            + '/' + aircraft_project + '/aero_data/')
            print('Warning: No aero data in temporary working directory found!  \n '
                  '        Aero data from clean sheet design folder is used!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Warning: No aero data in temporary working directory found!  \n '
                                 '        Aero data from clean sheet design folder is used!')
        else:
            error_string = 'Error: No aero data in temporary working directory found! ' \
                           'Study mission analysis without design sizing loop cannot be performed! \n Program aborted!'
            error_flag = True

    # check if the necessary engine data exist in the temporary working directory
    if not os.path.isdir(path_of_working_directory_rce + current_workflow_name
                         + '/' + aircraft_project + '/engine_data/'):
        if os.path.isdir(path_of_working_directory_rce + current_workflow_name
                         + '/' + aircraft_project + '/cleanSheetDesign/engine_data/'):
            shutil.copytree(path_of_working_directory_rce + current_workflow_name
                            + '/' + aircraft_project + '/cleanSheetDesign/engine_data/',
                            path_of_working_directory_rce + current_workflow_name
                            + '/' + aircraft_project + '/engine_data/')
            print('Warning: No engine data in temporary working directory found!  \n '
                  '        Engine data from clean sheet design folder is used!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Warning: No engine data in temporary working directory found!  \n '
                                 '        Engine data from clean sheet design folder is used!')
        else:
            error_string = 'Error: No engine data in temporary working directory found! ' \
                           'Study mission analysis without design sizing loop cannot be performed! \n Program aborted!'
            error_flag = True

    if error_flag:
        print(error_string)
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': ' + error_string)
        write_to_log_file(path_of_working_directory_rce, current_workflow_name, log_file_list)
        sys.exit(1)

    ''' read engine name from converged aircraft exchange file '''
    frame_info = getframeinfo(currentframe())
    path = path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/' + file_name
    root_of_aircraft_exchange_file, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'wing_design',
                                                             function_name, frame_info.lineno, log_file_list,
                                                             current_workflow_name)

    # Assumption that all propulsion systems use the same engine model
    engine_name = root_of_aircraft_exchange_file.find('./component_design/propulsion/specific/propulsion/engine/model'
                                                        '/value').text

    # check if engine data exist -> if false:
    # -> try to copy engine data from cleanSheetDesign folder otherwise raise an error and abort program
    if not os.path.isdir(path_of_working_directory_rce + current_workflow_name
                         + '/' + aircraft_project + '/' + 'engine_data/' + engine_name):
        try:
            if not os.path.isdir(path_of_working_directory_rce + current_workflow_name
                                 + '/' + aircraft_project + '/' + 'engine_data'):
                os.mkdir(path_of_working_directory_rce + current_workflow_name
                         + '/' + aircraft_project + '/' + 'engine_data')
            shutil.copytree(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/'
                            + 'cleanSheetDesign/engine_data/' + engine_name,
                            path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project + '/'
                            + 'engine_data/' + engine_name)
            print('Necessary engine data for ' + engine_name + ' successfully copied from "cleanSheetDesign" folder!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + 'Necessary engine data for '
                                 + engine_name + ' successfully copied from "cleanSheetDesign" folder!')

        except OSError:
            print('Error: Necessary engine data for ' + engine_name + ' not available! '
                  'Study mission analysis without design sizing loop cannot be performed! \n Program aborted!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                 'Error: Converged ' + engine_name + ' file not available! '
                                 'Study mission analysis without design sizing loop cannot be performed! \n '
                                 'Program aborted!')
            write_to_log_file(path_of_working_directory_rce, current_workflow_name, log_file_list)
            sys.exit(1)

    return log_file_list
