# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def select_configuration_files_from_existing_project(path_of_working_directory_rce, path_of_working_directory,
                                                     path_to_aircraft_projects, aircraft_project, aircraft_type,
                                                     use_configs_from_existing_project, log_file_list,
                                                     current_workflow_name='UNICADOworkflow'):
    """ select_configuration_files_from_existing_project prepare data to apply used configuration files from given project sources.
            Returns an output flag for following script execution.

        The input string "path_of_working_directory_rce" contains the system path to working directory of rce.

        The input string "path_of_working_directory" contains the system path to working directory of UNICADO.

        The input string "path_to_aircraft_projects" contains the system path to aircraft reference repository.

        The input string "aircraft_project" contains the name of the aircraft exchange file.

        The input string "aircraft_type" contains the name of the aircraft type of current project.

        The input integer "use_configs_from_existing_project" contains the number of source-location of existing engine.

        The input list "log_file_list" contains all system prints of function: set_configuration_parameter.py.

        The input string "current_workflow_name" contains the name of the current workflow execution.
            * If no input is given, default sting 'UNICADOworkflow' is used.

        The output list "log_file_list" contains all system prints of functions: set_configuration_parameter.py and select_engine_from_existing_project.py.

        :param: path_of_working_directory_rce: input string
        :param: path_of_working_directory: input string
        :param: path_to_aircraft_projects: input string
        :param: aircraft_project: input string
        :param: aircraft_type: input string
        :param: use_configs_from_existing_project: input integer
        :param: log_file_list: input list
        :param: current_workflow_name: input string
        :return: check_flag_xml_in_clean_sheet_folder: output bool, log_file_list: output list
    """

    ''' imports for python '''
    import os
    import shutil
    from datetime import datetime
    from inspect import currentframe, getframeinfo
    from read_xml_file import read_xml_file
    from recursive_data_remove import recursive_data_remove
    from sparse_checkout_from_git import sparse_checkout_from_git

    function_name = getframeinfo(currentframe()).function

    print_flag = True
    sparse_checkout_flag = False
    folder_name = str()
    path_to_config = str()

    list_of_working_directory_content = os.listdir(path_of_working_directory)
    check_flag_xml_in_clean_sheet_folder = str()

    ''' prepare data for selected config copy mode '''
    # Use configuration file from clean sheet design folder
    if use_configs_from_existing_project == 1:
        path_to_config = path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project \
            + '/cleanSheetDesign/config_files/'
        check_flag_xml_in_clean_sheet_folder =\
            os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                          + '/cleanSheetDesign/config_files/')
        folder_name = 'cleanSheetDesign'

    # Use configuration file from engine calibrated design folder
    if use_configs_from_existing_project == 2:
        path_to_config = path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project \
                         + '/retrofitDesign/engineCalibration/config_files/'
        check_flag_xml_in_clean_sheet_folder = \
            os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                          + '/retrofitDesign/engineCalibration/config_files/')
        folder_name = 'engineCalibration'

    # Use configuration file from aero calibrated design folder
    if use_configs_from_existing_project == 3:
        path_to_config = path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project \
                         + '/retrofitDesign/aerodynamicCalibration/config_files/'
        check_flag_xml_in_clean_sheet_folder = \
            os.path.isdir(path_of_working_directory_rce + current_workflow_name + '/' + aircraft_project
                          + '/retrofitDesign/aerodynamicCalibration/config_files/')
        folder_name = 'aerodynamicCalibration'

    # Use configuration file from repository folder
    if use_configs_from_existing_project == 4:
        # try to open module configuration file
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/unicado_workflow_conf.xml'
        root_of_workflow_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'unicado_workflow',
                                                        function_name, frame_info.lineno, log_file_list,
                                                        current_workflow_name)
        name_of_checkout_dir_from_repository = str(root_of_workflow_tree.find(
            './control_settings/program_specific_settings/iteration_settings/use_configs_from_existing_project/'
            'mode_specific_settings/mode_4/name_of_config_checkout_dir/value').text)
        # Check if the given input directory not equal to 'cleanSheetDesign' -> if true: -> retrofit design will be used
        if name_of_checkout_dir_from_repository == 'aerodynamicCalibration' \
                or name_of_checkout_dir_from_repository == 'engineCalibration' \
                or name_of_checkout_dir_from_repository == 'withoutCalibration':
            path_to_config = path_to_aircraft_projects + aircraft_type + '/' + aircraft_project + '/retrofitDesign/' +\
                             name_of_checkout_dir_from_repository + '/config_files/'
            check_flag_xml_in_clean_sheet_folder = os.path.isdir(path_to_config)
        else:
            path_to_config = path_to_aircraft_projects + aircraft_type + '/' + aircraft_project + '/' \
                             + name_of_checkout_dir_from_repository + '/config_files/'
            check_flag_xml_in_clean_sheet_folder = os.path.isdir(path_to_config)
        folder_name = 'local repository'

    # Use configuration file from repository folder
    if use_configs_from_existing_project == 5:
        # try to open module configuration file
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/unicado_workflow_conf.xml'
        root_of_workflow_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'unicado_workflow',
                                                        function_name, frame_info.lineno, log_file_list,
                                                        current_workflow_name)

        # set server information for sparse-checkout
        if not os.path.isdir(path_of_working_directory_rce + '/' + current_workflow_name + '/' + aircraft_project
                             + '/temporaryData'):
            os.mkdir(path_of_working_directory_rce + '/' + current_workflow_name + '/' + aircraft_project
                     + '/temporaryData')
        path_to_local_repository = path_of_working_directory_rce + '/' + current_workflow_name + '/' + aircraft_project\
            + '/temporaryData'
        repository_name = "rAIRCRAFTREFERENCES"
        branch = str(root_of_workflow_tree.find(
            './control_settings/program_specific_settings/iteration_settings/use_configs_from_existing_project/'
            'mode_specific_settings/mode_5/config_repository_branch/value').text)
        name_of_checkout_dir_from_server = str(root_of_workflow_tree.find(
            './control_settings/program_specific_settings/iteration_settings/use_configs_from_existing_project/'
            'mode_specific_settings/mode_4/name_of_config_checkout_dir/value').text)
        ssh_url = 'ssh://git@unicado.ilr.rwth-aachen.de:2222/source/' + repository_name + '.git'

        if name_of_checkout_dir_from_server == 'cleanSheetDesign':
            name_of_checkout_dir_from_server = aircraft_type + '/' + aircraft_project + '/cleanSheetDesign/config_files'
        else:
            name_of_checkout_dir_from_server = aircraft_type + '/' + aircraft_project + '/retrofitDesign/' \
                                               + name_of_checkout_dir_from_server + '/config_files'

        # call function for sparse-checkout from given ssh-git repository
        sparse_checkout_flag = sparse_checkout_from_git(path_to_local_repository, ssh_url, branch,
                                                        name_of_checkout_dir_from_server)

        path_to_config = path_of_working_directory_rce + '/' + current_workflow_name + '/' + aircraft_project \
            + '/temporaryData/config_files/'
        check_flag_xml_in_clean_sheet_folder = os.path.isdir(path_to_config)
        folder_name = 'ssh-server'

        if not sparse_checkout_flag:
            check_flag_xml_in_clean_sheet_folder = False
            print_flag = False
            print('Error: Configuration files could not checked out from ssh-server! \n '
                  'Be sure that you have necessary access rights to checkout from repository! \n '
                  'If you have, please check if the folder to checkout from git repository exist! \n '
                  'Current configuration files will be used!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) +
                                 ': Error: Configuration files could not checked out from ssh-server! \n '
                                 'Be sure that you have necessary access rights to checkout from repository! \n '
                                 'If you have, please check if the folder to checkout from git repository exist! \n '
                                 'Current configuration files will be used!')

    # check if current selected folder of configuration files exist
    #  -> if true: -> copy configuration files to tool directories
    if check_flag_xml_in_clean_sheet_folder:
        list_of_directory_content = os.listdir(path_to_config)
        for element in list_of_directory_content:
            if os.path.isdir(path_to_config + element) and not element == 'studyLoop' \
                    and not (element == 'preExecution'):
                list_of_sub_folder_content = os.listdir(path_to_config + element)
                for sub_element in list_of_sub_folder_content:
                    # extract the tool name from current sub_element string -> toolName_conf.xml[:-9] returns toolName
                    tool_name = sub_element[:-9]
                    for content in list_of_working_directory_content:
                        if content == tool_name:
                            if os.path.isdir(path_of_working_directory + content):
                                if content == 'aerodynamic_assessment':
                                    print(path_to_config + element + '/liftingLine_conf.xml')
                                    if os.path.isfile(path_to_config + element + '/liftingLine_conf.xml'):
                                        shutil.copyfile(path_to_config + element + '/liftingLine_conf.xml',
                                                        path_of_working_directory_rce + '/' + current_workflow_name
                                                        + '/liftingLine_conf.xml')
                                        print('Replacement of liftingLine configuration file from '
                                              + folder_name + ' folder successfully done!')
                                        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                                                             + ': Replacement of liftingLine configuration file from '
                                                             + folder_name + ' folder successfully done!')

                                if content == 'cpacsInterface':
                                    print(path_to_config + element + '/convertUNICADO2CPACS_conf.xml')
                                    if os.path.isfile(path_to_config + element + '/convertUNICADO2CPACS_conf.xml'):
                                        shutil.copyfile(path_to_config + element + '/convertUNICADO2CPACS_conf.xml',
                                                        path_of_working_directory_rce + '/' + current_workflow_name
                                                        + '/convertUNICADO2CPACS_conf.xml')
                                        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
                                                             + ': Replacement of convertUNICADO2CPACS configuration '
                                                               'file from ' + folder_name
                                                             + ' folder successfully done!')

                                shutil.copyfile(path_to_config + element + '/' + sub_element,
                                                path_of_working_directory_rce + '/' + current_workflow_name + '/'
                                                + sub_element)
                                print('Replacement of ' + tool_name + ' configuration file from ' + folder_name
                                      + ' folder successfully done!')
                                log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                                                     'Replacement of ' + tool_name + ' configuration file from '
                                                     + folder_name + ' folder successfully done!')

    elif not check_flag_xml_in_clean_sheet_folder and print_flag:
        print('Warning: Module configuration files could not found in ' + folder_name
              + ' folder! \n Current module configuration files will be used!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + 'Warning: '
                             'Module configuration files could not found in ' + folder_name + ' folder! \n '
                             'Current module configuration files will be used!')

    # remove temporary ssh-data
    if sparse_checkout_flag:
        recursive_data_remove(path_of_working_directory_rce + '/' + current_workflow_name + '/' + aircraft_project
                              + '/temporaryData')

    return check_flag_xml_in_clean_sheet_folder, log_file_list
