# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def select_engine_from_existing_project(path_of_working_directory_rce, path_to_aircraft_projects,
                                        aircraft_exchange_file, aircraft_type, aircraft_project,
                                        use_engine_from_existing_project, log_file_list,
                                        current_workflow_name='UNICADOworkflow'):
    """ select_engine_from_existing_project prepare data to apply used engines from given project sources.
        Returns an output flag for following script execution.

        The input string "path_of_working_directory_rce" contains the system path to working directory of rce.

        The input string "path_to_aircraft_projects" contains the system path to aircraft reference repository.

        The input string "aircraft_exchange_file" contains the name of the aircraft exchange file.

        The input string "aircraft_type" contains the name of the aircraft type of current project.

        The input string "aircraft_project" contains the name of the aircraft exchange file.

        The input integer "use_engine_from_existing_project" contains the number of source-location of existing engine.

        The input list "log_file_list" contains all system prints of function: set_configuration_parameter.py.

        The input string "current_workflow_name" contains the name of the current workflow execution.
            * If no input is given, default string 'UNICADOworkflow' is used.

        The output bool "engine_flag" contains the status of used engine copy.

        The output string "folder_name" contains the folder name of which engine where copied.

        The output list "log_file_list" contains all system prints of functions: set_configuration_parameter.py and select_engine_from_existing_project.py.

        :param: path_of_working_directory_rce: input string
        :param: path_to_aircraft_projects: input string
        :param: aircraft_exchange_file: input string
        :param: aircraft_type: input string
        :param: aircraft_project: input string
        :param: use_engine_from_existing_project: input integer
        :param: log_file_list: input list
        :param: current_workflow_name: input string
        :return: engine_flag: output bool, sparse_checkout_flag: output bool, folder_name: output string, log_file_list: output list
    """

    ''' imports for python '''
    import os
    import shutil
    from datetime import datetime
    from inspect import currentframe, getframeinfo
    from read_xml_file import read_xml_file
    from recursive_data_remove import recursive_data_remove
    from sparse_checkout_from_git import sparse_checkout_from_git

    function_name = getframeinfo(currentframe()).function

    engine_flag = False
    check_flag_engine_in_folder = False
    sparse_checkout_flag = False
    folder_name = str()
    source = str()

    ''' delete existing engine_data and all sub data '''
    if os.path.isdir(path_of_working_directory_rce + '/' + current_workflow_name + '/' + aircraft_project
                     + '/engine_data/'):
        recursive_data_remove(path_of_working_directory_rce + '/' + current_workflow_name + '/' + aircraft_project
                              + '/engine_data/')

    # use engine from clean sheet design folder
    if use_engine_from_existing_project == 1:
        check_flag_engine_in_folder = os.path.isdir(path_of_working_directory_rce + '/' + current_workflow_name + '/'
                                                    + aircraft_project + '/cleanSheetDesign/engine_data/')
        source = path_of_working_directory_rce + '/' + current_workflow_name + '/' + aircraft_project \
            + '/cleanSheetDesign/engine_data/'
        folder_name = 'cleanSheetDesign'

    # use engine from engine calibrated design folder
    if use_engine_from_existing_project == 2:
        check_flag_engine_in_folder = os.path.isdir(path_of_working_directory_rce + '/' + current_workflow_name + '/'
                                                    + aircraft_project
                                                    + '/retrofitDesign/engineCalibration/engine_data/')
        source = path_of_working_directory_rce + '/' + current_workflow_name + '/' + aircraft_project \
            + '/retrofitDesign/engineCalibration/engine_data/'
        folder_name = 'engineCalibration'

    # Use engine from aero calibrated design folder
    if use_engine_from_existing_project == 3:
        check_flag_engine_in_folder = os.path.isdir(path_of_working_directory_rce + '/' + current_workflow_name + '/'
                                                    + aircraft_project
                                                    + '/retrofitDesign/aerodynamicCalibration/engine_data/')
        source = path_of_working_directory_rce + '/' + current_workflow_name + '/' + aircraft_project \
            + '/retrofitDesign/aerodynamicCalibration/engine_data/'
        folder_name = 'aerodynamicCalibration'

    # use engine from repository
    if use_engine_from_existing_project == 4:
        aircraft_project = aircraft_exchange_file[0:len(aircraft_exchange_file) - 4]
        # try to open module configuration file
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/unicado_workflow_conf.xml'
        root_of_workflow_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'unicado_workflow',
                                                        function_name, frame_info.lineno, log_file_list,
                                                        current_workflow_name)
        name_of_checkout_dir_from_repository = str(root_of_workflow_tree.find(
            './control_settings/program_specific_settings/iteration_settings/use_engine_from_existing_project/'
            'mode_specific_settings/mode_4/name_of_engine_checkout_dir/value').text)
        # Check if the given input directory not equal to 'cleanSheetDesign' -> if true: -> retrofit design will be used
        if name_of_checkout_dir_from_repository == 'aerodynamicCalibration' \
                or name_of_checkout_dir_from_repository == 'engineCalibration' \
                or name_of_checkout_dir_from_repository == 'withoutCalibration':
            source = path_to_aircraft_projects + aircraft_type + '/' + aircraft_project + '/retrofitDesign/' \
                     + name_of_checkout_dir_from_repository + '/engine_data'
            check_flag_engine_in_folder = os.path.isdir(source)
        else:
            source = path_to_aircraft_projects + aircraft_type + '/' + aircraft_project + '/' \
                     + name_of_checkout_dir_from_repository + '/engine_data'
            check_flag_engine_in_folder = os.path.isdir(source)
        folder_name = 'local repository'

    if check_flag_engine_in_folder:
        destination = path_of_working_directory_rce + '/' + current_workflow_name + '/' + aircraft_project \
            + '/engine_data/'
        shutil.copytree(source, destination)
        engine_flag = True

    # use engine from ssh server branch
    if use_engine_from_existing_project == 5:
        # try to open module configuration file
        frame_info = getframeinfo(currentframe())
        path = path_of_working_directory_rce + current_workflow_name + '/unicado_workflow_conf.xml'
        root_of_workflow_tree, xml_tree = read_xml_file(path, path_of_working_directory_rce, 'unicado_workflow',
                                                        function_name, frame_info.lineno, log_file_list,
                                                        current_workflow_name)

        # set server information for sparse-checkout
        path_to_local_repository = path_of_working_directory_rce + '/' + current_workflow_name + '/' \
            + aircraft_project + '/'

        repository_name = str(root_of_workflow_tree.find(
            './control_settings/program_specific_settings/iteration_settings/use_engine_from_existing_project/'
            'mode_specific_settings/mode_5/engine_repository_name/value').text)
        branch = str(root_of_workflow_tree.find(
            './control_settings/program_specific_settings/iteration_settings/use_engine_from_existing_project/'
            'mode_specific_settings/mode_5/engine_repository_branch/value').text)
        name_of_checkout_dir_from_server = str(root_of_workflow_tree.find(
            './control_settings/program_specific_settings/iteration_settings/use_engine_from_existing_project/'
            'mode_specific_settings/mode_5/name_of_engine_checkout_dir/value').text)
        ssh_url = 'ssh://git@unicado.ilr.rwth-aachen.de:2222/source/' + repository_name + '.git'

        if repository_name.lower() == 'raircraftreferences':
            if name_of_checkout_dir_from_server == 'cleanSheetDesign':
                name_of_checkout_dir_from_server = aircraft_type + '/' + aircraft_project + '/cleanSheetDesign/engine_data'
            elif name_of_checkout_dir_from_server == 'aerodynamicCalibration' \
                    or name_of_checkout_dir_from_server == 'engineCalibration' \
                    or name_of_checkout_dir_from_server == 'withoutCalibration':
                name_of_checkout_dir_from_server = aircraft_type + '/' + aircraft_project + '/retrofitDesign/' \
                                                + name_of_checkout_dir_from_server + '/engine_data'
            else:
                path_to_local_repository += 'engine_data/'

        # call function for sparse-checkout from given ssh-git repository
        sparse_checkout_flag = sparse_checkout_from_git(path_to_local_repository, ssh_url, branch,
                                                        name_of_checkout_dir_from_server)

        folder_name = 'ssh-server'
        if not sparse_checkout_flag:
            print('Error: Engine data could not checked out from ssh-server! \n '
                  'Be sure that you have necessary access rights to checkout from repository! \n '
                  'If you have, please check if the folder to checkout from git repository exist! \n '
                  'Current engine data will be used!')
            log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) +
                                 ': Error: Engine data could not checked out from ssh-server! \n '
                                 'Be sure that you have necessary access rights to checkout from repository! \n '
                                 'If you have, please check if the folder to checkout from git repository exist! \n '
                                 'Current engine data will be used!')

        else:
            engine_flag = True

    if engine_flag:
        print('Engine data has been successfully replaced by engine from folder: ' + folder_name + '!')
        log_file_list.append(str(datetime.now().strftime('%Y-%m-%d %H:%M:%S')) + ': '
                             'Engine data has been successfully replaced by engine from folder: ' + folder_name + '!')

    return engine_flag, sparse_checkout_flag, folder_name, log_file_list
